package de.desy.acop.print.logbook;

import java.awt.print.PrinterJob;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import javax.print.PrintService;

class LogbookFinder {

    private static final Map<Maschine,LogbookFinder> map = new ConcurrentHashMap<>();
    static {
        sInitialize();
    }

    private static LogbookFinder findLogbook(final Maschine maschine, final PrintService[] services) {
        if (!maschine.logBuchName.isEmpty()) for (final PrintService service : services) {
            final String name = service.getName().toLowerCase();
            if (name.endsWith(("log")) && name.contains(maschine.logBuchName.toLowerCase()))
                return new LogbookFinder(maschine,service);
        }
        return new LogbookFinder(maschine,null);
    }

    public static void sInitialize() {
        LogbookFinder.map.clear();
        final PrintService[] services = PrinterJob.lookupPrintServices();
        for (final Maschine maschine : Maschine.values()) {
            LogbookFinder.map.put(maschine,findLogbook(maschine,services));
        }
    }

    public static LogbookFinder getLogbook(final Maschine maschine) {
        return LogbookFinder.map.get(maschine);
    }

    public static void main(final String... strings) {
        for (final Maschine maschine : map.keySet()) {
            System.out.println(maschine.toString() + "\t" + map.get(maschine).getService());
        }
    }

    //=============================================================================
    private final Maschine maschine;
    private final PrintService service;

    public LogbookFinder(final Maschine maschine, final PrintService service) {
        this.maschine = maschine;
        this.service = service;
    }

    public final Maschine getMaschine() {
        return this.maschine;
    }

    public final PrintService getService() {
        return this.service;
    }

    public final boolean isEnabled() {
        return this.service != null && !this.service.getName().isEmpty();
    }
}
