/*
 *
 * File: ADataRunnable.java
 * Date: Nov 14, 2007; 2:40:40 PM
 */
package de.desy.mst.libs.accelerator;

import java.util.concurrent.atomic.AtomicBoolean;

/**
 * This is the abstract class of the runnable interface. It declares all 
 * relevant methods and leaves the {@code run} to be declared by the special 
 * project.
 * As the only argument the constructor takes a data enumeration object and 
 * stores its information in a protected field.  
 * 
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: ADataRunnable.java 14446 2008-08-04 13:54:49Z walla $
 */
public abstract class ADataRunnable implements IRunnable
{
	/** Is the active field of this thread. It is to be true if this thread
	 * is active (i.e. it is running throw the {@code run} method otherwise
	 * it is false. */
	protected final AtomicBoolean _isActive;

	/** This is the data object component. */
	protected final IEData _data;
	
	/**
	 * Initialize the runnable object.
	 * 
	 * @param data_ Is the data object component.
	 */
	protected ADataRunnable(final IEData data_)
	{
		validate(data_);
		
		_isActive = new AtomicBoolean(false);
		_data = data_;
	}
	
	/**
	 * Validate all arguments of the constructor.
	 * 
	 * @param data_ Is the data object.
	 */
	private void validate(final IEData data_)
	{
		if (data_ == null)
			throw new NullPointerException("The data compound object is null.");
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.IRunnable#isActive()
	 */
	public boolean isActive()
	{
		return _isActive.get();
	}

}
/* end of file */