/*
 *
 * File: ApplicationVersion.java
 * Date: Oct 25, 2007; 11:14:50 AM
 */
package de.desy.mst.libs.accelerator;

import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;

import de.desy.mst.libs.accelerator.view.RFDialog;

/**
 * <p>This is the annotation for constructing the application name, the author
 * in charge, its e-mail address and the name of the application. Moreover
 * there can also be stored the version number and the release type.</p>
 *  
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: ApplicationVersion.java 30698 2009-11-19 17:20:11Z walla $
 */
@Retention(RetentionPolicy.RUNTIME)
public @interface ApplicationVersion
{
	/** Returns the developer name, normally with its first and surname, 
	 * i.e. "Marcus Walla".*/
	String developer();
	
	/** Returns the author name, normally this is her or his user name,
	 * i.e. "walla". */
	String author();
	
	/** Returns the e-mail address of the developer, i.e. 
	 * "marcus.walla(at)desy.de". */
	String email();
	
	/** Returns the application name, i.e. "PIA RF Server Application". */
	String application();
	
	/** Returns the mailing list e-mail address. As a default this returns 
	 * "developer@mcs.desy.de". */
	String mailinglist() default "developer@mcs.desy.de";
	
	/** Returns the building number, i.e. "30". */
	String building();
	
	/** Returns the room number, i.e. "517". */
	String room();
	
	/** Returns the group, i.e. "MCS1" */
	String group();
	
	/** Returns the phone number, i.e. "2628". */
	String phone();
	
	/** Returns the major version number. As a default this returns number 1. */
	int major() default 1;
	
	/** Returns the minor version number. As a default this returns number 0.*/
	int minor() default 0;
	
	/** Returns the build version number. As a default this returns number 0.*/
	int build() default 0;

	/** Returns the release date. */
	String releaseDate() default RFDialog.IVersion.DEFAULT_RELEASE_DATE;
	
	/** Returns the release type. This is an enumeration which can have 
	 * different values: ALPHA, BETA and PRODUCTION. By default this returns
	 * the ALPHA release type.*/
	EReleaseType release() default EReleaseType.ALPHA;
}
/* end of file */