/*
 *
 * File: CGeneralConstant.java
 * Date: Nov 14, 2007; 10:07:54 AM
 */
package de.desy.mst.libs.accelerator;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.logging.Level;

import javax.swing.Icon;
import javax.swing.ImageIcon;

import de.desy.mst.libs.accelerator.exception.EnumerationNotUsedException;
import de.desy.mst.libs.framework.mstapp.logging.Logger;

/**
 * <p>Different constant values are stored in this static class. This is
 * independent from all other classes in different packages and is therefore
 * replaced. This class is intended to be <b>abstract</b>, because the special
 * constant class will extends this one and overwrite the protected with a
 * private constructor to initialize this one.</p>
 * <p>This general and abstract class performs one method, which tells the
 * special library or project if the write action is performed or not.</p>
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: CGeneralConstant.java 71378 2016-05-23 11:28:07Z walla $
 */
@ApplicationVersion(author = "walla", developer = "Marcus Walla"
	, email = "marcus.walla@desy.de"
	, building = "30", room = "517", group = "MCS1"
	, phone = "2628", application = "Allgemeine HF Bibliothek (LibMST)"
	, major = 1, minor = 16, build = 10, releaseDate = "23.05.2016")
public abstract class CGeneralConstant {
	public final static String CLAZZ = CGeneralConstant.class.getSimpleName();
	
	/** The version number of this release. */
	public final static String VERSION;
	/** The name of this application. */
	public final static String APPLICATION_NAME;
	/** Developer(s) full name. */
	public final static String DEVELOPER;
	/** The author(s) user name. */
	public final static String AUTHOR;
	/** Phone number(s). */
	public final static String PHONE;
	/** Group. */
	public final static String GROUP;
	/** Building/ */
	public final static String BUILDING;
	/** Room. */
	public final static String ROOM;
	/** E-Mail address of the author (only one is specified).*/
	public final static String EMAIL;
	/** Mailing list. */
	public final static String MAILINGLIST;
	/** Release date. */
	public final static String RELEASE_DATE;
	/** TINE application name. */
	public final static String TINE_APPLICATION_NAME = "TINE (Kommunikationsprotokoll)";

	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	/**
	 * <p>Time slice after initializing the application and setting the frame title 
	 * to the appropriated one. This title includes beside the caption also the 
	 * version number.</p>
	 * <p>The CAM (Console Application Manager) will verify an ID from the application
	 * to identify it, where currently the title is taken. To be sure that the
	 * title is always the same, I set the real title only after a time slice.</p>
	 */
	public final static int TIME_SLICE_SET_GUI_FRAMETITLE = 5000;
	
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	
	/** An empty icon with the dimension of 32x32. */
	public static Icon ICON_EMPTY_32x32;
	/** An empty icon with the dimension of 16x16. */
	public static Icon ICON_EMPTY_16x16;
	
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	/** A big green icon. */
	public static Icon ICON_GREEN_CIRCLE_32x32;
	/** A normal green icon. */
	public static Icon ICON_GREEN_CIRCLE_16x16;

	/** A big icon shown when an error has occurred. */
	public static Icon ICON_RED_CIRCLE_32x32;
	/** A normal icon shown when an error has occurred. */
	public static Icon ICON_RED_CIRCLE_16x16;

	/** A big icon shown when an error or warning has occurred. */
	public static Icon ICON_YELLOW_CIRCLE_32x32;
	/** The yellow icon shown when an error or warning has occurred. */
	public static Icon ICON_YELLOW_CIRCLE_16x16;
	
	/** A big icon shown when an error or warning has occurred. */
	public static Icon ICON_MAGENTA_CIRCLE_16x16;
	/** The yellow icon shown when an error or warning has occurred. */
	public static Icon ICON_MAGENTA_CIRCLE_32x32;
	
	/** An animated big icon shown when an error has occurred. */
	public static Icon ICON_ANIMATE_ERRORMESSAGE_32x32;
	/** An animated normal icon shown when an error has occurred. */
	public static Icon ICON_ANIMATE_ERRORMESSAGE_16x16;

	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	/** Question icon shown in the question dialog box. */
	public static Icon ICON_MESSAGE_QUESTION_32x32;
	/** Information icon shown in the information dialog box. */
	public static Icon ICON_MESSAGE_INFORMATION_32x32;
	/** Warning icon shown in the warning dialog box. */
	public static Icon ICON_MESSAGE_WARNING_32x32;
	/** Error icon shown in the error dialog box. */
	public static Icon ICON_MESSAGE_ERROR_32x32;
	/** Fatal error icon shown in the fatal error dialog box. */
	public static Icon ICON_MESSAGE_FATAL_32x32;
	
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	static
	{
		ApplicationVersion version =
			CGeneralConstant.class.getAnnotation(ApplicationVersion.class);
		
		if (version != null)
		{
			VERSION = "" + version.major() + '.' + version.minor() + '.' + version.build();
			APPLICATION_NAME = version.application();
			DEVELOPER = version.developer();
			AUTHOR = version.author();
			PHONE = version.phone();
			GROUP = version.group();
			BUILDING = version.building();
			ROOM = version.room();
			EMAIL = version.email();
			MAILINGLIST = version.mailinglist();
			RELEASE_DATE = version.releaseDate();		
		}
		else
		{
			VERSION = "UNKNOWN";
			APPLICATION_NAME = "UNKNOWN";
			DEVELOPER = "UNKNOWN";
			AUTHOR = "UNKNOWN";
			PHONE = "UNKNOWN";
			GROUP = "UNKNOWN";
			BUILDING = "UNKNOWN";
			ROOM = "UNKNOWN";
			EMAIL = "UNKNOWN";
			MAILINGLIST = "UNKNOWN";
			RELEASE_DATE = "UNKNOWN";
		}
		
		try
		{
			// read the empty icons  - - - - - - - - - - - - - - - - - - - - - -
			ICON_EMPTY_32x32 = 					new ImageIcon(CGeneralConstant.class.getResource("/resources/icons/ICON_EMPTY_32x32.PNG"));
			ICON_EMPTY_16x16 = 					new ImageIcon(CGeneralConstant.class.getResource("/resources/icons/ICON_EMPTY_16x16.PNG"));
			
			// read the static icons used for successful messages  - - - - - - -
			ICON_GREEN_CIRCLE_32x32 = 			new ImageIcon(CGeneralConstant.class.getResource("/resources/icons/ICON_GREEN_CIRCLE_32x32.PNG"));
			ICON_GREEN_CIRCLE_16x16 = 			new ImageIcon(CGeneralConstant.class.getResource("/resources/icons/ICON_GREEN_CIRCLE_16x16.PNG"));
			
			// read the static icons used for error messages - - - - - - - - - -
			ICON_RED_CIRCLE_32x32 = 			new ImageIcon(CGeneralConstant.class.getResource("/resources/icons/ICON_RED_CIRCLE_32x32.PNG"));
			ICON_RED_CIRCLE_16x16 = 			new ImageIcon(CGeneralConstant.class.getResource("/resources/icons/ICON_RED_CIRCLE_16x16.PNG"));
			
			// read the static icons used for warning messages - - - -
			ICON_YELLOW_CIRCLE_16x16 = 			new ImageIcon(CGeneralConstant.class.getResource("/resources/icons/ICON_YELLOW_CIRCLE_16x16.PNG"));
			ICON_YELLOW_CIRCLE_32x32 = 			new ImageIcon(CGeneralConstant.class.getResource("/resources/icons/ICON_YELLOW_CIRCLE_32x32.PNG"));
			
			// read the static icons used for warning messages - - - -
			ICON_MAGENTA_CIRCLE_16x16 = 		new ImageIcon(CGeneralConstant.class.getResource("/resources/icons/ICON_MAGENTA_CIRCLE_16x16.PNG"));
			ICON_MAGENTA_CIRCLE_32x32 = 		new ImageIcon(CGeneralConstant.class.getResource("/resources/icons/ICON_MAGENTA_CIRCLE_32x32.PNG"));
			
			// read the animated error message icons - - - - - - - - - - - - - -
			ICON_ANIMATE_ERRORMESSAGE_32x32 = 	new ImageIcon(CGeneralConstant.class.getResource("/resources/icons/ICON_ANIMATE_ERRORMESSAGE_32x32.GIF"));
			ICON_ANIMATE_ERRORMESSAGE_16x16 = 	new ImageIcon(CGeneralConstant.class.getResource("/resources/icons/ICON_ANIMATE_ERRORMESSAGE_16x16.GIF"));

			// read the dialog box icons - - - - - - - - - - - - - - - - - - - -
			
			ICON_MESSAGE_QUESTION_32x32 = 		new ImageIcon(CGeneralConstant.class.getResource("/resources/icons/ICON_WINDOWS_XP_QUESTION_32x32.PNG"));
			ICON_MESSAGE_INFORMATION_32x32 = 	new ImageIcon(CGeneralConstant.class.getResource("/resources/icons/ICON_WINDOWS_XP_INFO_32x32.PNG"));
			ICON_MESSAGE_WARNING_32x32 = 		new ImageIcon(CGeneralConstant.class.getResource("/resources/icons/ICON_WINDOWS_XP_WARNING_32x32.PNG"));
			ICON_MESSAGE_ERROR_32x32 = 			new ImageIcon(CGeneralConstant.class.getResource("/resources/icons/ICON_WINDOWS_XP_ERROR_32x32.PNG"));
			ICON_MESSAGE_FATAL_32x32 = 			new ImageIcon(CGeneralConstant.class.getResource("/resources/icons/ICON_WINDOWS_XP_FATAL_32x32.PNG"));

			// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
			
			Logger.SWApp.debug(CLAZZ + " - successful reading all icons");
		}
		catch(Throwable throwable)
		{
			Logger.SWApp.error(CLAZZ + " - a throwable has been thrown. Maybe the static" +
				" initializer of this class cannot open the path of the" +
				" icons?!");
			Utility.fatalErrorMessage("Fehler bei Initialisierung der Icons",
					"Bei der Initialisierung der Icons ist ein Fehler aufgetreten.<br>" +
					"Offensichtlich kann der Pfad nicht gefunden werden!", throwable);
			System.exit(-1);
		}

	}

	// File status -------------------------------------------------------------

	/** Constant file status field: Success */
	public final static int STATUS_FILE_SUCCESS               = 0x0000;

	/** Constant file status field: Runtime exception. */
	public final static int STATUS_FILE_RUNTIME_EXCEPTION     = 0x1000;

	/** Constant file status field: IO exception. */
	public final static int STATUS_FILE_IO_EXCEPTION          = 0x2000;

	/** Constant file status field: Unmarshal exception. */
	public final static int STATUS_FILE_UNMARSHAL_EXCEPTION   = 0x2001;

	/** Constant file status field: Marshal exception. */
	public final static int STATUS_FILE_MARSHAL_EXCEPTION     = 0x2002;

	/** Constant file status field: JAXB exception. */
	public final static int STATUS_FILE_JAXB_EXCEPTION        = 0x2003;

	/** Constant file status field: TINE exception. */
	public final static int STATUS_FILE_TINE_EXCEPTION        = 0x2004;

	/** Constant file status field: Not Operating, i.e. the method doesn't exist
	 * currently. */
	public final static int STATUS_FILE_NOT_OPERATING_WARNING = 0x8000;

	/** Constant file status field: Load Default, i.e. the current file cannot
	 * be loaded an there for the internal default values are set. */
	public final static int STATUS_FILE_LOAD_DEFAULT_WARNING  = 0x8001;

	/** Constant file status field: Initialize warning, i.e. you should at first
	 * initialize the class before calling any other method. */
	public final static int STATUS_FILE_INITIALIZE_WARNING    = 0x8002;

	public final static Map<Integer, String> MAP_STATUS_FILE =
		new HashMap<Integer, String>();

	static
	{
		MAP_STATUS_FILE.put(STATUS_FILE_SUCCESS, "success");

		MAP_STATUS_FILE.put(STATUS_FILE_RUNTIME_EXCEPTION, "Runtime exception");
		MAP_STATUS_FILE.put(STATUS_FILE_IO_EXCEPTION, "IO exception");
		MAP_STATUS_FILE.put(STATUS_FILE_UNMARSHAL_EXCEPTION, "JAXB unmarshal exception");
		MAP_STATUS_FILE.put(STATUS_FILE_MARSHAL_EXCEPTION, "JAXB marshal exception");
		MAP_STATUS_FILE.put(STATUS_FILE_JAXB_EXCEPTION, "JAXB exception");
		MAP_STATUS_FILE.put(STATUS_FILE_TINE_EXCEPTION, "TINE exception");

		MAP_STATUS_FILE.put(STATUS_FILE_NOT_OPERATING_WARNING, "Not operating warning");
		MAP_STATUS_FILE.put(STATUS_FILE_LOAD_DEFAULT_WARNING, "Load default warning");
		MAP_STATUS_FILE.put(STATUS_FILE_INITIALIZE_WARNING, "Initialize warning");
	}

	// -------------------------------------------------------------------------

	/** The start parameter of the error number of the enumeration
	 * {@link de.desy.mst.libs.accelerator.model.EErrorExt}. */
	public final static int ERROR_BASE = 0x800;

	// -------------------------------------------------------------------------

	/** This sets the write action of the CDI server. It is armed if it is
	 * set to {@code true}; otherwise it is set to {@code false}. */
	private static boolean CDI_WRITE_ARMED;

	/** This sets the write action of the TINE device server. It is armed if it
	 * is set to {@code true}; otherwise it is set to {@code false}. This server
	 * is normally written by the developer who also created the special common
	 * library. */
	private static boolean TINE_WRITE_ARMED;

	/** This sets the write action of the EVENT ARCHIVE server. It is armed if
	 * it is set to {@code true}; otherwise it is set to {@code false}. */
	private static boolean EVENT_WRITE_ARMED;

	// -------------------------------------------------------------------------

	/** This is the graphic debugging value. The value of {@code true} means
	 * that graphically a red border is drawn around the component. Otherwise,
	 * if the value is {@code false} nothing will be drawn.
	 */
	private static boolean DEBUG_GRAPHIC = false;

	/** Is for debugging normal statements. */
	private static boolean DEBUG = false;

	// -------------------------------------------------------------------------

	/** This tells the class of ACDIDevices how many times it should try to read
	 * the information before it will throw an error message and abort the
	 * program execution. */
	private static int LOOP_CDI_ARGUMENTS = 3;

	// -------------------------------------------------------------------------

	/** Is a set of TINE error codes which will be ignored. */
	private static Set<Integer> TINE_IGNORE_ERROR;

	// -------------------------------------------------------------------------

	/** The length of the TINE device name. */
	public static final int TINE_LENGHT_OF_DEVICENAME = 32;

	// -------------------------------------------------------------------------

	/** This is the protected instance of this class, which is used like a
	 * singleton. The real constructor is called in the special project which
	 * initialize this one and sets the static fields of write action. */
	protected static CGeneralConstant _thisConstant;

	// -------------------------------------------------------------------------

	/** This is the constant final string field of the conversion factor of the
	 * receive float value of the CDI table under the column of DESCRIPTION. */
	public final static String DESCRIPTION_RECV = "recv";

	/** This is the constant final string field of the conversion factor of the
	 * receive float value of the CDI table under the column of DESCRIPTION. */
	public final static String DESCRIPTION_SEND = "send";

	/** This is the constant final string field of the conversion factor of the
	 * receive float value of the CDI table under the column of DESCRIPTION. */
	public final static String DESCRIPTION_CHANNEL = "channel";

	// -------------------------------------------------------------------------

	// Error messages
	public static final String ERROR_TINE = 
		"Ein Fehler ist beim starten der Desy2 HF Applikation<br>" +
		"aufgetreten. Es wurde eine Fehlermeldung des<br>" +
		"Kommunikationsprotokolls geworfen, die den folgenden<br>" +
		"Inhalt besitzt:<br><br>";
	public static final String ERROR_TINE_TITLE = "Fehler im" +
			" Kommunikationsprotokoll";

	public static final String ERROR_RUNTIME = 
		"Ein Laufzeitfehler ist beim starten der Desy2 HF<br>" +
		"Applikation aufgetreten. Die folgende Fehlermeldung<br>" +
		"wurde uebergeben:<br><br>";
	public static final String ERROR_RUNTIME_TITLE = "Laufzeitfehler";

	// -------------------------------------------------------------------------

	/** This is the waiting time between the sending of the current slider
	 * value. */
	public static final int TINE_WAIT_CAVITY_SWITCH = 10000;

	// -------------------------------------------------------------------------

	/** <p>The string for preliminary TINE address must match the following string
	 * value if it is a CDI call. The value of \\x2e is a dot (".") and \\x2d is
	 * a minus ("-").</p>
	 * <p>More information can be found at
	 * <a href="http://java.sun.com/javase/6/docs/api/java/util/Formatter.html#syntax">here</a>.
	 * The ASCII code table can be found <a href="http://www.torsten-horn.de/techdocs/ascii.htm">here</a>.</p> */
	public static final String TINE_CDI_MATCH =
		"^/\\w+/[\\w\\x2d\\x2e]+\\x2eCDI$";

	/** <p>Verifies if the string values is a valid e-mail address. The value of
	 * \\x2e is a dot (".") and \\x40 is the at-sign ("@").</p>
	 * <p>More information can be found at
	 * <a href="http://java.sun.com/javase/6/docs/api/java/util/Formatter.html#syntax">here</a>.
	 * The ASCII code table can be found <a href="http://www.torsten-horn.de/techdocs/ascii.htm">here</a>.</p> */
	public static final String VALID_EMAIL =
		"^[\\p{Alnum}]+[\\x2e?\\p{Alnum}+]*\\x40" +
		"[\\p{Alnum}]+[\\x2e[\\p{Alnum}]+]*[\\x2e][dD][eE]$";

	/** <p>Pattern for controlling the cavity numeric precision number or a 
	 * character value. The number precision is of the following format: 
	 * <tt>^%(+|&lt;space&gt;)?\d?\d.\d)?[fg])$</tt>. The character string is of the
	 * following format: <tt>^%\d?s$</tt>.</p>
	 * <p>More information can be found at
	 * <a href="http://java.sun.com/javase/6/docs/api/java/util/Formatter.html#syntax">here</a>.
	 * The ASCII code table can be found <a href="http://www.torsten-horn.de/techdocs/ascii.htm">here</a>.</p> */
	public static final String PATTERN_CAVITY_PRECISION =
		"^%((((\\x2B|\\x20)?\\d?\\d.\\d)?[fg])|(\\d?s))$";

	/**
	 * <p>Normal number precision of floating point fields. The number is formated
	 * by the following format: <tt>%^((+|&lt;space&gt;)?\d?\d.\d)?[fg]$</tt>.</p>
	 * <p>More information can be found on 
	 * <a href="http://java.sun.com/javase/6/docs/api/java/util/Formatter.html#syntax">here</a>.
	 * The ASCII code table can be found <a href="http://www.torsten-horn.de/techdocs/ascii.htm">here</a>.</p> */
	public static final String PATTERN_NUMBER_PRECISION =
		"^%((\\x2B|\\x20)?\\d?\\d.\\d)?[fg]$";

	/** <p>Pattern for controlling the error key. The formated value is:
	 * <tt>\d?\d\d.\d\d(.\d)?$</tt>. This is <b>NOT</b> a precision value!
	 * x2E='.'</p>
	 * <p>More information can be found at
	 * <a href="http://java.sun.com/javase/6/docs/api/java/util/Formatter.html#syntax">here</a>.
	 * The ASCII code table can be found <a href="http://www.torsten-horn.de/techdocs/ascii.htm">here</a>.</p> */
	public static final String PATTERN_ERROR_KEY =
		"^\\d?\\d\\d\\x2E\\d\\d(\\x2E\\d)?$";

	/** Pattern for controlling the error value.
	 * x20=' ', x21='!', x23='#', x23='#', x24='$', x28='(', x29=')', x2A='*',
	 * x2B='+', x2C=',', x2D='-', x2E='.', x2F='/', x3A=':', x3B=';', x3C='<',
	 * x3D='=', x3E='>', x3F='?' 
	 * <p>More information can be found on 
	 * <a href="http://java.sun.com/javase/6/docs/api/java/util/Formatter.html#syntax">here</a>.
	 * The ASCII code table can be found <a href="http://www.torsten-horn.de/techdocs/ascii.htm">here</a>.</p> */
	public static final String PATTERN_ERROR_VALUE =
		"^(\\d?\\d\\x2C)?[\\w\\x20\\x21\\x23\\x24\\x28\\x29\\x2A\\x2B" +
		"\\\\x2C\\x2D\\x2E\\x2F\\x3A\\x3B\\x3C\\x3D\\x3E\\x3F]+$";

	/** <p>Pattern for controlling the device group of a TINE link.</p>
	 * <p>More information can be found on 
	 * <a href="http://java.sun.com/javase/6/docs/api/java/util/Formatter.html#syntax">here</a>.
	 * The ASCII code table can be found <a href="http://www.torsten-horn.de/techdocs/ascii.htm">here</a>.</p> */
//	public static final String PATTERN_DEVICEGROUP_LINK1 =
//	"^/\\w+/[\\w.]+$";
	public static final String PATTERN_DEVICEGROUP_LINK1 =
		"^/\\w+/[\\w\\x2E]+(\\x2DPIA)?(\\x2DT)?\\x2ECDI$";

	/** <p>Pattern for controlling the device group of a TINE link.<p>
	 * <p>More information can be found on 
	 * <a href="http://java.sun.com/javase/6/docs/api/java/util/Formatter.html#syntax">here</a>.
	 * The ASCII code table can be found <a href="http://www.torsten-horn.de/techdocs/ascii.htm">here</a>.</p> */
//	public static final String PATTERN_DEVICEGROUP_LINK2 =
//	"^/\\w+/[\\w.]+/\\w+$";
	public static final String PATTERN_DEVICEGROUP_LINK2 =
		"^/\\w+/[\\w\\x2e\\x2D]+/(#0|\\w+)$";

	/** <p>Pattern for controlling the control precision.</p>
	 * <p>More information can be found on 
	 * <a href="http://java.sun.com/javase/6/docs/api/java/util/Formatter.html#syntax">here</a>.</p> */
	public static final String PATTERN_CONTROL_PRECISION =
		"^\\d\\.\\d$";

	/** <p>Pattern for controlling the switch precision.</p>
	 * <p>More information can be found on 
	 * <a href="http://java.sun.com/javase/6/docs/api/java/util/Formatter.html#syntax">here</a>.</p> */
	public final static String PATTERN_SWITCH_PRECISION =
		"^\\d\\.\\d$";

	/** <p>The normal data element precision read from the CDI table.</p>
	 * <p>More information can be found on 
	 * <a href="http://java.sun.com/javase/6/docs/api/java/util/Formatter.html#syntax">here</a>.</p> */
	public final static String PATTERN_DATA_PRECISION =
		"^\\d\\.\\d$";

	/** <p>Pattern. Every string element which matches this pattern will be stored
	 * in a set. These set is afterwards take to control the TINE error code if
	 * it fits these set. If yes it will not report the error on the console or
	 * log file.</p>
	 * <p>More information can be found on 
	 * <a href="http://java.sun.com/javase/6/docs/api/java/util/Formatter.html#syntax">here</a>.</p> */
	public final static String PATTERN_IGNORE_ERROR =
		"^\\d?\\d?\\d?\\d$";

	// -------------------------------------------------------------------------

	/** Sets the java current logging level to be INFO. I need this logging
	 * level for class from the package
	 * {@code de.desy.mst.libs.accelerator.logging}. */
	public static final Level LOG_LEVEL = Level.INFO;

	// -------------------------------------------------------------------------

	/** Enables the displaying of the debug ticker time by setting its argument 
	 * to {@code true} (i.e. <tt>AppArguments.setNumber(CLA_ENABLE_DEBUG_TICKER, new DBoolean(true))</tt>) 
	 * or disables it by setting its argument to {@code false} (i.e.
	 * <tt>AppArguments.setNumber(CLA_ENABLE_DEBUG_TICKER, new DBoolean(false))</tt>). */
	public static final String CLA_ENABLE_DEBUG_TICKER = "ENABLE_DEBUG_TICKER";
	
	/** The command line argument of MODE. For example, 
	 * <tt>AppArguments.setString(CLA_MODE, "Expert")</tt> changes to the expert mode.
	 * Possible values are <tt>Betrachten</tt>, <tt>Betrieb</tt>, or <tt>Expert</tt>.*/
	public static final String CLA_MODE = "MODE";

	/** The command line argument of TAB. For example,
	 * <tt>AppArguments.setNumber(CLA_TAB, new DInteger(1))</tt> sets the tabbed
	 * pane to 1. */
	public static final String CLA_TAB = "TAB";

	/** The command line argument of ARMED. For example,
	 * <tt>AppArguments.setNumber(CLA_ARMED1, new DBoolean(true))</tt> enables
	 * the displaying GUI component of the TINE write/read access. */
	public static final String CLA_ARMED1 = "ARMED";

	/** The command line argument of ARMED. For example,
	 * <tt>AppArguments.setNumber(CLA_ARMED2, new DBoolean(true))</tt> enables
	 * the displaying GUI component of the TINE write/read access. */
	public static final String CLA_ARMED2 = "ARMED";

	/** The command line argument of LOG. For example,
	 * <tt>AppArguments.setString(CLA_LOG, "WARN")</tt> sets the 
	 * debugging output mode to warning. Allowed are <tt>DEBUG</tt>, <tt>INFO</tt>,
	 * <tt>WARN</tt>, <tt>ERROR</tt>, and <tt>FATAL</tt>. */
	public static final String CLA_LOG = "LOG";

	/** Created new command line argument which tells the application the delay
	 * time before starting the application if a CDI Server error occurs. For
	 * example, <tt>AppArguments.setNumber(CLA_INITDELAY, 15)</tt> sets the
	 * delay time to 15 seconds. */
	public static final String CLA_INITDELAY = "INITDELAYERROR";

	// -------------------------------------------------------------------------

	/** The debug logging level. */
	public static final String DEBUG_DEBUG = "DEBUG";

	/** The info logging level. */
	public static final String DEBUG_INFO = "INFO";

	/** The warn logging level. */
	public static final String DEBUG_WARN = "WARN";

	/** The error logging level. */
	public static final String DEBUG_ERROR = "ERROR";

	/** The fatal logging level. */
	public static final String DEBUG_FATAL = "FATAL";

	// -------------------------------------------------------------------------

	/** Is the super user password for all applications. */
	public static final String APPLICATION_PASSWORD = "007";

	// -------------------------------------------------------------------------

	/** The following constant is usually used to print out an error message. */
	public static final String ERROR = "ERR: ";

	/** The following constant is usually only used for reporting error messages
	 * as a tool tip. */
	public static final String ERROR_TOOLTIP = "Fehler: ";

	/** For undefined values. */
	public static final String UNDEFINED = "UNDEF";

	/** For undefined values, tool tip text */
	public static final String UNDEFINED_TOOLTIP = "UNDEFINIERT";

	/** Near operate state is active. */
	public static final String NEAR_OPERATE_ACTIVE =
		"Der Sender steht auf NAH und kann zurzeit nicht bedient werden.";

	// -------------------------------------------------------------------------

	/** The delay time for showing tool tips on screen in milliseconds. */
	public static final int TOOL_TIP_DISPLAYED_IN_MS = 120000;

	// -------------------------------------------------------------------------

	/**
	 * This protected constructor sets all write values as boolean if they
	 * are performed ({@code true}) or not ({@code false}).
	 *
	 * @param isCDIWriteArmed_ Is the write action on the CDI server.
	 * @param isTINEWriteArmed_ Is the write action on the TINE server.
	 * @param isEventWriteArmed_ Is the write action on the EVENT ARCVIVE
	 * 		server.
	 * @param loopCDIArguments_ Is the loop value for getting CDI arguments.
	 * 		These value can be from 1 up to 3. If the value is not between
	 * 		these values it will be set to 3.
	 * @param isDebug_ Is the normal debug value as a {@code boolean}.
	 * @param isDebugGraphic_ Draws a red border around the component if this
	 * 		value is set.
	 * @param tineIgnoreError_ Is a set of errors which the application will
	 * 		ignore on reporting it. This should never be {@code null}!
	 */
	protected CGeneralConstant(final boolean isCDIWriteArmed_,
			final boolean isTINEWriteArmed_, final boolean isEventWriteArmed_,
			final int loopCDIArguments_,
			final boolean isDebug_, final boolean isDebugGraphic_,
			final Set<Integer> tineIgnoreError_)
	{
		CDI_WRITE_ARMED = isCDIWriteArmed_;
		TINE_WRITE_ARMED = isTINEWriteArmed_;
		EVENT_WRITE_ARMED = isEventWriteArmed_;
		LOOP_CDI_ARGUMENTS =
			(loopCDIArguments_ < 1 || loopCDIArguments_ > 3)?3:loopCDIArguments_;
		DEBUG = isDebug_;
		DEBUG_GRAPHIC = isDebugGraphic_;
		if (tineIgnoreError_ == null)
			throw new NullPointerException("The value of tineIgnoreError_  is" +
					" null.");
		TINE_IGNORE_ERROR = tineIgnoreError_;

		// Now the tool tip is displayed for TOOL_TIP_DISPLAYED_IN_MS in
		// milliseconds on screen.
		//ToolTipManager.sharedInstance().setDismissDelay(TOOL_TIP_DISPLAYED_IN_MS);
	}

	/**
	 * This static method returns on the delivered enumeration of the server
	 * the corresponding write action. This is a boolean value which tells the
	 * caller if the action is performed or not.
	 *
	 * @param server_ Is the enumeration of the server. If the server doesn't
	 * 		exist a {@code EnumerationNotUsedException} is thrown.
	 * @return Returns a boolean value.
	 */
	public static final boolean isWriteArmed(final EServer server_)
	{
		switch(server_)
		{
		case CDI:
			return CDI_WRITE_ARMED;
		case TINE:
			return TINE_WRITE_ARMED;
		case EVENTARCHIVE:
			return EVENT_WRITE_ARMED;
		default:
			throw new EnumerationNotUsedException("The enumeration of "
					+ server_.toString() + " can't be found.");
		}
	}

	/**
	 * Returns the debug value of the corresponding enumeration instance
	 * as a {@code boolean} instance.
	 *
	 * @param debug_ Is the debug enumeration. This shows which things you
	 * 		wanted to debug (normal or graphical).
	 * @return The {@code boolean} debug value.
	 */
	public static final boolean isDebug(final EDebug debug_)
	{
		switch(debug_)
		{
		case DEBUG:
			return DEBUG;
		case DEBUG_GRAPHIC:
			return DEBUG_GRAPHIC;
		default:
			throw new EnumerationNotUsedException("The enumeration of "
					+ debug_.toString() + " can't be found.");
		}
	}

	/**
	 * Returns the loop value for getting CDI arguments. These CDI arguments
	 * are the precision, unit, range, etc and are requested at starting the
	 * application. It can also be set in the property file of the concrete
	 * project with the key of {@code LOOP_CDI_ARGUMENTS}.
	 *
	 * @return Returns the loop value for getting CDI arguments.
	 */
	public static final int loopCDIArguments()
	{
		return LOOP_CDI_ARGUMENTS;
	}

	/**
	 * In the property file you can also define a list of TINE error codes,
	 * each separated with a comma, which this application will ignore and never
	 * report them on console or in a log file. If you miss these entry in the
	 * property file the internal set will be empty (i.e. with no elements in
	 * it).
	 *
	 * @param errorCode_ Is the TINE error code.
	 * @return Returns a boolean {@code true} if the error code exists otherwise
	 * 		a {@code false} is returned.
	 */
	public static final boolean ignoreTINEError(final int errorCode_)
	{
		return TINE_IGNORE_ERROR.contains(errorCode_);
	}
}
/* end of file */