/*
 *
 * File: IECDIDevice.java
 * Date: Nov 29, 2007; 6:39:59 PM
 */
package de.desy.mst.libs.accelerator;

import java.util.Collection;
import java.util.Map;
import java.util.SortedMap;

/**
 * Is the enumeration interface of the CDI device (groups?).
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: IECDIDevice.java 12066 2008-05-29 08:35:57Z walla $
 */
public interface IECDIDevice
{
	
	/**
	 * Returns the corresponding map of this enumeration interface for
	 * each preliminary TINE address string. The first
	 * argument is the string value of the TINE address which returns a map
	 * with the device names and the corresponding enumeration data element.
	 * 
	 * @return Returns the map hole map for each TINE address string which
	 * 		has a map of all device names and its enumeration data interface.
	 */
	public Map<String, SortedMap<String, IEData>> getMap();
	
	/**
	 * Returns the map of the corresponding preliminary TINE address element.
	 * 
	 * @param tineAddress_ The preliminary TINE address string.
	 * @return Returns the map of this enumeration interface.
	 */
	public Map<String, IEData> getDeviceMap(final String tineAddress_);

	/**
	 * Returns the device names as a collection of string values. The length
	 * depends on the size of the map of the TINE address element.
	 * 
	 * @param tineAddress_ The preliminary TINE address string.
	 * @return Return a collection of all device names.
	 */
	public Collection<String> getDeviceNames(final String tineAddress_);
	
	/**
	 * Returns a collection of all data component enumerations. The length 
	 * depends on the size of the map.  
	 * 
	 * @param tineAddress_ The preliminary TINE address string.
	 * @return Returns a collection of all data component enumeration 
	 * 		interfaces.
	 */
	public Collection<IEData> getDataComponent(final String tineAddress_);

}
/* end of file */