/*
 *
 * File: IEDeviceElement.java
 * Date: May 25, 2007; 1:47:05 PM
 */
package de.desy.mst.libs.accelerator;

/**
 * <p>Each device can also have different element values. These are stored
 * inside this interface. It has no direct members except those from the
 * inherited interface called {@link IEEnum}.</p> 
 * <p>Here is a small example of a device which has different elements per 
 * device. So, say, you have different devices <tt>H</tt> which are enumerated 
 * with an postive natural number from <tt>1</tt> to <tt>400</tt>. Each device
 * has different elements: <tt>TEMPERATURE</tt>,<tt>TEMPERATURE_MAX</tt>,
 * <tt>TEMPERATURE_MIN</tt>,<tt>CALIBRATION_MIN</tt>, <tt>CALIBRATION_MAX</tt>, 
 * ...</p>
 * <p>The result is that each device now has different elements of the above 
 * items:</p>
 * <pre>
 * H1.TEMPERATURE
 * H1.TEMPERATURE_MIN
 * H1.TEMPERATURE_MAX
 * H1.CALIBRATION_MIN
 * H1.CALIBRATION_MAX
 * ...
 * 
 * H2.TEMPERATURE
 * H2.TEMPERATURE_MIN
 * H2.TEMPERATURE_MAX
 * H2.CALIBRATION_MIN
 * H2.CALIBRATION_MAX
 * ...
 * 
 * H400.TEMPERATURE
 * H400.TEMPERATURE_MIN
 * H400.TEMPERATURE_MAX
 * H400.CALIBRATION_MIN
 * H400.CALIBRATION_MAX
 * ...
 * </pre>
 * 
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: IEDeviceElement.java 12066 2008-05-29 08:35:57Z walla $
 */
public interface IEDeviceElement extends IEEnum
{
	/**
	 * Returns the size of the data component element.
	 * 
	 * @return The size is return as a positive integer value.
	 */
	public abstract int size();
	
	/**
	 * Returns the corresponding data component object of the device element.
	 * 
	 * @return The corresponding data component object.
	 */
	public abstract IEData template();
}
/* end of file */