/*
 *
 * File: IEEnum.java
 * Date: Nov 28, 2006; 10:15:52 AM
 */
package de.desy.mst.libs.accelerator;

import de.desy.mst.libs.accelerator.exception.EnumerationNotUsedException;

/**
 * <p>The interface for each inherited enumeration type. The abstract 
 * method of description returns a non null {@code java.lang.String} value. This
 * is the super interface for all derivated enumerations.</p>
 * <p>It has different abstract beans for getting and setting the corresponding
 * {@link EName} enumeration.</p>
 *
 * @author <a href="marcus.walla(at)desy.de">walla</a>
 * @version $Id: IEEnum.java 34945 2010-04-12 08:26:36Z walla $
 */
public interface IEEnum
{
	/**
	 * Returns according to the enumeration of the description the corresponding
	 * string value. If the enumeration is not available a {@code null} value 
	 * is returned, except the enumeration of {@code ID} which will always
	 * return a value.
	 * 
	 * @param description_ The corresponding enumeration type which you wanted
	 * 		to get. For more information see at the {@link EName} enumeration.
	 * @return The corresponding string value is returned.
	 * @throws EnumerationNotUsedException If the enumeration indeed exists but
	 * 		is not used these exception is thrown.
	 */
	public abstract String getName(final EName description_)
		throws EnumerationNotUsedException;
	
	/**
	 * Sets to the according enumeration the string value of the corresponding
	 * element. If the enumeration is not available an 
	 * <tt>EnumerationNotUsedException</tt> is thrown.
	 * 
	 * @param description_ Is the <tt>EName</tt> enumeration one wants to set.
	 * @param name_ Is the name of the string value. This can also be 
	 * 		<tt>null</tt>.
	 * @throws EnumerationNotUsedException Is thrown if the enumeration is known
	 * 		but not used.
	 */
	public abstract void putName(final EName description_, final String name_)
		throws EnumerationNotUsedException;

}
/* end of file */