/*
 *
 * File: Version.java
 * Date: Jun 2, 2008; 5:43:24 PM
 */
package de.desy.mst.libs.accelerator;

import de.desy.mst.libs.accelerator.view.RFDialog;

/**
 * Application version object.
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: Version.java 13092 2008-06-25 14:05:15Z walla $ 
 */
public final class Version implements RFDialog.IVersion
{
	/** The application version string. */
	private final String _applicationVersion;
	
	/** The application name string. */
	private final String _applicationName;
	
	/** The application release date. */
	private final String _applicationReleaseDate;
	
	// --- CONSTRUCTOR ---------------------------------------------------------
	
	/**
	 * Constructor.
	 * 
	 * @param applicationName_ Application name.
	 * @param applicationVersion_ Application version.
	 */
	public Version(
			final String applicationName_, 
			final String applicationVersion_)
	{
		validate(applicationName_, applicationVersion_, null);

		_applicationReleaseDate = DEFAULT_RELEASE_DATE;
		_applicationName = applicationName_;
		_applicationVersion = applicationVersion_;
	}
	
	/**
	 * Constructor.
	 * 
	 * @param applicationName_ Application name.
	 * @param applicationVersion_ Application version.
	 * @param applicationReleaseDate_ Application release date.
	 */
	public Version(
			final String applicationName_, 
			final String applicationVersion_,
			final String applicationReleaseDate_)
	{
		validate(applicationName_, applicationVersion_, applicationReleaseDate_);

		_applicationReleaseDate = applicationReleaseDate_;
		_applicationName = applicationName_;
		_applicationVersion = applicationVersion_;
	}

	// --- INSTANCE METHODS ----------------------------------------------------
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.RFDialog.IVersion#getApplicationName()
	 */
	public String getApplicationName()
	{
		return _applicationName;
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.RFDialog.IVersion#getApplicationVersion()
	 */
	public String getApplicationVersion()
	{
		return _applicationVersion;
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.RFDialog.IVersion#getApplicationReleaseDate()
	 */
	public String getApplicationReleaseDate()
	{
		return _applicationReleaseDate;
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString()
	{
		if (_applicationReleaseDate.equals(DEFAULT_RELEASE_DATE))
			return _applicationName + " - Version " + _applicationVersion;
		return _applicationName + " - Version " + _applicationVersion 
			+ " vom " + _applicationReleaseDate;
	}
	
	/**
	 * Validates all parameters.
	 * 
	 * @param applicationName_ Application name.
	 * @param applicationVersion_ Application version.
	 * @param applicationReleaseDate_ Application release date.
	 */
	private void validate(
			final String applicationName_, 
			final String applicationVersion_,
			final String applicationReleaseDate_)
	{
		if (applicationName_ == null)
			throw new NullPointerException("Application name is null");
		if (applicationVersion_ == null)
			throw new NullPointerException("Application version is null");

		if (applicationName_.isEmpty())
			throw new IllegalArgumentException("Application name is empty");
		if (applicationVersion_.isEmpty())
			throw new IllegalArgumentException("Application version is empty");
		
		if (applicationReleaseDate_ != null && !applicationReleaseDate_.matches(RELEASE_DATE_MATCH))
			throw new IllegalArgumentException("Application release date doesn't match a date: " + applicationReleaseDate_ + ", must be of: " + RELEASE_DATE_MATCH);
	}
}
/* end of file */