/*
 *
 * File: EnumerationException.java
 * Date: Jun 7, 2007; 9:15:11 AM
 */
package de.desy.mst.libs.accelerator.exception;

import de.desy.mst.libs.accelerator.IEEnum;

/**
 * The general enumeration exception.
 *
 * @author <a href="mailto:marcus.walla at desy.de">walla</a>
 * @version $Id: EnumerationException.java 18321 2008-11-07 11:31:46Z walla $
 */
public class EnumerationException extends Exception
{
	/** Is the unique serial version UID */
	public static final long serialVersionUID = 6344501023656093402l;

	/** The data element if set. */
	protected IEEnum _data = null;

	/**
	 * The constructor of this exception.
	 */
	public EnumerationException()
	{
	}

	/**
	 * The constructor of this exception.
	 *
	 * @param message_ Sets the message as its description.
	 */
	public EnumerationException(final String message_)
	{
		super(message_);
	}

	/**
	 * The constructor of this exception.
	 *
	 * @param cause_ Is the throwable cause of this exception.
	 */
	public EnumerationException(final Throwable cause_)
	{
		super(cause_);
	}

	/**
	 * The constructor of this exception.
	 *
	 * @param message_ Sets the message as its description.
	 * @param cause_ Is the throwable cause of this exception.
	 */
	public EnumerationException(final String message_, final Throwable cause_)
	{
		super(message_, cause_);
	}
	/**
	 * The constructor of this exception.
	 *
	 * @param data_ The data element on which this exception occurred.
	 */
	public EnumerationException(final IEEnum data_)
	{
		super();
		_data = data_;
	}

	/**
	 * The constructor of this exception.
	 *
	 * @param data_ The data element on which this exception occurred.
	 * @param message_ Sets the message as its description.
	 */
	public EnumerationException(final IEEnum data_, final String message_)
	{
		super(message_ + ((data_!=null)?" - data=" + data_.toString():""));
		_data = data_;
	}

	/**
	 * The constructor of this exception.
	 *
	 * @param data_ The data element on which this exception occurred.
	 * @param cause_ Is the throwable cause of this exception.
	 */
	public EnumerationException(final IEEnum data_, final Throwable cause_)
	{
		super(cause_);
		_data = data_;
	}

	/**
	 * The constructor of this exception.
	 *
	 * @param data_ The data element on which this exception occurred.
	 * @param message_ Sets the message as its description.
	 * @param cause_ Is the throwable cause of this exception.
	 */
	public EnumerationException(final IEEnum data_, final String message_,
			final Throwable cause_)
	{
		super(message_ + ((data_!=null)?" - data=" + data_.toString():""),
			cause_);
		_data = data_;
	}

	/**
	 * Returns the data element. Can also be {@code null}.
	 *
	 * @return The data element.
	 */
	public IEEnum getData()
	{
		return _data;
	}
}
/* end of file */