/*
 *
 * File: GeneralUserException.java
 * Date: Apr 8, 2008; 2:04:19 PM
 */
package de.desy.mst.libs.accelerator.exception;

import de.desy.mst.auxiliary.devl.deviceserverlib.exceptions.UserException;
import de.desy.mst.libs.accelerator.model.EErrorExt;
import de.desy.tine.definitions.TErrorList;

/**
 * This is a general error exception which will report error messages on the
 * current error extension enumeration. This provides error items which are
 * not listed in the TINE error codes.
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: GeneralUserException.java 47183 2011-11-29 13:11:36Z walla $
 */
public class GeneralUserException extends UserException {
	// --- CLASS FIELDS --------------------------------------------------------
	private static final long serialVersionUID = 3245054596256234617L;

	// --- CONSTRUCTORS --------------------------------------------------------
	/**
	 * A general error code exception is created with the error extension
	 * enumeration. 
	 * @param error_ Is the error extension enumeration.
	 */
	public GeneralUserException(final EErrorExt error_) {
		super(error_.getErrorCode());
	}

	/**
	 * A general error code exception is created with the error extension
	 * enumeration. 
	 * @param error_ Is the error extension enumeration.
	 * @param message_ Is a detailed explanation of the error.
	 */
	public GeneralUserException(final EErrorExt error_, final String message_) {
		super(error_.getErrorCode(), error_.getError() + 
				((message_ == null || message_.isEmpty())?(": " + message_):null));
	}
	
	// --- INSTANCE METHODS ----------------------------------------------------
	@Override public String toString() {
		final String message = super.getMessage() == null ? "" : super.getMessage() + " - ";
		return "general failure: " + message + TErrorList.getErrorString(super.getErrorKey()) + " (" + super.getErrorKey() + ')';
 	}
}
/* end of file */