/*
 *
 * File: IllegalTinePropertyException.java
 * Date: Oct 22, 2008; 12:46:54 PM
 */
package de.desy.mst.libs.accelerator.exception;

import de.desy.mst.auxiliary.devl.deviceserverlib.exceptions.UserException;
import de.desy.tine.definitions.TErrorList;

/**
 * Informs the user that the TINE property value is out of range or illegal.
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: IllegalTinePropertyException.java 47183 2011-11-29 13:11:36Z walla $
 */
public class IllegalTinePropertyException extends UserException {
	// --- CLASS FIELDS --------------------------------------------------------
	private static final long serialVersionUID = 5220113000006042982L;

	// --- CONSTRUCTORS --------------------------------------------------------
	/**
	 * Constructor.
	 */
	public IllegalTinePropertyException() { /* no functionality */ }

	/**
	 * Constructor.
	 * @param message_ Message.
	 */
	public IllegalTinePropertyException(final String message_) {
		super(message_);
	}

	/**
	 * Constructor.
	 * @param errorCode_ TINE error code.
	 */
	public IllegalTinePropertyException(final int errorCode_) {
		super(errorCode_);
	}

	/**
	 * Constructor.
	 * @param errorCode_ TINE error code.
	 * @param errorMessage_ Message.
	 */
	public IllegalTinePropertyException(final int errorCode_, final String errorMessage_) {
		super(errorCode_, errorMessage_);
	}
	
	// --- INSTANCE METHODS ----------------------------------------------------
	@Override public String toString() {
		final String message = super.getMessage() == null ? "" : super.getMessage() + " - ";
		return "illegal TINE property: " + message + TErrorList.getErrorString(super.getErrorKey()) + " (" + super.getErrorKey() + ')';
 	}
}
/* end of file */