/*
 *
 * File: NotImplementedException.java
 * Date: Jan 29, 2009; 4:05:46 PM
 */
package de.desy.mst.libs.accelerator.exception;


/**
 * Tells the developer that this method is currently not implemented.
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @since Jan 29, 2009
 */
public class NotImplementedException extends RuntimeException
{
	/** Is the serial version UID to unify this class. */
	private static final long serialVersionUID = -287553622460170316L;
	
	private final String _className;
	private final String _methodName;

	/**
	 * Constructor.
	 */
	public NotImplementedException()
	{
		super("This method is currenty not implemented.");
		
		_className = null;
		_methodName = null;
	}

	/**
	 * Constructor.
	 * 
	 * @param message_ A message of this exception.
	 */
	public NotImplementedException(
			final String message_)
	{
		super(message_);
		
		_className = null;
		_methodName = null;
	}
	

	/**
	 * Constructor.
	 * 
	 * @param message_ A message of this exception.
	 * @param className_ The class name.
	 */
	public NotImplementedException(
			final String message_,
			final String className_)
	{
		super(message_);
		
		_className = className_;
		_methodName = null;
	}
	
	/**
	 * Constructor.
	 * 
	 * @param message_ A message of this exception.
	 * @param className_ The class name.
	 * @param methodName_ The method name.
	 */
	public NotImplementedException(
			final String message_, 
			final String className_, 
			final String methodName_)
	{
		super(message_);
		
		_className = className_;
		_methodName = methodName_;
	}

	/**
	 * Constructor.
	 * 
	 * @param cause_ The cause of this exception.
	 */
	public NotImplementedException(
			final Throwable cause_)
	{
		super("This method is currently not implemented.", cause_);

		_className = null;
		_methodName = null;
	}

	/**
	 * Constructor.
	 * 
	 * @param message_ A message of this exception.
	 * @param cause_ The cause of this exception.
	 */
	public NotImplementedException(
			final String message_, 
			final Throwable cause_)
	{
		super(message_, cause_);

		_className = null;
		_methodName = null;
	}

	/**
	 * Returns the method name.
	 * 
	 * @return Returns the method name.
	 */
	public String getMethodName()
	{
		return (_methodName == null ? "" : _methodName);
	}
	
	/**
	 * Returns the class name.
	 * 
	 * @return Returns the class name.
	 */
	public String getClassName()
	{
		return (_className == null ? "" : _className);
	}
	
	/**
	 * Returns the fill qualified class name including the method name.
	 * 
	 * @return Returns the fill qualified class name including the method name.
	 */
	public String getFullClassName()
	{
		return (_className == null ? "":_className + '.') + 
			(_methodName == null ? "" : _methodName); 
	}
}
/* end of file */