/*
 *
 * File: ECavityView.java
 * Date: Feb 6, 2008; 10:19:06 AM
 */
package de.desy.mst.libs.accelerator.model.cavity;

import de.desy.mst.libs.accelerator.EName;
import de.desy.mst.libs.accelerator.IEViewGeneric;
import de.desy.mst.libs.accelerator.exception.NotImplementedException;

/**
 * This is the enumeration of all viewable cavity components stored as an
 * entity. Currently their are four:
 *
 * <table cellspacing="4" cellpadding="4">
 *
 * <tr><th>Enumeration {@link ECavityView}</th><th>Description</th></tr>
 * <tr bgcolor="#A0A0A0"><td>{@link #GENERAL}</td><td>In this enumeration are all general information of
 *  the cavity stored: the device TINE context, the server and its device or
 *   property name and the STATUS of the transmitter.</td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link #ELEMENT}</td><td>All normal and standard values of a cavity are
 *  stored here. Each enumeration value are mandatory: Stamp 1 and 2, power of
 *   VORLAUF and RUECKLAUF, voltage of cavity and its phase etc.</td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link #SWITCH}</td><td>The switches of each cavity. Currently their are four
 * mandatory ones: the minimum, maximum, and resonance frequency and switching
 *  the hole cavity off. The SOLL voltage is for free and can be set.</td></tr>
 * <tr bgcolor="#A0A0A0"><td>{@link #ALL}</td><td>The switches of all cavities.</td></tr>
 *
 * </table>
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: ECavityView.java 19491 2008-12-07 15:18:19Z walla $
 */
public enum ECavityView implements IEViewGeneric {
	// --- ENUMERATIONS --------------------------------------------------------
	GENERAL,
	ELEMENT,
	SWITCH,
	ALL;

	// --- INSTANCE METHODS ----------------------------------------------------
	/**
	 * Always returns {@code null}.
	 *
	 * @see de.desy.mst.libs.accelerator.IEEnum#getName(de.desy.mst.libs.accelerator.EName)
	 */
	@Override public String getName(final EName name_) { return null; }
	/**
	 * Allways throws an unchecked exception.
	 *
	 * @see de.desy.mst.libs.accelerator.IEEnum#putName(de.desy.mst.libs.accelerator.EName, java.lang.String)
	 */
	@Override public void putName(final EName name_, final String value_) { throw new NotImplementedException(); }
}
/* end of file */