/*
 *
 * File: DDouble.java
 * Date: Nov 29, 2006; 6:18:11 PM
 */
package de.desy.mst.libs.accelerator.number;

/**
 * This will encapsulate a single double value.
 *
 * @author <a href="marcus.walla at desy.de">walla</a>
 * @version $Id: DDouble.java 22583 2009-03-10 11:22:31Z walla $
 * @see INumber
 * @see DByte
 * @see DShort
 * @see DLong
 * @see DInteger
 * @see DFloat
 *
 */
public final class DDouble implements INumber
{
	
	/** The internal double variable. */
	private Double _double = 0.0;
	
	/**
	 * Constructor of a the field double. The double value is set to zero.
	 */
	public DDouble()
	{
		_double = 0.0;
	}
	
	/**
	 * The constructor of the field double. Will get one argument as a double value.
	 * 
	 * @param double_ Gets one argument as a double value.
	 */
	public DDouble(double double_)
	{
		_double = double_;
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#booleanValue()
	 */
	public boolean booleanValue()
	{
		return _double == 1.0 ? true : false;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#byteValue()
	 */
	public byte byteValue()
	{
		return _double.byteValue();
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#doubleValue()
	 */
	public double doubleValue()
	{
		return _double;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#floatValue()
	 */
	public float floatValue()
	{
		return _double.floatValue();
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#intValue()
	 */
	public int intValue()
	{
		return _double.intValue();
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#longValue()
	 */
	public long longValue()
	{
		return _double.longValue();
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#shortValue()
	 */
	public short shortValue()
	{
		return _double.shortValue();
	}
	
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#setValue(byte)
	 */
	public void setValue(boolean bool_)
	{
		_double = (bool_ == true ? 1.0 : 0.0);
	}


	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#setValue(byte)
	 */
	public void setValue(byte byte_)
	{
		setValue((double) byte_);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#setValue(short)
	 */
	public void setValue(short short_)
	{
		setValue((double) short_);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#setValue(int)
	 */
	public void setValue(int int_)
	{
		setValue((double) int_);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#setValue(long)
	 */
	public void setValue(long long_)
	{
		setValue((double) long_);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#setValue(float)
	 */
	public void setValue(float float_)
	{
		setValue((double) float_);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#setValue(double)
	 */
	public void setValue(double double_)
	{
		_double = double_;
	}
	
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(final Object object_)
	{
		return _double.equals(object_);
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode()
	{
		return _double.hashCode();
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString()
	{
		return _double.toString();
	}

}
/* end of file */