/*
 *
 * File: DInteger.java
 * Date: Nov 29, 2006; 6:03:37 PM
 */
package de.desy.mst.libs.accelerator.number;

/**
 * This will encapsulate a single integer value.
 *
 * @author <a href="marcus.walla at desy.de">walla</a>
 * @version $Id: DInteger.java 22583 2009-03-10 11:22:31Z walla $
 * @see INumber
 * @see DByte
 * @see DShort
 * @see DLong
 * @see DFloat
 * @see DDouble
 *
 */
public class DInteger implements INumber
{
	/** The internal integer variable. */
	private Integer _integer = 0;
	
	/**
	 * Constructor of a the field integer. The integer value is set to zero.
	 */
	public DInteger()
	{
		_integer = 0;
	}
	
	/**
	 * The constructor of the field integer. Will get one argument as a integer value.
	 * 
	 * @param int_ Gets one argument as a integer value.
	 */
	public DInteger(int int_)
	{
		_integer = int_;
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#booleanValue()
	 */
	public boolean booleanValue()
	{
		return _integer == 1 ? true : false;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#byteValue()
	 */
	public byte byteValue()
	{
		return _integer.byteValue();
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#doubleValue()
	 */
	public double doubleValue()
	{
		return _integer.doubleValue();
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#floatValue()
	 */
	public float floatValue()
	{
		return _integer.floatValue();
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#intValue()
	 */
	public int intValue()
	{
		return _integer;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#longValue()
	 */
	public long longValue()
	{
		return _integer;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#shortValue()
	 */
	public short shortValue()
	{
		return _integer.shortValue();
	}
	
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#setValue(byte)
	 */
	public void setValue(boolean bool_)
	{
		_integer = (bool_ == true ? 1 : 0);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#setValue(byte)
	 */
	public void setValue(byte byte_)
	{
		_integer = (int) byte_;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#setValue(short)
	 */
	public void setValue(short short_)
	{
		_integer = (int) short_;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#setValue(int)
	 */
	public void setValue(int int_)
	{
		_integer = int_;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#setValue(long)
	 */
	public void setValue(long long_)
	{
		_integer = (int) long_;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#setValue(float)
	 */
	public void setValue(float float_)
	{
		_integer = (int) float_;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#setValue(double)
	 */
	public void setValue(double double_)
	{
		_integer = (int) double_;
	}
	
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(final Object object_)
	{
		return _integer.equals(object_);
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode()
	{
		return _integer.hashCode();
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString()
	{
		return _integer.toString();
	}
}
/* end of file */