/*
 *
 * File: INTEGER.java
 * Date: Jul 16, 2008; 4:53:09 PM
 */
package de.desy.mst.libs.accelerator.tine.types;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import de.desy.tine.definitions.TFormat;
import de.desy.tine.types.TCompoundDataObject;

/**
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: INTEGER.java 48102 2012-01-17 14:03:18Z walla $ 
 */
public class INTEGER extends ACompoundDataObject
{
	/** The current float field of this data element. */
	public int ival;
	
	/** The size of bytes of this data element. */
	public static final int sizeInBytes = 4;
	
	/** TINE data type. */
	public static final short dFormat = TFormat.CF_INT32;
	
	/** Buffered array with the size of bytes. */
	ByteArrayOutputStream dBuffer = new ByteArrayOutputStream(sizeInBytes);
	
	/** Is the corresponding value as a byte array. */
	public byte[] hByteArray;
	
	/**
	 * Constructor. Sets the current float value to 0.0.
	 */
	public INTEGER() 
	{
		ival = 0;
	}  

	/**
	 * Constructor. Sets the current float value to {@code f}.
	 * 
	 * @param i
	 */
	public INTEGER(final int i) 
	{ 
		ival = i; 
	} 
	
	/**
	 * Constructor. Copies the current instance of float to the new created one.
	 * 
	 * @param instance_ Instance of a float value.
	 */
	public INTEGER(final INTEGER instance_) 
	{ 
		ival = instance_.ival; 
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.tine.types.TCompoundDataObject#getSizeInBytes()
	 */
	public int getSizeInBytes() 
	{
		return sizeInBytes; 
	}
	
	/**
	 * Sets the float value.
	 * 
	 * @param newFValue_ Sets the current float value.
	 */
	public void setValues(int newFValue_)
	{
	    ival = newFValue_;
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.tine.types.TCompoundDataObject#getFormat()
	 */
	public short getFormat() 
	{ 
		return dFormat; 
	}    
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.tine.types.TCompoundDataObject#setByteArray(byte[])
	 */
	public void setByteArray(final byte[] byteArray_) 
	{ 
		hByteArray = byteArray_; 
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.tine.types.TCompoundDataObject#getByteArray()
	 */
	public byte[] getByteArray() 
	{ 
		return hByteArray; 
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.tine.types.TCompoundDataObject#getIntegerValue()
	 */
	public int getIntegerValue()
	{
		return ival;
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.tine.types.TCompoundDataObject#getDoubleValue()
	 */
	public double getDoubleValue()
	{
		return ival;
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString()
	{
		return "["+ ival + "]";
	}
	
	/**
	 * Checks if both value are equal and returns the corresponding boolean
	 * value.
	 * 
	 * @param instance_ Request the equality of the delivered instance of a 
	 * 		float with the current one.
	 * @return Returns a {@code true} if both values are equal otherwise a
	 * 		{@code false}.
	 */
	public boolean equals(final INTEGER instance_)
	{
		if (instance_.ival != ival) return false;
	    return true;
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.tine.types.TCompoundDataObject#toByteArray()
	 */
	public byte[] toByteArray()
	{
		try
	    {
	    	dBuffer.reset();
	    	DataOutputStream ds = new DataOutputStream(dBuffer);
	    	//byte[] b = Swap.Long(ival);
	    	ds.writeInt(ival);
	    	ds.close();
	    }
	    catch (IOException e)
	    {
	    	e.printStackTrace();
	    }
	    hByteArray = dBuffer.toByteArray();
	    return hByteArray;
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.tine.types.TCompoundDataObject#getTag()
	 */
	public String getTag()
	{
		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.tine.types.TCompoundDataObject#toStruct()
	 */
	public void toStruct()
	{
		try
	    {
	    	ByteArrayInputStream dis = new ByteArrayInputStream(hByteArray);
	    	DataInputStream ds = new DataInputStream(dis);
	    	ival = ds.readInt();
	    	ds.close(); 
	    	dis.close();
	    }
	    catch (IOException exception)
	    {
	    	exception.printStackTrace();
	    }	 
	}
	  
	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode()
	{
//		final int PRIME = 31;
//		int result = 1;
//		result = PRIME * result + ival;
//		return result;
		return ival;
	}
	  
	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj)
	{
		if (this == obj) return true;
		if (obj == null) return false;
		if (getClass() != obj.getClass()) return false;
		final FLOAT other = (FLOAT) obj;
		return equals(other);
	}
	
	@Override public TCompoundDataObject clone() { return new INTEGER(this); }
}
/* end of file */