/*
 *
 * File: CColor.java
 * Date: Feb 1, 2007; 5:39:32 PM
 */
package de.desy.mst.libs.accelerator.view;

import java.awt.Color;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JLabel;

import de.desy.mst.libs.accelerator.exception.EnumerationNotUsedException;
import de.desy.mst.libs.framework.mstapp.util.gui.AppColor;

/**
 * From this final class there are different color objects which help one
 * to install the right color values. These are nearly equivalent to the
 * color defined in the java documentation of DESY, MST.
 *
 * @author <a href="marcus.walla at desy.de">walla</a>
 * @version $Id: CColor.java 33461 2010-02-23 14:20:33Z walla $
 */
public final class CColor
{
	public static final Color GREY1 = new Color(21, 21, 21);

	public static final Color GREY2 = new Color(42, 42, 42);

	public static final Color GREY3 = new Color(63, 63, 63);

	public static final Color GREY4 = new Color(84, 84, 84);

	public static final Color GREY5 = new Color(105, 105, 105);

	public static final Color GREY6 = new Color(126, 126, 126);

	public static final Color GREY7 = new Color(147, 147, 147);

	public static final Color GREY8 = new Color(188, 188, 188);

	public static final Color GREY9 = new Color(209, 209, 209);

	public static final Color GREY10 = new Color(230, 230, 230);

	public static final Color BLACK = Color.BLACK;

	public static final Color WHITE = Color.WHITE;

	public static final Color GRAY = Color.GRAY;

	public static final Color GRAY_LIGHT = Color.GRAY.brighter();

	public static final Color GRAY_DARK = Color.GRAY.darker();

	public static final Color RED = Color.RED;

	public static final Color RED_LIGHT = Color.RED.brighter();

	public static final Color RED_DARK = Color.red.darker();

	public static final Color GREEN = Color.GREEN;

	public static final Color GREEN_LIGHT = Color.GREEN.brighter();

	public static final Color GREEN_DARK = Color.GREEN.darker();

	public static final Color YELLOW = Color.YELLOW;

	public static final Color BLUE = Color.BLUE;

	public static final Color LIGHT_BLUE = new Color(200,220,255);

	public static final Color TRANSPARENCY = new Color(0, 0, 0, 0);
	
	private static final int LENGTH_OF_FLASH = 40;
	
	@Deprecated
	public static final Color[] FLUSH = new Color[LENGTH_OF_FLASH];

	@Deprecated
	public static final Color[] FLUSH_YELLOW = new Color[LENGTH_OF_FLASH];

	private static final Color[] FLASH_GREEN = new Color[LENGTH_OF_FLASH];
	private static int _idxFlashGreen = 0;
	
	private static final Color[] FLASH_YELLOW = new Color[LENGTH_OF_FLASH];
	private static int _idxFlashYellow = 0;
	
	private static final Color[] FLASH_RED = new Color[LENGTH_OF_FLASH];
	private static int _idxFlashRed = 0;

	/** The foreground color of the ON switch/status. */
	public static final Color MST_ON_FOREGROUND = AppColor._EIN_UND_OK.getForeGround();

	/** The background color of the ON switch/status. */
	public static final Color MST_ON_BACKGROUND = AppColor._EIN_UND_OK.getBackGround();

	/** The foreground color of the OFF switch/status. */
	public static final Color MST_OFF_FOREGROUND = AppColor._AUS.getForeGround();

	/** The background color of the OFF switch/status. */
	public static final Color MST_OFF_BACKGROUND = AppColor._AUS.getBackGround();

	/** The foreground color of the ERROR switch/status. */
	public static final Color MST_ERROR_FOREGROUND = AppColor._FEHLER.getForeGround();

	/** The background color of the ERROR switch/status. */
	public static final Color MST_ERROR_BACKGROUND = AppColor._FEHLER.getBackGround();

	/** The foreground color of the WARNING switch/status. */
	public static final Color MST_WARNING_FOREGROUND = AppColor._WARNUNG.getForeGround();

	/** The background color of the WARNING switch/status. */
	public static final Color MST_WARNING_BACKGROUND = AppColor._WARNUNG.getBackGround();

	/** The foreground color of the UNDEFINED switch/status. */
	public static final Color MST_UNDEF_FOREGROUND = AppColor._UNDEFINIERT.getForeGround();

	/** The background color of the UNDEFINED switch/status. */
	public static final Color MST_UNDEF_BACKGROUND = AppColor._UNDEFINIERT.getBackGround();

	/** The foreground color of a table header. */
	public static final Color MST_TABLEHEADER_FOREGROUND = AppColor._TABLEHEADER.getForeGround();

	/** The background color of a table header. */
	public static final Color MST_TABLEHEADER_BACKGROUND = AppColor._TABLEHEADER.getBackGround();

	/** The foreground color of the UNDEFINED switch/status. */
	public static final Color MST_OFFLINE_FOREGROUND = Color.WHITE;

	/** The background color of the UNDEFINED switch/status. */
	public static final Color MST_OFFLINE_BACKGROUND = Color.MAGENTA;

	/** The foreground color of the VALUE switch/status. */
	public static final Color MST_VALUE_FOREGROUND = Color.BLACK;

	/** The background color of the VALUE switch/status. */
	public static final Color MST_VALUE_BACKGROUND = null; //Color.LIGHT_GRAY;

	/** A private label for getting the default fore- and background color. */
	private static final JLabel LABEL = new JLabel();

	/** The default background color. */
	public static final Color MST_DEFAULT_BACKGROUND = LABEL.getBackground();

	/** The default foreground color. */
	public static final Color MST_DEFAULT_FOREGROUND = LABEL.getForeground();

	/** An array of all MST foreground colors. */
	public static final Color[] MST_COLOR_FOREGROUND = {
		MST_ON_FOREGROUND, MST_OFF_FOREGROUND, MST_ERROR_FOREGROUND,
		MST_WARNING_FOREGROUND, MST_UNDEF_FOREGROUND, MST_OFFLINE_FOREGROUND,
		MST_VALUE_FOREGROUND, MST_TABLEHEADER_FOREGROUND};

	/** An array of all MST background colors. */
	public static final Color[] MST_COLOR_BACKGROUND = {
		MST_ON_BACKGROUND, MST_OFF_BACKGROUND, MST_ERROR_BACKGROUND,
		MST_WARNING_BACKGROUND, MST_UNDEF_BACKGROUND, MST_OFFLINE_BACKGROUND,
		MST_VALUE_FOREGROUND, MST_TABLEHEADER_BACKGROUND};

	/** An associated map of all MST foreground and background colors. */
	public static final Map<String, Color> MST_COLOR =
		new HashMap<String, Color>();
	
	public enum EFlashColor { GREEN, YELLOW, RED };

	static
	{
		for(int i = 0; i < FLASH_GREEN.length; i++)
		{
			FLASH_GREEN[i] = new Color(
					0, //255 - (int) (flash(((double) i) / ((double) FLASH_GREEN.length)) * 128), 
					255 - (int) (flash(((double) i) / ((double) FLASH_GREEN.length)) * 128), 
					0); //255 - (int) (flash(((double) i) / ((double) FLASH_GREEN.length)) * 255));
			FLUSH[i] = FLASH_GREEN[i];
		}
		
		for (int i = 0; i < FLASH_YELLOW.length; i++)
		{
			FLASH_YELLOW[i] = new Color(
					255, // - (int) (flash(((double) i) / ((double) FLASH_YELLOW.length)) * 128), 
					255 - (int) (flash(((double) i) / ((double) FLASH_YELLOW.length)) * 128), 
					0); //255 - (int) (flash(((double) i) / ((double) FLASH_YELLOW.length)) * 255));
			FLUSH_YELLOW[i] = FLASH_YELLOW[i];
		}
		
		for (int i = 0; i < FLASH_RED.length; i++)
			FLASH_RED[i] = new Color(
					255 - (int) (flash(((double) i) / ((double) FLASH_RED.length)) * 128), 
					0, // - (int) (flash(((double) i) / ((double) FLASH_RED.length)) * 255), 
					0); // - (int) (flash(((double) i) / ((double) FLASH_RED.length)) * 255));

		MST_COLOR.put("ON_FOREGROUND", MST_ON_FOREGROUND);
		MST_COLOR.put("ON_BACKGROUND", MST_ON_BACKGROUND);
		MST_COLOR.put("OFF_FOREGROUND", MST_OFF_FOREGROUND);
		MST_COLOR.put("OFF_BACKGROUND", MST_OFF_BACKGROUND);
		MST_COLOR.put("ERROR_FOREGROUND", MST_ERROR_FOREGROUND);
		MST_COLOR.put("ERROR_BACKGROUND", MST_ERROR_BACKGROUND);
		MST_COLOR.put("WARNING_FOREGROUND", MST_WARNING_FOREGROUND);
		MST_COLOR.put("WARNING_BACKGROUND", MST_WARNING_BACKGROUND);
		MST_COLOR.put("UNDEF_FOREGROUND", MST_UNDEF_FOREGROUND);
		MST_COLOR.put("UNDEF_BACKGROUND", MST_UNDEF_BACKGROUND);
		MST_COLOR.put("OFFLINE_FOREGROUND", MST_OFFLINE_FOREGROUND);
		MST_COLOR.put("OFFLINE_BACKGROUND", MST_OFFLINE_BACKGROUND);
		MST_COLOR.put("VALUE_FOREGROUND", MST_VALUE_FOREGROUND);
		MST_COLOR.put("VALUE_BACKGROUND", MST_VALUE_BACKGROUND);
		MST_COLOR.put("TABLEHEADER_FOREGROUND", MST_TABLEHEADER_FOREGROUND);
		MST_COLOR.put("TABLEHEADER_BACKGROUND", MST_TABLEHEADER_BACKGROUND);
	}
	
	/**
	 * Returns the next flash color of the specified flash color enumeration.
	 * 
	 * @param flashColor_ Flash color enumeration.
	 * @return The next flash color.
	 */
	public static Color getNextFlashColor(final EFlashColor flashColor_)
	{
		switch(flashColor_)
		{
		case GREEN:
			return FLASH_GREEN[_idxFlashGreen++ % FLASH_GREEN.length];
		case YELLOW:
			return FLASH_YELLOW[_idxFlashYellow++ % FLASH_YELLOW.length];
		case RED:
			return FLASH_RED[_idxFlashRed++ % FLASH_RED.length];
		default:
			throw new EnumerationNotUsedException("Enumeration of " + flashColor_ 
					+ " indeed exits but is not used");
		}
	}

	/**
	 * Resets all flash color counters for restarting them at the beginning.
	 */
	public static void resetFlashColor()
	{
		for(final EFlashColor color: EFlashColor.values())
			resetFlashColor(color);
	}

	/**
	 * Resets the flash color counter of the specified enumeration {@link EFlashColor} 
	 * for restarting them at the beginning.
	 * 
	 * @param flashColor_ Flash color enumeration.
	 */
	public static void resetFlashColor(final EFlashColor flashColor_)
	{
		switch(flashColor_)
		{
		case GREEN:
			_idxFlashGreen = 0;
			break;
		case YELLOW:
			_idxFlashYellow = 0;
			break;
		case RED:
			_idxFlashRed = 0;
			break;
		default:
			throw new EnumerationNotUsedException("Enumeration of " + flashColor_ 
					+ " indeed exits but is not used");
		}
	}
	
	/**
	 * <p>Converts from the base input value between 0.0 to 1.0 to an output value 
	 * which reaches between 1.0 up to 0.0. The transformation is done with an
	 * absolute value of a sine curve.</p>
	 * 
	 * <p><b>REMARK:</b> Currently I use a <tt>sine</tt> transformation, where the values
	 * from 0.0 (inclusive) up to 1.0 (inclusive) are linear transformed to 0.0
	 * (inclusive) up to 2*PI (inclusive), this is move up by adding a 1.0. The
	 * resulting values between 0.0 (inclusive) and 2.0 (inclusive) is transformed
	 * to 0.0 (inclusive) up to 1.0 (inclusive) by dividing the result with 2.0.</p>
	 * 
	 * @param value_ Transformation value.
	 * @return Calculated sine curve result.
	 */
	private static double flash(final double value_)
	{
		return Math.abs((Math.sin(value_ * 2 * Math.PI) + 1.0) / 2.0);
	}
}
/* end of file */