/**
 * 
 * Last changed: $Date$
 * By: $Author$
 * URL: $HeadURL$
 * $Revision$ 
 *
 */
package de.desy.mst.libs.framework.mstapp.appinf;

import de.desy.mst.libs.framework.mstapp.logging.Logger;



/**
 * @author labudda
 *
 */
public final class ContactInformation implements IContactInformation {
	private final String name;
	private final String phone;
	private final String email;

	/** define object by property file
	 * @param properties
	 */
	public ContactInformation(final java.util.Properties properties) {
		final String firstName = properties.getProperty("author.firstName", null); 
		final String lastName = properties.getProperty("author.lastName", null);
		final String emailDefault;
		if (firstName == null || lastName == null) {
			this.name = null;
			emailDefault = null;
		} else {
			this.name = firstName + " " + lastName;
			emailDefault = firstName + "." + lastName;
		}
		this.phone = properties.getProperty("author.phone", null);
		final String propertyEmail = properties.getProperty("author.email", null);
		if (propertyEmail == null) {
			if (emailDefault == null) {
				this.email = null;
			} else {
				this.email = emailDefault + "@desy.de";
			}
		} else {
			this.email = propertyEmail;
		}
	}

	/** define object by jnlp vendor string
	 * @param vendor
	 */
	public ContactInformation(final String vendor) {
		Logger.SWApp.debug("Vendor: " + vendor);
		final String phoneTag = "phone:"; 
		final String emailTag = "email:";
		this.name = ContactInformation.getTagValue(vendor, "");
		this.phone = ContactInformation.getTagValue(vendor, phoneTag);
		this.email = ContactInformation.getTagValue(vendor, emailTag);
	}
	/**
	 * @param vendor
	 * @param string
	 * @return value of tag or null
	 */
	private static String getTagValue(final String vendor, final String tag) {
		final int tagStart = vendor.toLowerCase().indexOf(tag.toLowerCase());
		if (tagStart < 0) {
			return null;
		} else {
			int tagEnd = vendor.indexOf(':', tagStart + tag.length());
			if (tagEnd < 0) {
				return vendor.substring(tagStart + tag.length()).trim();
			} else {
				final String tempValue = vendor.substring(tagStart + tag.length(), tagEnd - 1);
				tagEnd = tempValue.lastIndexOf(" ");
				if (tagEnd < 0) {
					return tempValue.trim();
				} else {
					return tempValue.substring(0, tagEnd).trim();
				}
			}
		}
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.IContactInformation#getEmail()
	 */
	@Override public String getEmail() {
		return this.email;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.IContactInformation#getName()
	 */
	@Override public String getName() {
		return this.name;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.IContactInformation#getPhone()
	 */
	@Override public String getPhone() {
		return this.phone;
	}

}
