/**
 * 
 * Last changed: $Date$
 * By: $Author$
 * URL: $HeadURL$
 * $Revision$ 
 *
 */
package de.desy.mst.libs.framework.mstapp.appinf.ide;

import java.io.IOException;

import de.desy.mst.libs.framework.mstapp.appinf.AApp;
import de.desy.mst.libs.framework.mstapp.appinf.ContactInformation;
import de.desy.mst.libs.framework.mstapp.appinf.IContactInformation;
import de.desy.mst.libs.framework.mstapp.appinf.ILib;
import de.desy.mst.libs.framework.mstapp.appinf.Lib;

/**
 * @author labudda
 *
 */
class DebugApp extends AApp {
	private final java.util.Properties projectProperties;
	private final java.util.Properties buildProperties;
	private final String iconFileName;
	DebugApp(final String projectFolder) {
		this.projectProperties = new java.util.Properties();
		final String projectPropertiesFilename = DebugApp.getProjectPropertiesFilename(projectFolder);
		try {
			this.projectProperties.load(DebugApp.getStream(projectPropertiesFilename));
		} catch (final IOException e) {
			this.projectProperties.setProperty("process", "undefined");
			this.projectProperties.setProperty("name", "undefined");
			this.projectProperties.setProperty("accelerator", "undefined");
		}
		this.buildProperties = new java.util.Properties();
		final String buildPropertiesFilename = DebugApp.getBuildPropertiesFilename(projectFolder);
		try {
			this.buildProperties.load(DebugApp.getStream(buildPropertiesFilename));
		} catch (final IOException e) {
			this.buildProperties.setProperty("Major", "0");
			this.buildProperties.setProperty("Minor", "0");
			this.buildProperties.setProperty("Revision", "0");
			this.buildProperties.setProperty("build.Number", "0");
		}
		this.iconFileName = this.getIconFilename(projectFolder);
	}

	/**
	 * @param projectFolder
	 * @return the komplet filename to potential iconfile
	 */
	private String getIconFilename(final String projectFolder) {
		final String name = this.projectProperties.getProperty("name");
		if (name == null) {
			return null;
		} else {
			return projectFolder + java.io.File.separator + "src" + java.io.File.separator + "resources" + java.io.File.separator + "webstart"  + java.io.File.separator + name.toLowerCase() + ".png";
		}
	}

	/**
	 * @param projectFolder
	 * @return path and name of propertyfile
	 */
	private static String getBuildPropertiesFilename(final String projectFolder) {
		return projectFolder + java.io.File.separator + "src" + java.io.File.separator + "build.Number";
	}

	/**
	 * @param projectFolder
	 * @return path and name of propertyfile
	 */
	private static String getProjectPropertiesFilename(final String projectFolder) {
		return projectFolder + java.io.File.separator + "project.properties";
	}
	
	static boolean isProjectFolderValid(final String projectFolder) {
		final boolean projectPropertiesExist = DebugApp.existsFile(DebugApp.getProjectPropertiesFilename(projectFolder)); 
		final boolean buildPropertiesExist = DebugApp.existsFile(DebugApp.getBuildPropertiesFilename(projectFolder));
		return  projectPropertiesExist && buildPropertiesExist;
	}

	/**
	 * @param projectPropertiesFilename
	 * @return true if file exists
	 */
	private static boolean existsFile(final String filename) {
		final java.io.File toCheck = new java.io.File(filename);
		return toCheck.isFile();
	}

	/**
	 * @param projectFolder
	 * @return file of properties
	 */
	private static java.io.InputStream getStream(final String filename) throws java.io.IOException {
		final java.io.File file = new java.io.File (filename);
		return new java.io.FileInputStream (file); 
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.IApp#getContact()
	 */
	@Override public IContactInformation getContact() {
		return new ContactInformation(this.projectProperties);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.IApp#getDescription()
	 */
	@Override public String getDescription() {
		return this.projectProperties.getProperty("description");
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.IApp#getHelp()
	 */
	@Override public String getHelp() {
		return this.projectProperties.getProperty("help");
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.IApp#getJars()
	 */
	@Override public java.util.Collection<ILib> getJars() {
		final String classpath = System.getProperty("java.class.path");
		System.out.println("Classpath: " + classpath); 
		final String[] jarNames = classpath.split(java.io.File.pathSeparator);
		final java.util.ArrayList<ILib> jars = new java.util.ArrayList<ILib>(); 
		for(final String jarName: jarNames) {
			try {
				if (jarName.toLowerCase().endsWith(".jar")) {
					final java.util.jar.JarFile jarFile = new java.util.jar.JarFile(jarName); 
					jars.add(new Lib(jarFile));
				} else {
					// not a lib
				}
			} catch (final java.io.IOException e) {
				e.printStackTrace();
			}
		}
		return jars;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.IApp#getName()
	 */
	@Override public String getName() {
		return this.projectProperties.getProperty("name");
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.IApp#getVersion()
	 */
	@Override public String getVersion() {
		final StringBuilder sb = new StringBuilder(this.buildProperties.getProperty("major", "1"));
		sb.append('.');
		sb.append(this.buildProperties.getProperty("minor", "0"));
		sb.append('.');
		sb.append(this.buildProperties.getProperty("release", "0"));
		sb.append('.');
		sb.append(this.buildProperties.getProperty("revision", "0"));
		final String build = this.buildProperties.getProperty("build.number", null);
		if (build == null) {
			// do nothing
		} else {
			sb.append('.');
			sb.append(build);
		}
		return sb.toString();
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.IApp#getIcon()
	 */
	@Override public javax.swing.ImageIcon getIcon() {
		final javax.swing.ImageIcon icon = new javax.swing.ImageIcon(this.iconFileName);
		return icon;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.IApp#getTitle()
	 */
	@Override public String getTitle() {
		return this.projectProperties.getProperty("title");
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.appinf.AApp#getReleaseTimestamp()
	 */
	@Override
	public String getReleaseTimestamp() {
		return (new java.util.Date()).toString();
	}
	
	

}
