/*
 * Created on Mar 17, 2005 - Jun 16, 2005
 * Aug 21, 2007 - Aug 24, 2007	ws:	add threadsafety fr prozentFertig
 * 									add kleinerText
 *	Feb 15, 2008 - Feb 15, 2008 ws: added "@Override"es
 */
package de.desy.mst.libs.framework.mstapp.flash;

import de.desy.mst.libs.framework.mstapp.logging.Logger;

/**
 * Pro ProgressSorte existiert ein Progress pro Applikation.
 * der progress ist ein int mit folgender Bedeutung:
 * 	  -1 ... noch nicht angefangen
 *     0 ... angefangen
 *     n ... n% fertign. 0 <= n <= 100
 *   100 ... fertig
 * Der Wert laesst sich nur erhoehen. Alles andere wird still ignoriert.
 *
 * @author schuette
 * @version 1.0.0
 */
/**
 * @author schuette
 * @version 1.0.0
 */
public final class Progress {
	private static final int START  =   0;	//of prozentFertig
	private static final int FERTIG = 100;	//of prozentFertig
    public static final Progress READ_INI_FILES;
    public static final Progress READ_TABLE_DATA;
	public static final Progress FORM_INIT;
	public static final Progress[] VALUES;
	static{
        READ_INI_FILES = new Progress(ProgressSorte.READ_INI_FILES);
        READ_TABLE_DATA = new Progress(ProgressSorte.READ_TABLE_DATA);
		FORM_INIT = new Progress(ProgressSorte.FORM_INIT);

		//Beachte: Index(Array) = index(Objekt)
		VALUES = (new Progress[]{READ_INI_FILES, READ_TABLE_DATA, FORM_INIT});
	}

	private final ProgressSorte sorte;

	/**@guarded with this*/ private int prozentFertig = -1;
	/**@guarded with this*/ private String kleinerText = "";
	public static final class ProgressSorte {
		/*
		 * Hier sind die verschiedenen Progress Sortenaufgezhlt.
		 * Reine Datenklasse (immutable) typsichere Enumeration.
		*/
		public static final ProgressSorte READ_INI_FILES;
        public static final ProgressSorte READ_TABLE_DATA;
		public static final ProgressSorte FORM_INIT;
		public static final ProgressSorte[] VALUES;

		static{
			READ_INI_FILES 	= new ProgressSorte("Lesen der Initialisierungsdatei",	0);
            READ_TABLE_DATA	= new ProgressSorte("Lesen der Tabellen Daten",			1);
			FORM_INIT 		= new ProgressSorte("Initialisierung der Hauptform",	2);

			//Beachte: Index(Array) = index(Objekt)
			VALUES = (new ProgressSorte[]{READ_INI_FILES, FORM_INIT});
		}

		public final String name;
		public final int index;

		/**
		 * @param name	Bezeichnung der Progesssorte (eindeutig, kann sich aber von Version zu Version ndern)
		 * @param index Eindeutig und gleich dem Index in ProgressSorte.VALUES
		 *
		 * Konstruktor muss privat sein um == Abfrage zu ermoeglichen
		 */
		private ProgressSorte(final String name, final int index) {
			super();
			this.name = name;
			this.index = index;
		}

//		may return null
		public final static ProgressSorte valueOf(final String name){
			for(int index = 0; index < VALUES.length; index++){
				if(name == VALUES[index].name){return VALUES[index];}
			}
			return null;	//Maybe through an error instead?
		}

		/* (non-Javadoc)
		 * @see java.lang.Object#toString()
		 */
		@Override public String toString() {
			return "Progress Sorte [" + this.name + "]";
		}

	}

//=============================================================================

	public Progress(final ProgressSorte sorte) {
		super();
		this.sorte = sorte;
	}

//	=============================================================================

 
	public Progress setProgress
					(final int prozentFertig, final String kleinerText) {
		if(prozentFertig < getProzentFertig()){
			Logger.SWFW.warn("on "
					       + this.toString()
					       + " tried to set Progress to "
					       + prozentFertig
					       + " (since <= silently ignored)"
					        );
		}
		this.prozentFertig =
				Math.min(FERTIG, Math.max(getProzentFertig(),prozentFertig));
		this.kleinerText = (kleinerText == null) ? "" : kleinerText;
		return this;
	}
	public synchronized Progress setProgress(final int prozentFertig){
		return this.setProgress(prozentFertig, "");
	}
	/**Wert wird immer auf 0 zurueckgesetzt auch wenn er vorher hoeher war*/
	public synchronized Progress setStart()	{
		//nur bei setStart kann der Wert zurueckgesetzt werden
		this.prozentFertig = 0;
		return this.setProgress(0);
	}
	public synchronized Progress setFertig(){return this.setProgress(100);}

	public synchronized int getProzentFertig(){return this.prozentFertig;}

	public boolean isInitialized(){return getProzentFertig() >= Progress.START;}
	public boolean isFertig()     {return getProzentFertig() >= Progress.FERTIG;}
	public final String getName() {
		return this.sorte.name + " " + this.kleinerText;
	}
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override public String toString() {
		return this.sorte.toString() + " " + getProzentFertig() + "% fertig.";
	}
}
