/**
 * Mar  5, 2010 - Mar  8, 2010	ws:	Creation from scratch/AnLa's AboutDisplay
 * Apr 28, 2010 - Apr 28, 2010	ws:	allow modal = FALSE for crashframe
 * Dec 22, 2014 - Dec 22, 2014	ws:	change: calcJarInfo(ILib) protect against exception
 * Aug 11, 2014 - Aug 11, 2014	ws: removed: "Ursprngliches About im Hilfeserver"
 */
package de.desy.mst.libs.framework.mstapp.hilfe;

import java.awt.Color;
import java.awt.Component;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.Toolkit;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.ListCellRenderer;
import javax.swing.SwingConstants;

import de.desy.mst.libs.framework.mstapp.appinf.IApp;
import de.desy.mst.libs.framework.mstapp.appinf.IContactInformation;
import de.desy.mst.libs.framework.mstapp.appinf.ILib;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.util.gui.DialogShowingHTML;


/**
 * MAIN_COMMENT_HERE
 * @author ws
 * @version 1.0.0
 */
class _AboutDialog extends DialogShowingHTML {
	private static final long serialVersionUID = 1L;
//-----------------------------------------------------------------------------
	static final public _AboutDialog createAboutDialog(final Component cmp, final String title, final String sUri, final IApp anLaApp, final boolean modal){
		final Component centralCmp = createCentralCmp(sUri, anLaApp);
		final _AboutDialog aboutDialog = new _AboutDialog(title, centralCmp, anLaApp, modal);

		aboutDialog.setLocationRelativeTo(cmp);

		return aboutDialog;

	}
	static final public _AboutDialog createAboutDialog(final Component cmp, final String title, final String sUri, final IApp anLaApp){
		return createAboutDialog(cmp, title, sUri, anLaApp, false);
	}
//-----------------------------------------------------------------------------
	private static Component createCentralCmp(final String sUri, final IApp anLaApp) {
		final JTabbedPane tp = new JTabbedPane();
		tp.addTab("ber die Applikation"
				, extractScaledImage(anLaApp)
				, createAboutApplication(anLaApp)
			);
		tp.addTab("Benutzte Bibliotheken"
				, extractScaledDesyLogo()
				, createJarDisplay(anLaApp.getJars())
			);
//		tp.addTab("Ursprngliches About im Hilfeserver", createEditorPaneFromSURI(sUri));
		tp.addTab("ber das System", getSystemInfo());

		return tp;
//		final JPanel userPanel = new JPanel();
//		userPanel.add(tp);
//		return userPanel;
	}
	@SuppressWarnings({ "rawtypes", "unchecked" })
	private static Component createJarDisplay(final Collection<ILib> jars) {
		final JList display = new JList(calcJarInfo(jars).toArray());
		display.setCellRenderer(new ListCellRenderer(){
			@Override public Component getListCellRendererComponent(final JList list, final Object value, final int index, final boolean isSelected, final boolean cellHasFocus) {
				final JLabel label = new JLabel(value.toString());
				label.setOpaque(true);
				label.setBackground(((index % 2) > 0) ? Color.LIGHT_GRAY : Color.GRAY);
				return label;
			}
		});
		return new JScrollPane(display);
	}
	private static Collection<String> calcJarInfo(final Collection<ILib> jars) {
		final Collection<String> list = new ArrayList<String>(jars.size());
		for (final ILib jar : jars)
			list.add(calcJarInfo(jar));
		return list;
	}
	private static String calcJarInfo(final ILib jar) {
		final StringBuffer sb = new StringBuffer("<html><H3>");
		try {
			sb.append(jar.getName())
			.append("</H3>Datum: ")	.append(jar.getDate())
			.append("<br>Version: ").append(jar.getMajor())
			.append(".")			.append(jar.getMinor())
			.append(".")			.append(jar.getRevision())
			.append("</html>");
		} catch (final Exception ex) {
			Logger.SWFW.warn(_AboutDialog.class.getSimpleName() 
					      + ".calcJarInfo(ILib) throws an exception " + ex, ex);
			sb.append("????</html>");
		}
		final String jarInfo = sb.toString();
		return jarInfo;
	}
	private static Icon extractScaledDesyLogo() {
		final URL logo = _AboutDialog.class.getResource("/resources/desy_logo.gif");
		return scale(Toolkit.getDefaultToolkit().getImage(logo));
	}
	private static ImageIcon extractScaledImage(final IApp anLaApp) {
		return scale(extractImage(anLaApp));
	}
	private static ImageIcon scale(final Image image) {
		final ImageIcon scaled = (image == null) ? null
			: new ImageIcon(image.getScaledInstance(16, 16, Image.SCALE_SMOOTH));
		return scaled;
	}
	private static Image extractImage(final IApp anLaApp) {
		final ImageIcon icon = anLaApp.getIcon();
		final Image image = icon != null ? icon.getImage() : null;
		return image;
	}
	private static JPanel createAboutApplication(final IApp anLaApp) {
			final JPanel panel = new JPanel();
			panel.setLayout(new GridLayout(2,1));
			panel.add(createTitledLabel(calcDeveloperInfo(anLaApp), "Entwicklerin / Entwickler"));
			panel.add(createTitledLabel(calcAppInfo      (anLaApp), "Version"));
			panel.setVisible(true);
			return panel;
	}
	private static JLabel createTitledLabel(final String text, final String title) {
		final JLabel developerDisplay = new JLabel(text);
		developerDisplay.setBorder(BorderFactory.createTitledBorder(title));
		return developerDisplay;
	}
	private static String calcAppInfo(final IApp anLaApp) {
		final StringBuffer sb = new StringBuffer();
		sb.append("<html>").append("Nummer: ")            .append(anLaApp.getVersion         ())
		  .append("<br>")  .append("Datum: ")             .append(anLaApp.getReleaseTimestamp())
		  .append("<br>")  .append("Letzte nderung: ")   .append(anLaApp.getReleaseReason   ())
		  .append("<br>")  .append("Mehr Informationen: ").append(anLaApp.getHelpUrl         ())
		  .append("<br>");
		return sb.toString();
	}
	private static String calcDeveloperInfo(final IApp anLaApp) {
		final StringBuffer sb = new StringBuffer();
		final IContactInformation contact = anLaApp.getContact();
		sb.append("<html>")
		  .append("<H3>").append("Autor: ")	 .append(contact.getName ())
		  .append("<br>").append("Telefon: ").append(contact.getPhone())
		  .append("<br>").append("E-Mail: ") .append(contact.getEmail())
		  .append("</H3>")
		  .append("</html>");
		return sb.toString();
	}
	private static Component getSystemInfo() {
		final JLabel info = new JLabel(calcSystemInfo());
		info.setHorizontalAlignment(SwingConstants.CENTER);
		return info;
	}
	private static String calcSystemInfo() {
		final StringBuffer sb = new StringBuffer();
		sb.append("<html>")
		  .append("<H3>").append(System.getProperty("java.vendor"))
		  .append("<br>").append(System.getProperty("java.vm.name"))
		  .append("<br>Version ").append(System.getProperty("java.version"))
		  .append("<br>Applikationsstarter ").append(System.getProperty("sun.java.launcher"))
		  .append("</H3>")
		  .append("</html>");
		final String string = sb.toString();
		return string;
	}

//=============================================================================
	private _AboutDialog(final String title, final Component centralCmp, final IApp anLaApp, final boolean modal) {
		super(title, centralCmp, modal);
		this.setIconImage(extractImage(anLaApp));
	}
}
