/*
 * Created on 	May 26, 2005 - Jun 10, 2005
 *	Aug 16, 2005 - Aug 23, 2005 ... implements IIniFileObserver, String IO hier implementiert
 *	Oct 25, 2005 - ...	new logging system
 *	Nov .., 2005 - Dec, 06 2005 ... secure string reading from web, buffer inputstreams
 *	Jul 20, 2006 -	... 		ws: SRV_NAME = "Ini" -> "IniDaten" (neue package Struktur -> neues WebVerzeichnis)
 *	Mar  9, 2007 - 				ws: support specificInfo (XTor Problem) (IT IS VERY UGLY AND TRUELY NEEDS SOME DESIGNWORK)
 *	Apr 11, 2007 -	...			ws: added: cleanAppname()	
 *	May 10, 2007 -	...			ws: fileName jedesmal berechnet statt gespeichert
 *	Jul 04, 2007 - Jul 04, 2007	ws: added: name
 *	Aug 29, 2007 - Aug 29, 2007 ws: hierachisch + nutzte PfadeIni
 *	Jun 10, 2015 - Jun 10, 2015	ws:	track web accesses by name via LinkedMap accessedWebAdresses	
 *	Jun 17, 2015 - Jun 17, 2015	ws:	added readFileAsString(final String fileName)	
 */
package de.desy.mst.libs.framework.mstapp.ini;
 
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PushbackInputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.Collections;
import java.util.List;

import de.desy.mst.libs.framework.mstapp.ApplicationInfo;
import de.desy.mst.libs.framework.mstapp.hilfe.AHilfePfade;
import de.desy.mst.libs.framework.mstapp.logging.Logger;

/**
 * package privat
 * holds the IO data for the IniMgr
 * Both the specification and the data itself (IniFileRoot)
 * This class holds the difference between user and app data.
 * @author ws
 * @version 1.0.0
 */
abstract class ABaseData<FR extends IIniFileRoot> implements IIniFileObserver{
	public static final String SRV_NAME = "Ini";   
    public static final String FILE_SUFFIX = ".xml";
	public static final String BACKUP_SUFFIX = ".bak";

	private static final String CHARSET_NAME = "UTF-8";	//TODO potentielles Problem, da hart einkodiert
	
	private static final List<String> accessedWebAdresses = 
										new AccessedWebAdresses(ABaseData.class.getSimpleName());

	private final String fileTyp;
    private final ApplicationInfo appInfo;
	private final String name;
//	private final Object specificInfo;
//    private final String fileName; 
//    private final String webAddress;
    
    private FR fileRoot;
    
//=============================================================================    
  
    public static String cleanAppname(final String appName){
    	return AHilfePfade.cleanName(appName);
    }
    public static final List<String> getAccessedWebAdresses() {
    	return Collections.unmodifiableList(accessedWebAdresses) ;
    }
	public static String readFileAsString(final String fileName) throws FileNotFoundException, IOException {
		return readInputStreamAsString(new FileInputStream(fileName));
	}
//=============================================================================    

	protected ABaseData(final String name, final ApplicationInfo appInfo) {
        super();
        this.fileTyp = calcFileTyp();
        this.name = name;
        this.appInfo = appInfo;
	}
//	protected ABaseData(final String name, final ApplicationInfo appInfo, final Object specificInfo) {
//        super();
//        this.fileTyp = calcFileTyp();
//        this.name = name;
//        this.appInfo = appInfo;
//        this.specificInfo = specificInfo;
//        this.fileName = calcFileName(appName, specificInfo);
//        this.webAddress = calcWebAddress(getAppName(), specificInfo);
//	}
	
//=============================================================================    

    abstract public    String getShortDescription();
    abstract protected String calcFileTyp();
    abstract protected String calcFileName(final String appName);
    /**if before complete Construction more Info is needed 
     * !needs to be existent befor XTor!
     */
    protected String calcFileName(final String appName, final Object specificInfo){
    	return calcFileName(appName);
    }
    abstract protected String calcWebAddress();
//=============================================================================    
	public String          getName()            {return this.name;}
	public ApplicationInfo getApplicationInfo() {return this.appInfo;}
//	public Object          getSpecificInfo()	{return this.specificInfo;}
	
    public final boolean isOkForWriting(){
        return !LocalHome.noLocalHomeWanted();
    }
    /** @throws IllegalStateException, if not isOkForWriting */
    public final void checkUserOkForWriting() {
    	if(!isOkForWriting()){
    		throw new IllegalStateException("User does not want data to be stored. "
    				+ "You may reconfigure it via the Configurations menu."); 
    	}
    }
    
	public final InputStream getFileInputStream() throws FileNotFoundException {
		return new FileInputStream(getFileName());
	}
	public final InputStream getWebInputStream() throws MalformedURLException, IOException{
        final String webAddress = getWebAddress();
        accessedWebAdresses.add(webAddress);
		final URL url = new URL(webAddress);
        final URLConnection con = url.openConnection();
        final InputStream stream = con.getInputStream();
		return stream;
	}
	/**	@throws IllegalStateException, if User does not allow writing
	 * 	@return true on success, else false
	 */
	public final boolean backupWebToFile() {
		//IDEE Improvements: (1)export to workerthread, (2) mache .Bak.XML Zwischenkopie
		checkUserOkForWriting();//@throws IllegalStateException
		
		String string = null;
		boolean stringOK = false;
		Exception ex = null;
		for(int iTry = 0; iTry< 10 && !stringOK; iTry++){
			try {string = readWebAsString();} catch (final IOException e) {ex = e;}
			if(string != null && string.length() > 0){stringOK = true; break;}
			try {Thread.sleep(100*(iTry+1));} catch (final InterruptedException e){;}
		}
		
		if(!stringOK || ex != null){
			if(ex == null) {ex = new Exception("Problem with backupWebToFile (readWebAsString). Skip it.");}
			Logger.SWFW.warn(ex);
			return false;	//failure
		}
		
		try {
			writeFileAsString(string);
		} catch (final FileNotFoundException e) {
			Logger.SWFW.warn("Problem in Web File Backup (writeFileAsString). Skip it.",e);
			return false;	//failure
		} catch (final IOException e) {
			Logger.SWFW.warn("Problem in Web File Backup (writeFileAsString). Try to skip it.",e);
			return false;	//failure
		}
		
		return true;	//success
	}
	
	/**Buffered read (Web -> local without touching the fileRoot
	 * @throws FileNotFoundException */
	protected String readAsStringWithoutIOManager() throws FileNotFoundException, IOException{
		String result = null;
		if(this.hasWebAddress()){
			try {
				result = readWebAsString();
				final String msg = "Retrieved web hosted ini file. '" 
					             + getWebAddress() + "'.";
		        Logger.SWFW.info(msg);
			} catch (final FileNotFoundException e) {
				final String msg = "Kann die Datei '" + this.getWebAddress() 
						   + "' nicht lesen, versuche es mit dem lokalen Cache."
						   + " Msg: '" + e.getMessage() + "'.";
				Logger.SWFW.warn(msg);
			} catch (final MalformedURLException e) {
				final String msg = "Kann die Datei '" + this.getWebAddress() 
				   		   + " nicht lesen, versuche es mit dem lokalen Cache."
				   		   + " Msg: '" + e.getMessage() + "'.";
				Logger.SWFW.warn(msg);
			} catch (final IOException e) {
				final String msg = "Kann die Datei '" + this.getWebAddress() 
				   		   + " nicht lesen, versuche es mit dem lokalen Cache."
				   		   + " Msg: '" + e.getMessage() + "'.";
				Logger.SWFW.warn(msg);
			}
		}

		if(result != null){
			backupWebAsStringToFile(result);
		}else{
			try {
				result = readFileAsString();
				final String msg = "Retrieved cached ini file. '" 
		             + getFileName() + "'.";
				Logger.SWFW.info(msg);
			} catch (final FileNotFoundException e) {
				final String msg = "Kann die Datei '" + this.getFileName() 
						   + " nicht lesen." 
						   + " Msg: '" + e.getMessage() + "'.";
				Logger.SWFW.warn(msg);
				throw e;
			} catch (final IOException e) {
				final String msg = "Kann die Datei '" + this.getFileName() 
						   + " nicht lesen."
						   + " Msg: '" + e.getMessage() + "'.";
				Logger.SWFW.warn(msg);
				throw e;
			}	
		}
		final FR fileRoot = this.getFileRoot();
		if(fileRoot != null){fileRoot.setContentRoot(result);}
		return result;
	}
	private void backupWebAsStringToFile(final String result) {
		try {
			writeFileAsString(result);
			final String msg = "Backed web hosted ini file. '" 
	             + getWebAddress() + "' to 'local' file cache '" 
	             + getFileName() + "'.";
			Logger.SWFW.info(msg);
		} catch (final FileNotFoundException e) {
			final String msg = "Problem in Web File Backup (writeFileAsString). "
				       + "Skip it." 
					   + " Msg: '" + e.getMessage() + "'.";
			Logger.SWFW.warn(msg);
		} catch (final IOException e) {
			final String msg = "Problem in Web File Backup (writeFileAsString). "
			       + "Skip it." 
				   + " Msg: '" + e.getMessage() + "'.";
			Logger.SWFW.warn(msg);
		}
	}
	
	public final String readWebAsString() throws FileNotFoundException, MalformedURLException, IOException {
		return readInputStreamAsString(getWebInputStream());
	}
	public final String readFileAsString() throws FileNotFoundException, IOException   {
		return readInputStreamAsString(getFileInputStream());
	}
	public final void writeFileAsString(final String string) throws FileNotFoundException, IOException {
		checkUserOkForWriting();//@throws IllegalStateException
		final String fileName = getFileName();
		createAllParentDirs(fileName);
		final boolean renamedFileExists = renameFileToBackupName();
		OutputStream stream = null;
		try{
			stream = new BufferedOutputStream(new FileOutputStream(fileName));
			stream.write(string.getBytes(CHARSET_NAME));
		}catch(final Exception ex){
			Logger.SWFW.warn(this.getClass().getSimpleName() + ".writeFileAsString(...) to " + fileName + " resulted in " + ex);
			if(renamedFileExists) {renameToOriginal();}
		}finally{
			if(stream != null)
				stream.close();
			new File(calcBackupName(fileName)).delete();
		}
	}

//=============================================================================    

	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.ini.IIniFileObserver#getDefaultIniBaumRoot()
	 */
	@Override public Object getDefaultIniBaumRoot() {return null;}
	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.ini.IIniFileObserver#onNewIniAvailable(java.lang.Object)
	 */
	@Override abstract public void onNewIniAvailable(Object iniBaumRoot);//need here to 
	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.ini.IIniFileObserver#onPrepareForExport()
	 */
	@Override public Object onPrepareForExport() {return null;}

//=============================================================================    

    /**never null*/
    final FR getFileRoot() {
		return this.fileRoot;
	}
	final void setFileRoot(final FR fileRoot) {
		if(fileRoot == null){throw new NullPointerException("fileRoot darf nicht null sein");}
		this.fileRoot = fileRoot;
	}


    final String getAppName() {return getApplicationInfo().getAppName();
    }
    final String getFileName() {
        return calcFileName(getAppName());
    }
    final String getFileTyp() { 
        return this.fileTyp;
    }
    final boolean hasFileName(){
        return (getFileName() != null && getFileName().length() > 0);
    }
    final String getWebAddress() {
        //return webAddress;
    	//this.webAddress = calcWebAddress(getAppName(), specificInfo);
    	return calcWebAddress();
    }
    final boolean hasWebAddress(){
        return (getWebAddress() != null && getWebAddress().length() > 0);
    }
    final boolean isWebSrvReachable() throws MalformedURLException {
        final String srvName = WebHome.calcServerName(getWebAddress());
        return WebHome.isReachable(srvName);
    }
    
    final String getReadComment(){
        final String READ_SETTINGS_FROM  = "Read " + getFileTyp() + " from '";
        final String POSTFIX             = "'.";
        if(hasWebAddress()){
            return READ_SETTINGS_FROM + getWebAddress() + POSTFIX;
        }
        if(hasFileName()){
            return READ_SETTINGS_FROM + getFileName() + POSTFIX;
        }
        return "Can not read " + getFileTyp() + ", have no source address.";
    }
    
    final String getWriteComment(){
        final String WRITE_SETTINGS_TO   = "Write " + getFileTyp() + " to ";
        final String POSTFIX             = "'.";
        if(!isOkForWriting()){
            return "Can not write " + getFileTyp() + ", have no permission.";
        }
        if(hasFileName()){
            return WRITE_SETTINGS_TO + getFileName() + POSTFIX;
        }
        return "Can not write " + getFileTyp() + ", have no target address.";
    }

//=============================================================================

    private final void createAllParentDirs(final String fileName) {
    	//VORSICHT nicht die local home routine verwenden, da erstmal einen
    	//Schritt hochgehen
    	final File file = new File(fileName);
    	final File parentDir = file.getParentFile();
    	if (!parentDir.exists()) {
    		parentDir.mkdirs(); // das zugehoerige Verzeichnis koennte auch
    		// nicht ex.
    	}
    }

	private boolean renameFileToBackupName() {
		final String oriName = getFileName();
		return renameFileTo(oriName, calcBackupName(oriName));
	}
	private boolean renameToOriginal() {
		final String oriName = getFileName();
		return renameFileTo(calcBackupName(oriName), oriName);
	}
	private boolean renameFileTo(final String srcName, final String dstName) {
		final File srcFile = new File(srcName);
		if(!srcFile.exists() || srcFile.length() <= 0){
			return false;
		}
//		File dstFile = deleteFileIfExists(dstName);
		return srcFile.renameTo(new File(dstName));
	}
	private String calcBackupName(final String fileName) {
		return fileName + BACKUP_SUFFIX;
	}
	private static String readInputStreamAsString(final InputStream inStream) throws IOException {
		final PushbackInputStream stream = new PushbackInputStream(
											new BufferedInputStream(inStream)
															);
		String retVal = "";
        while(true){
    		if(isAtEndOfFile(stream) ) {break;}//evtl. auf input warten (synchron, ohne Nebenwirkungen)
    		final int n = stream.available();	
    		if(n > 0){//Zahl, der ohne Warten verfuegbaren Bytes 
	        	final byte[] b = new byte[n];
	        	@SuppressWarnings("unused") //result 
 final
	        	int result = stream.read(b);
				retVal += new String(b, CHARSET_NAME);
        	}
        }
        stream.close();
        return retVal;
	}
	/**Nebenwirkungsfrei, synchron d.h. es kann blockieren*/
	private static boolean isAtEndOfFile(final PushbackInputStream stream) throws IOException {
		final int readResult = stream.read();
		final boolean isEOF  = (readResult == -1);
		if(!isEOF) {stream.unread(readResult);}
		return isEOF;
	}
}
