/** 
 *  Mar  5, 2007 - Mar  9, 2007 ws: creation from BaseDataForAnApplication
 *	Aug 29, 2007 - Sep 11, 2007 ws: String appName -> ApplicationInfo appInfo	
 */
package de.desy.mst.libs.framework.mstapp.ini;

import de.desy.mst.libs.framework.mstapp.ApplicationInfo;


/**
 * Basis Spezifikation fuer Applikationsspezifische Files, die explizit vom
 * Entwickler definiert werden. (Nicht die Standard Initialisierung 
 * (siehe: BaseDataForAnApplication)
 * @author ws
 * version 1.0.0
 */
class BaseDataForTheDeveloper<FR extends IIniFileRoot> 
extends ABaseData<FR>{
	public static final String NAME = "EntwicklerDatei";
	public static final String FILETYP = 
    						"Entwicklerspezifische Applications Einstellungen";
    public static final String SHORTDESCRIPTION = FILETYP;
    public static final String FILE_SUFFIX_DUMMY = ""; //Suffix ignored Filenamepart
	private final String developerFileName;
	
//=============================================================================

	/**@throws NullPointerException, if appName or fileName is null.*/
    BaseDataForTheDeveloper(final ApplicationInfo appInfo, final String developerFileName) {
    	super(NAME, appInfo);	
    	if(appInfo == null) 
			{throw new NullPointerException("The appName can not be null.");}
    	if(developerFileName == null) 
    		{throw new NullPointerException("The fileName can not be null.");}
    	this.developerFileName = developerFileName;
    	this.setFileRoot(createFileRoot());
    }
    
//=============================================================================
    /**should only be called after complete construction*/
    /* (non-Javadoc)
     * @see de.desy.mst.libs.framework.mstapp.ini.ABaseData#calcFileName(java.lang.String)
     */
    @Override protected String calcFileName(final String appName) {
        //final String fName = getDeveloperFileName();
		//return calcFileName(appName, fName);
    	final PfadeIni pfadeIni = createPfadeIni();
    	return pfadeIni.getCacheDateiPfad();
    }

    /* (non-Javadoc)
     * @see de.desy.mst.libs.framework.mstapp.ini.ABaseData#calcWebAddress()
     */
    @Override protected String calcWebAddress() {
    	final PfadeIni pfadeIni = createPfadeIni();
    	return pfadeIni.getDateiPfad(); 
	}

    /* (non-Javadoc)
     * @see de.desy.mst.libs.framework.mstapp.ini.ABaseData#calcFileTyp()
     */
    @Override protected String calcFileTyp() {return FILETYP;}//!super XTor Aufr
    /* (non-Javadoc)
     * @see de.desy.mst.libs.framework.mstapp.ini.ABaseData#getShortDescription()
     */
    @Override public String getShortDescription() {return SHORTDESCRIPTION;}
	@SuppressWarnings("unchecked")
	/**most likely you do not want to use it, since your data are of nonstd format*/
	@Deprecated @Override public void onNewIniAvailable(final Object iniBaumRoot) {
		if(iniBaumRoot instanceof IIniFileRoot){
			this.setFileRoot((FR)iniBaumRoot);
		}else if(iniBaumRoot == null){
			//do nothing
		}else{
			final String errMsg = "iniBaumRoot is not of Type '"
						  + IIniFileRoot.class.getName() 
						  + "', but of Type '"
						  + iniBaumRoot.getClass().getName()
						  + "'.";
			throw new IllegalArgumentException(errMsg);
		}
	}
//=============================================================================
	
	public String getDeveloperFileName() {return this.developerFileName;}
	
//=============================================================================

	@SuppressWarnings("unchecked") private FR createFileRoot() {
		return (FR) new IIniFileRoot(){
			private Object content = "";
			//------------------------------------------------------------------
			@Override public Object getContentRoot() 					{return this.content;}
			@Override public void setContentRoot(final Object contentRoot) 	{this.content = contentRoot;}
		};
	} 
	private PfadeIni createPfadeIni() {
		return new PfadeIni(getApplicationInfo(), getDeveloperFileName());
	}
}
