/**
 * ws:	May  8, 2007 - May  8, 2007 creation from scratch
 * ws:	Sep 29, 2008 - Sep 29, 2008	support for extra Protokoll
 */
package de.desy.mst.libs.framework.mstapp.ini;

import java.io.File;

import de.desy.mst.libs.framework.mstapp.logging.appender.file.PfadeProtokoll;
import de.desy.mst.libs.framework.mstapp.preferences.IPrefContainer;
import de.desy.mst.libs.framework.mstapp.preferences.IPreferences;
import de.desy.mst.libs.framework.mstapp.preferences.helper.APrefClientPanel;
import de.desy.mst.libs.framework.mstapp.preferences.helper.APrefClientPanelEinfach;
import de.desy.mst.libs.framework.mstapp.preferences.helper.APreferences;
import de.desy.mst.libs.framework.mstapp.preferences.implementation.PrefManager;

/**
 * Einstellen der lokalen Cache Optionen (Nicht Grafischer Teil)
 * @author ws
 * version 1.0.0
 */
public class PrefCache extends APreferences {
	private static final String   TITLE    = "Cache Einstellungen  (PC Weit)";
	private static final String[] KEYWORDS = {"Cache", "Puffer"};
//-----------------------------------------------------------------------------
	private final PrefManager prefManager;
	private final IPreferences prefAbweichendesProtokoll;
//=============================================================================
	public PrefCache(final PrefManager prefManager, final IPreferences parent) {
		super(TITLE, parent, KEYWORDS);
		this.prefManager = prefManager;
		this.prefAbweichendesProtokoll = new APreferences("Abweichendes Protokoll (PC Weit)", this, KEYWORDS[0], KEYWORDS[1], "Extra Protokoll", "Extra Logging"){
			@Override public APrefClientPanel createPrefClientPanel(IPrefContainer prefContainer) {
				return new PrefCachePanel(prefContainer, getPrefManager(), null){
					private static final long serialVersionUID = 1L;
					@Override String getTFVReadOnlyText(){return super.getTFVReadOnlyText() + PfadeProtokoll.SRV_NAME + File.separator;}
					@Override String getLabelTTText(){return "Unter welchem Verzeichnis ist das 'lokale' Protokoll angelegt (PC Weit und User Spezifisch). Bitte beachten: die Applikationen braucht Schreibrechte.";}
					@Override String getCBText() 	 {return "Spezielles Protokoll Verzeichnis";}
					@Override String getCBTTText() 	 {return "Falls angekreuzt kann ein extra Protokollverzeichnis genommen werden.";}
					@Override boolean isChecked_System(){return LocalHome.hatAbweichendesProtokollVerzeichnis();}
					@Override String getVerzeichnis_System(){return LocalHome.getHomeProtokoll();}
					@Override void setSystem(final String verzeichnis, final boolean isAbweichendesProtokoll){
				    	LocalHome.setHomeProtokoll(verzeichnis, isAbweichendesProtokoll);
				    	initFromSystem();
				    }
					@Override String stripVerzeichnis(String verzeichnis) {
						if(verzeichnis == null)
							return "";
						verzeichnis = stripVerzeichnis(verzeichnis, File.separator);
						verzeichnis = stripVerzeichnis(verzeichnis, PfadeProtokoll.SRV_NAME);
						verzeichnis = stripVerzeichnis(verzeichnis, File.separator);
						return super.stripVerzeichnis(verzeichnis);	// "/mst_app/"
					}
				};
			}

		};
		PrefCachePanel.manageAbweichendesProtokoll(getPrefManager(), getPrefAbweichendesProtokoll());
	}
//=============================================================================
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.preferences.helper.APreferences#createPrefClientPanel(de.desy.mst.libs.framework.mstapp.preferences.IPrefContainer)
	 */
	@Override public APrefClientPanelEinfach createPrefClientPanel(IPrefContainer prefContainer) {
		return new PrefCachePanel(prefContainer, getPrefManager(), getPrefAbweichendesProtokoll());
	}
//=============================================================================
	private PrefManager getPrefManager() {return this.prefManager;}
	private IPreferences getPrefAbweichendesProtokoll() {
		return this.prefAbweichendesProtokoll;
	}
}
