/**
 *  Created on 	Jul 13, 2005 - Jul 13, 2005
 *  			Aug 11, 2005 ... checkCalledByXMLDecoder
 *  			Aug 24, 2005 ... isCalledByClass privat->public, added checkIsCalledByClass
 */
package de.desy.mst.libs.framework.mstapp.ini;

import java.beans.Encoder;
import java.beans.XMLDecoder;

/**
 * @author ws
 * @version 1.0.0
 */
public class XMLUtil {
	
	/**
	 * Filtert eine Property vom rausgeschrieben werden (falls ihr default null ist,
	 * sonst explizit angeben).<br>
	 * Implementation:<br>
	 * Kuckt ob vom XML Reader aufgerufen worden. Falls ja gibt null als
	 * default zurueck sonst den urspruenglichen Wert. Da der XMLEncoder
	 * default werte aus Optimierungsgruenden nicht abspeichert erfuellt diese
	 * Funktion ihren Zweck. Aber es ist gehackt.<br>
	 * Java Beans konforme Alternative:<br>
	 * Zu er Ini Klasse die entsprechende BeanInfo Klasse ausprogrammieren und
	 * den relevanten PropertyDescriptor explizit zu transient setzen:
	 * <code>pd.setValue("transient", Boolean.TRUE);</code> 
	 * @link http://java.sun.com/products/jfc/tsc/articles/persistence4/
	 * @param <E> Type der Property (keine nativen Typen implementiert)
	 * @param property die zu Filternde Property
	 * @return null oder die property, je nach execution context
	 */
	public static <E> E filter(E property){
		return filter(property, null);
	}
	/**
	 * @link siehe andere filter Routine
	 * Dies Routine wird gebraucht falls der property default nicht null ist.
	 * @param <E> 				Type der Property 
	 * @param property			zu filternde Property
	 * @param defaultProperty	default Wert der Property
	 */
	public static <E> E filter(E property, E defaultProperty){
		return isForXMLEncoder() ? defaultProperty : property;
	}
	/**kann wie setValue auch direkt benutzt werden*/
	public static final boolean isForXMLEncoder() {
		return isCalledByClass(Encoder.class.getName())
			|| isCalledByClass(XMLDecoder.class.getName());
	}
	/**@throws UnsupportedOperationException, bei Aufruf (Ausnahme java.beans.Encoder)*/
	public static void checkCalledByXMLDecoder() {
		if(!isForXMLEncoder()){
			throw new UnsupportedOperationException("Zugriff nur dem XMLDecoder erlaubt.");
		}
	}

	public static final boolean isCalledByClass(final String className) {
		Exception exception = new Exception("Dummy Exception only");
		StackTraceElement[] stackTrace = exception.getStackTrace();
		return classNameIsInStackTrace(className, stackTrace);
	}
	/**@throws UnsupportedOperationException, if not called by class of name className*/
	public static void checkIsCalledByClass(String className) throws UnsupportedOperationException {
		if(!isCalledByClass(className)){
			throw new UnsupportedOperationException("Zugriff nur '" + className + "' erlaubt.");
		}
	}

//=============================================================================
	
	private static final boolean classNameIsInStackTrace(final String className, final StackTraceElement[] stackTrace) {
		for (StackTraceElement element : stackTrace) {
			if(classNameIsInStackTraceElement(className, element)) {return true;}
		}
		return false;
	}
	private static final boolean classNameIsInStackTraceElement(final String className, final StackTraceElement element) {
		return className.equals(element.getClassName());
	}
}
