/**
 *  Created on  Oct 24, 2005 - Nov  8, 2005
 *  			Dec 15, 2005 - Dec 15, 2005	update to MstDateFormat.*, Application & PCName ans Ende
 *  			Jan 11, 2006 - Jan 11, 2006 handle runId, pad MessageAndThrowable
 *  			Feb 24, 2006 - Feb 24, 2006	escape '"' and put it all into escapedSubStrings, msgFormat all subStrings
 *  			Aug 07, 2006 - Aug 11, 2006	ws: generalise for ServerFormatte use
 *  			Jun 04, 2007 - Jun 07, 2007	ws: added ignoreThrowable stuff, format(List), getHeader()
 *  			Nov 23, 2007 - Nov 23, 2007 ws: throwableFormat/Uncascaded
 *  			Dec 14, 2007 - Dec 14, 2007 ws: encode(String msg) extracted
 *  											added decode(String encodedMsg)
 *  											Map -> List<StringPair>
 *	Feb 15, 2008 - Feb 15, 2008 ws: added "@Override"es
 *	Jun 10, 2008 - Jun 10, 2008	ws: if(getReduceThrowable()) dann im msgFormat (keine Multiline Msgs im Output File)
 *	Jun 27, 2008 - Jun 27, 2008	ws: protect against nunnull but empty stacktrace in deepest cause
 *	Apr 24, 2009 - Apr 24, 2009	ws:	added:	encodeTheNewLine, trivia
 *	Apr 30, 2009 - Apr 30, 2009	ws: added:	protected static final String HEADER_*
 *									changed handling of StringPair Collection
 *									added:  StringPair equals, hashCode
 *	May 13, 2009 - May 18, 2009	ws:	changed: runIdFormat(runId) robust against null arg
 */
package de.desy.mst.libs.framework.mstapp.logging;

import java.util.Collections;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.UUID;

import de.desy.mst.libs.framework.mstapp.util.IFormatAsString;
import de.desy.mst.libs.framework.mstapp.util.MstDateFormat;

/**
 * dateFormat(), levelFormat() lassen sich in abgeleiteter Klasse leicht
 * ueberschreiben.
 * @author ws
 * @version 1.0.0
 */
/**
 * @author ws
 * @version 1.0.0
 */
public class EinfacherLogFormatter implements ILogFormatter {
	public  static final char   PAD_CHARACTER	= ' ';	//Character zum Right Padden von Msg/Throwable
	public  static final String SEPARATOR_InMSG	= "\t";
	public  static final String LINE_SEPARATOR	= "\n";	//new lines etc will be replaced by LINE_SEPARATOR_ESCAPED

	protected static final String HEADER_NR 		 = "Nr";
	protected static final String HEADER_LEVEL 		 = "Level";
	protected static final String HEADER_DATUM 		 = "Datum";
	protected static final String HEADER_KATEGORIE 	 = "Kategorie";
	protected static final String HEADER_MELDUNG 	 = "Meldung";
	protected static final String HEADER_APPLIKATION = "Applikation";
	protected static final String HEADER_RUN_ID 	 = "RunId";
	protected static final String HEADER_PCNAME 	 = "PCName";

	private   static final String TRACE_SEPARATOR = LINE_SEPARATOR + LINE_SEPARATOR
					   + "Obiger Fehler ist durchgeschlagen auf: " + LINE_SEPARATOR;

	//-------------------------------------------------------------------------
	protected final static class StringPair{
		public final String key;
		public final String value;
		public StringPair(final String key, final String value) {
			super();
			this.key = key;
			this.value = value;
		}
		public StringPair createReversed() {
			return new StringPair(this.value, this.key);
		}
		@Override public String toString() {return "[" + this.key + ", " + this.value + "]";}
		@Override public int hashCode() {
			final int prime = 31;
			int result = 1;
			result = prime * result + ((this.key   == null) ? 0 : this.key  .hashCode());
			result = prime * result + ((this.value == null) ? 0 : this.value.hashCode());
			return result;
		}
		@Override public boolean equals(final Object obj) {
			if (this == obj)
				return true;
			if (obj == null)
				return false;
			if (getClass() != obj.getClass())
				return false;
			final StringPair other = (StringPair) obj;
			if (this.key == null) {
				if (other.key != null) 
					return false;
			} else if (!this.key.equals(other.key))
				return false;
			if (this.value == null) {
				if (other.value != null)
					return false;
			} else if (!this.value.equals(other.value))
				return false;
			return true;
		}
	}
	//-------------------------------------------------------------------------
	protected static final StringPair NEWLINEPAIR = new StringPair("\n", "&nl;");	//only for single line formatting
	private static final List<StringPair> escapedSubStrings;
	private static final List<StringPair> reverseEscapedSubStrings;
	static {
		final LinkedList<StringPair> list = new LinkedList<StringPair>();
		final LinkedList<StringPair> reverseList = new LinkedList<StringPair>();
		add(list, reverseList, "&", "&amp;");
		add(list, reverseList, "\t", "&tab;" );	//(key, value)
	  //add(list, reverseList, "\n", "&nl;"  );	// not anymore "|" due to Consolewindow direct jump to errors feature in eclipse
		add(list, reverseList, "\"", "&quot;");
		add(list, reverseList, NEWLINEPAIR);	//nur temporaer
		list.remove(NEWLINEPAIR);	//nur zum dekodieren
		escapedSubStrings        = Collections.unmodifiableList(list);
		reverseEscapedSubStrings = Collections.unmodifiableList(reverseList);
	}
	/**Subklassen wie ClogTextFormatter haben eigene escapedSubStrings*/
	protected List<StringPair> getEscapedSubStrings() {
		return EinfacherLogFormatter.escapedSubStrings;
	}
	/**Subklassen wie ClogTextFormatter haben eigene escapedSubStrings*/
	protected List<StringPair> getReverseEscapedSubStrings() {
		return EinfacherLogFormatter.reverseEscapedSubStrings;
	}
	protected final static void add(final LinkedList<StringPair> list, final LinkedList<StringPair> reverseList, final String key, final String value) {
		add(list, reverseList, new StringPair(key, value  ));
	}
	protected final static void add(final LinkedList<StringPair> list, final LinkedList<StringPair> reverseList, final StringPair stringPair) {
		list.addLast(stringPair);
		reverseList.addFirst(stringPair.createReversed());
	}
//=============================================================================
	public static EinfacherLogFormatter createNopadNothrowableFormatter() {
		final EinfacherLogFormatter formatter = new EinfacherLogFormatter();
		formatter.setReduceThrowable(true);	//Protokoll ohne Throwables
		formatter.setPadRaster(1);			//Dense Format
		return formatter;
	}
	//deepest cause
	public static String extractUrsache(final Throwable ex) {
		String retVal = null;
		if(ex == null){
			retVal = "";
		} else {
			final Throwable cause = ex.getCause();
			if(cause == null){
				//deepest cause
				final StackTraceElement[] stackTrace = ex.getStackTrace();
				retVal = ex.toString()
				   	   + " at "
				       + ((stackTrace.length > 0) ? stackTrace[0].toString()
				    		   					 : "<<<TRACE EMPTY>>>");
			}else{
				retVal = extractUrsache(cause);	//recursion
			}
		}
		return retVal;
	}
//=============================================================================
	private IFormatAsString<Date> formatDate;
	//public final long   nr;		//local count um Messages nach Erzeugung sortieren zu koennen
	//public final LogLvl level;
	//public final Date	date;
	//public final String applikation;
	//public final String pcName;
	//public final LogKat kategorie;
	//public final String message;
	//public final Throwable throwable;
	private int padRaster = 100;
	private boolean reduceThrowable = false;	//throwable wird ignoriert
//=============================================================================
	public EinfacherLogFormatter() {
		this.formatDate = MstDateFormat.ISO.getIFormatAsString();
	}
//=============================================================================
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.ILogFormatter#getHeader()
	 */
	@Override public String getHeader(final boolean withMultiMessageSeparator) {
		return 					  HEADER_NR			//nrFormat(logMsg.nr)
		 + getSEPARATOR_InMSG() + HEADER_LEVEL		//levelFormat(logMsg.level)
		 + getSEPARATOR_InMSG() + HEADER_DATUM		//getFormatDate().format(logMsg.date)
		 + getSEPARATOR_InMSG() + HEADER_KATEGORIE	//kategorieFormat(logMsg.kategorie)
		 + getSEPARATOR_InMSG() + HEADER_MELDUNG	//msgAndThrowableFormat(logMsg.message, logMsg.throwable)
		 + getSEPARATOR_InMSG() + HEADER_APPLIKATION//applikationFormat(logMsg.applikation)
		 + getSEPARATOR_InMSG() + HEADER_RUN_ID		//runIdFormat(logMsg.runId)
		 + getSEPARATOR_InMSG() + HEADER_PCNAME		//pcNameFormat(logMsg.pcName)
		 + (withMultiMessageSeparator ? getMultiMessageSeparator() : "")
		 ;
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.logging.ILogFormatter#format(de.desy.mst.mstapp.logging.LogMsg)
	 */
	@Override public String format(final LogMsg logMsg) {
		final String msg =
			nrFormat(logMsg.nr)
		  + getSEPARATOR_InMSG() + levelFormat(logMsg.level)
		  + getSEPARATOR_InMSG() + getFormatDate().format(logMsg.date)
		  + getSEPARATOR_InMSG() + kategorieFormat(logMsg.kategorie)
		  + getSEPARATOR_InMSG() + msgAndThrowableFormat(logMsg.message, logMsg.throwable)
		  + getSEPARATOR_InMSG() + applikationFormat(logMsg.applikation)
		  + getSEPARATOR_InMSG() + runIdFormat(logMsg.runId)
		  + getSEPARATOR_InMSG() + pcNameFormat(logMsg.pcName);
		return getReduceThrowable() ? encodeTheNewLine(msg) : msg;
	}

	protected String encodeTheNewLine(final String msg) {
		return encode(msg, NEWLINEPAIR);
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.ILogFormatter#format(java.util.List)
	 */
	@Override public String format(final List<LogMsg> msgList) {
		final StringBuffer buffer = new StringBuffer();
		for (final LogMsg msg : msgList)
			buffer.append(this.format(msg))
				  .append(getMultiMessageSeparator());
		return buffer.toString();
	}

//=============================================================================


	/**never null*/
	public final IFormatAsString<Date> getFormatDate() {return this.formatDate;}
	/**@throws NullPointerException, null*/
	public final void setFormatDate(final IFormatAsString<Date> formatDate) {
		if(formatDate == null){throw new NullPointerException("Das Objekt formatDate darf nicht null sein.");}
		this.formatDate = formatDate;
	}

	public String getSEPARATOR_InMSG() 		{return EinfacherLogFormatter.SEPARATOR_InMSG;}
	public String getLINE_SEPARATOR()		{return EinfacherLogFormatter.LINE_SEPARATOR; }
	public String getMultiMessageSeparator(){return EinfacherLogFormatter.LINE_SEPARATOR;}


	public String nrFormat(final long nr) 						{return Long.toString(nr);}
	public String levelFormat(final LogLvl level) 				{return msgFormat(level.toString());}

	public String applikationFormat(final String application)	{return application;}
	public String runIdFormat(final UUID runId) {return msgFormat("" + runId);}
	public String pcNameFormat(final String pcName) {return pcName;}
	public String kategorieFormat(final Logger logKategorie)	{return msgFormat(logKategorie.toString());}
	public String msgAndThrowableFormat(final String msg, final Throwable throwable) {
		if(throwable == null)							/*kein throwable*/
			return decorate(msgFormat(msg));
		else if((msg == null) || (msg.length() == 0))	/*keine msg*/
			if(getReduceThrowable())
				return decorate(msgFormat(extractUrsache(throwable)));	//throwable als message
			else
				return decorate(throwableFormat(throwable));
		else											/*beides vorhanden*/{
			if(getReduceThrowable()){
				final String string = msg
							  + " [Ursache: "
							  + extractUrsache(throwable)
							  + "]";
				return decorate(msgFormat(string));
			}
			else{
				final String throwableString = throwableFormat(throwable);
				if(throwableString.contains(msg)){return decorate(throwableString);}
				return decorate(msgFormat(msg) + getLINE_SEPARATOR() + throwableString);
			}
		}
	}

	public String throwableFormat(Throwable throwable) {
		//no causes are simple:
		if(throwable.getCause() == null){
			return throwableFormatUncascaded(throwable);
		}

		//put the causes in List with innermost first
		final LinkedList<String> liste = new LinkedList<String>();
		do {
			liste.addFirst(throwableFormatUncascaded(throwable));
			throwable = throwable.getCause();
		} while (throwable != null);

		//now render to single String
		final StringBuffer sb = new StringBuffer();
		for (final String string : liste) {
			if(sb.length() > 0) {
						sb.append(TRACE_SEPARATOR);
			}
			sb.append(string);
		}
		return sb.toString();
	}
	private String throwableFormatUncascaded(final Throwable throwable) {
		StringBuffer tSB = (new StringBuffer(msgFormat(throwable.toString())))
								.append("[")
								.append(getLINE_SEPARATOR());
		for (final StackTraceElement traceElement : throwable.getStackTrace()) {
			tSB = tSB.append(msgFormat(traceElement.toString()))
					 .append(getLINE_SEPARATOR());
		}
		return tSB.deleteCharAt(tSB.length()-1).append("]").toString();
	}

	/**never a null Message, Tabs are replaced*/
	public String msgFormat(String msg) {
		if(msg == null)
			return "";
		msg = encode(msg);
		return msg;
	}

	/**pp for testing Makes out of \t (Tabs) etc. escaped normal characters*/
	String encode(final String msg) {return encode(msg, getEscapedSubStrings());}

	private String encode(final String msg,	final List<StringPair> map) {
		String encodedMsg = msg;
		for(final StringPair pair : map)
			encodedMsg = encode(encodedMsg, pair);
		return encodedMsg;
	}

	private String encode(final String msg, final StringPair pair) {
		return msg.replace(pair.key, pair.value);
	}
	/**pp for testing reverse of encode(String msg*/
	String decode(final String encodedMsg){
		return encode(encodedMsg, getReverseEscapedSubStrings());
	}

	public String decorate(final String string) {
		return pad("\"" + string + "\"");
	}
	public String pad(final String string) {
		final int charsIntoPadRaster = string.length() % getPadRaster();
		if(charsIntoPadRaster <= 0)	{return string;}
		else				{return string + paddedString(getPadRaster() - charsIntoPadRaster);}
	}
	public String paddedString(final int padLength) {
		final StringBuffer buffer = new StringBuffer(padLength);
		for (int i = 0; i < padLength; i++) {
			buffer.append(PAD_CHARACTER);
		}
		return buffer.toString();
	}

	public final int getPadRaster() {return this.padRaster;}
	/**@throws IllegalArgumentException, falls <= 0*/
	public final void setPadRaster(final int padRaster) {
		if(padRaster <= 0){throw new IllegalArgumentException("Das padRaster muss groesser 0 sein.");}
		this.padRaster = padRaster;
	}
	/**Throwable wird bei true als einzeiler sonst als iterativer Stacktrace dargestellt*/
	public void setReduceThrowable(final boolean reduceThrowable) {
		this.reduceThrowable = reduceThrowable;
	}
	/**Throwable wird bei true als einzeiler sonst als iterativer Stacktrace dargestellt*/
	public boolean getReduceThrowable() {return this.reduceThrowable;}


//=============================================================================
//
//	public static String formatDateAsISO(final Date date){
//		//YYYY-MM-DD HH:mm:ss.mmm
//		//Y    m  d  H  M  S  L		/%[n]t$* Kuerzel
//		//F          T				/%[n]t$* Kuerzel
//		String string = String.format("%1$tF %1$tT", date);
//		//System.out.println(string);
//		return string;
//	}
}
