/**
 * Jun 11, 2009 - Jun 18, 2009	ws:	Creation from scratch
 */
package de.desy.mst.libs.framework.mstapp.logging;

import java.util.LinkedList;
import java.util.List;

/**
 * LogMsg Tag (es kann nur genau einen Tag pro TagNamen geben. Runtime enforced)
 * <br> hiermit kann man LogMsg'es auszeichnen um spezielle Effekte zu erzielen
 * <br><br>Z.B.
 * <li> Ins Standard Protokoll schreiben auch wenn der LogLvl zu niedrig waere
 * <li> In eins der temporaeren Protokolle schreiben
 * <li> Auf den Tine Logging/Protokoll Server schreiben.
 * <li> via Nicht(Tag) verhindern, dass etwas ins Protokoll geschrieben wird,
 *  	obwohl LogLvl und Kategorie das vorsehen wuerden
 * @author ws
 * @version 1.0.0
 */
public final class LMTag {
	private static final String INVERSIONPREFIX = "Nicht";
	/**synchronized(LMTag.class)*/
	private static final List<LMTag> instantiatedTags = new LinkedList<LMTag>();
	//-------------------------------------------------------------------------
	/**MstLog soll auf den Tine Logging/Protokoll Server geschrieben werden (muss angeschlossen sein)*/
	public static final LMTag NachSrvTineLog	= new LMTag("NachTineLogSrv");

	/**MstLog soll auf die Standard Logging/Protokoll Datei geschrieben werden (Benutzer muss es erlauben)*/
	public static final LMTag NachDateiStdLog 	= new LMTag("NachDateiStdLog");
	/**MstLog soll NICHT auf die Standard Logging/Protokoll Datei geschrieben werden trotz hohem LogLvl*/
	public static final LMTag NichtNachDateiStdLog 	= NachDateiStdLog.getInvertedTag();

	/**MstLog soll auf die Temporre Logging/Protokoll Datei \\tempstorage\tempdata$\... geschrieben werden (Windows only)*/
	public static final class NachDateiTemp{
		public static final LMTag Ein_Tag		= new LMTag("1_Tage");
		public static final LMTag Zehn_Tage		= new LMTag("10_Tage");
		public static final LMTag Hundert_Tage	= new LMTag("100_Tage");
	}
	//-------------------------------------------------------------------------

	public static  boolean exists(final String tagName) {
		return get(tagName) != null;
	}
	/**@return wenn vorhanden den LMTag, sonst null*/
	public static LMTag get(final String tagName){
		synchronized(LMTag.class) {
			for (final LMTag tag : instantiatedTags)
				if(tag.name.equals(tagName))	//tag.name never null
					return tag;
			return null;
		}
	}
//=============================================================================
//=============================================================================


	private final String name;
	private final boolean isInverted;
//=============================================================================
	/**
	 * @param 	tagName	   ...  Eindeutiger Name != null, != "",
	 * 						    und noch nicht vorhanden,
	 * 							Ein invertierter Tag wird automatisch erzeugt
	 * @throws 	IllegalArgumentException	falls null, empty oder vorhanden
	 */
	public LMTag(final String tagName) throws IllegalArgumentException{
		checkTagName(tagName);
		this.name = tagName;
		this.isInverted = isInverted(tagName);
		synchronized(LMTag.class) {
			instantiatedTags.add(this);
			if(!this.isInverted)
				new LMTag(toggleName(tagName));
		}
	}
	private void checkTagName(final String tagName) throws IllegalArgumentException{
		if(tagName == null) 
			throw new IllegalArgumentException("TagName darf nicht null sein.");
		if(tagName.isEmpty())
			throw new IllegalArgumentException("TagName darf nicht empty sein.");
		if(get(tagName) != null)
			throw new IllegalArgumentException("TagName darf nicht existieren.");
	}
//=============================================================================
	public final String  getName() 	  		{return this.name;}
	public final boolean isInverted() 		{return this.isInverted;}
	public final LMTag   getInvertedTag()	{return get(toggleName(getName()));}
	@Override public String toString() {return getName();}
	@Override public boolean equals(final Object obj){return this == obj;}
	@Override public int     hashCode() 			 {return super.hashCode();}
//=============================================================================
	private boolean isInverted(final String tagName) {
		return tagName.startsWith(INVERSIONPREFIX)
			&& exists(removeInversionPrefix(tagName));//erst not dann inv Tag
	}
	private String toggleName(final String tagName) {
		return 	isInverted() ? removeInversionPrefix(tagName)
							 :    addInversionPrefix(tagName);
	}
	private String addInversionPrefix(final String tagName) {
		return INVERSIONPREFIX + tagName;
	}
	private String removeInversionPrefix(final String tagName) {
		return tagName.substring(INVERSIONPREFIX.length());
	}
}
