/**
 *  Created on  Oct 20, 2005 - Oct 25, 2005
 *  Jan 12, 2006 - Jan 12, 2006	ws:	Dokuverbesserung, is
 *  Apr 20 2007 - ...			ws:	added support for Java JDK Levels
 *  Feb 27 2008 - ...			ws:	added: getAppColor()
 */
package de.desy.mst.libs.framework.mstapp.logging;

import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;

import de.desy.mst.libs.framework.mstapp.util.gui.AppColor;

/**
 * Levels des Mst Logging von nur so bis ernst
 * 	  <li>debug	//0
 * 	  <li>info	//1
 * 	  <li>warn	//2
 * 	  <li>error	//3
 * 	  <li>fatal	//4
 * 
 * @author ws
 * @version 1.0.0
 */
public enum LogLvl {
	  /**LogLvl 0*/debug	 
	, /**LogLvl 1*/info	 
	, /**LogLvl 2*/warn	 
	, /**LogLvl 3*/error	 
	, /**LogLvl 4*/fatal	 
	;
	  private final static Map<Level, LogLvl> levelMap = new HashMap<Level, LogLvl>();
	  
//=============================================================================
		public static LogLvl translateFromJDK(final Level level) {
			if(LogLvl.levelMap.size() == 0) fillLevelMap();
			if(levelMap.containsKey(level)){
				return levelMap.get(level);
			}else{
				final String msg = "Unbekanntes Java JDK LogLevel '" 
					             + level + "' translate to warn.";
				Logger.SWFW.warn(msg);
				return LogLvl.warn;
			}
		}
		
//=============================================================================

  /**Dieses Objekt ist mindestens so ernst oder gleich ernst wie das Argument*/
	  public final boolean istErnsterAls(final LogLvl logLvl){
		  return  this.compareTo(logLvl) >= 0;
	  }
	  /**never null*/
	  public AppColor getAppColor() {
		  if(this.ordinal() < warn.ordinal())
			  return AppColor._Info;
		  if(this.equals(warn))
			  return AppColor._WARNUNG;
		  return AppColor._FEHLER;
	  }
	  
//=============================================================================

	  /**Nach Josef Wilgen*/
	  private static void fillLevelMap() {
			levelMap.put(Level.ALL, LogLvl.debug);
			levelMap.put(Level.FINE, LogLvl.debug);
			levelMap.put(Level.FINER, LogLvl.debug);
			levelMap.put(Level.FINEST, LogLvl.debug);
			levelMap.put(Level.INFO, LogLvl.info);
			levelMap.put(Level.CONFIG, LogLvl.info);
			levelMap.put(Level.WARNING, LogLvl.warn);
	        // This is not exactly correct, since SEVERE rather
	        // maps to FATAL, but JDK does not know ERRORs
	        // which are not fatal.
			levelMap.put(Level.SEVERE, LogLvl.error);
			levelMap.put(Level.OFF, LogLvl.fatal);	  }
}
