/**
 *  Created on	Jan 06, 2006 - Jan 06, 2006
 *  Aug 03, 2006 - Aug 20, 2006	ws: Buffered Messages Implementation
 *  Apr 12, 2007 - ...			ws: added support for XLogHandler
 *  Jan 10, 2008 - Jan 11, 2008	ws: added getDateiAppender(), flush()
 *  								getSystemLogProvider()
 *	Oct 14, 2008 - Oct 14, 2008	ws:	added: setMstApp(mstApp);
 * 	Jun 22, 2008 - Jun 22, 2008	ws:	LogAppenderFile use Factory, temp Files,
 * 	Aug 21, 2009 - Aug 21, 2009	ws: added getFirstLogMsg()
 */
package de.desy.mst.libs.framework.mstapp.logging;

import java.util.List;

import de.desy.mst.libs.framework.mstapp.MstApp;
import de.desy.mst.libs.framework.mstapp.logging.appender.ILogAppender;
import de.desy.mst.libs.framework.mstapp.logging.appender.ILogAppenderKern;
import de.desy.mst.libs.framework.mstapp.logging.appender.file.LogAppenderFile;

/**
 * Um die Logger Enum ein moeglichst einfaches Interface zu geben
 * ist das Management der Appender etc. in einer anderen Klasse.<br>
 * REIN STATISCHE KLASSE
 * @author ws
 * @version 1.0.0
 */
public class LoggerManager {
	private static final int DEFAULT_INITIAL_SYSTEM_BUFFER_CAPACITY = 500;
	private static volatile ILogFormatter defaultShortFormatter = null;

//=============================================================================

	/**Kontrolliert das Schreiben von Log Messages auf die Java Runtime Konsole*/
	public static final ILogAppender   getConsolAppender(){return LoggerImpl.getConsolAppender();}
	/**Kontrolliert das (Default) Schreiben von Log Messages auf das Datei System*/
	public static final LogAppenderFile getDateiAppender(){return LoggerImpl.getDateiAppender();}
	/**Kontrolliert das (Default) Schreiben von Log Messages auf das Windows only Temporary Datei System*/
	public static final List<LogAppenderFile>	getTempDateiAppender()	{
		return LoggerImpl.getTempDateiAppender();
	}

	/** >= 0, package private*/
	static int getSystemBufferInitialCapacity() {
		return DEFAULT_INITIAL_SYSTEM_BUFFER_CAPACITY;
	}
	/**>=0*/
	public static int getSystemBufferCapacity() {
		return LoggerImpl.getSystemBufferCapacity();
	}
	/**@throws IllegalArgumentException, falls systemBufferCapacity kleiner 0*/
	public static void setSystemBufferCapacity(final int systemBufferCapacity) {;
		LoggerImpl.setSystemBufferCapacity(systemBufferCapacity);
	}
	/**never Null, maximum of last getSystemBufferCapacity() Messages ungefiltert*/
	public static List<LogMsg>  getBufferedMessages() {
		return getSystemLogProvider().getMessages();
	}

	/**never null*/
	public static ILogFormatter getDefaultShortFormatter(){
		if(LoggerManager.defaultShortFormatter == null)
					LoggerManager.defaultShortFormatter = new ServerFormatter();
		return LoggerManager.defaultShortFormatter;
		}

	public static ILogProvider getSystemLogProvider()
					{return LoggerImpl.getSystemLogProvider();}
	public static boolean   register  (final ILogAppenderKern appender)
					{return getSystemLogProvider().  register  (appender);}
	public static boolean deregister  (final ILogAppenderKern appender)
					{return getSystemLogProvider().deregister  (appender);}
	public static boolean isRegistered(final ILogAppenderKern appender)
					{return getSystemLogProvider().isRegistered(appender);}

	public static void flush() {LoggerImpl.flush();}

	public static void setMstApp(final MstApp mstApp) {
		LoggerImpl.setMstApp(mstApp);
	}
	public static LogMsg getFirstLogMsg() {return LoggerImpl.getFirstLogMsg();}

//=============================================================================

	/**rein statische Klasse
	 * @throws InstantiationException, falls XTor gerufen wird.
	 */
	private LoggerManager() throws InstantiationException
				{throw new InstantiationException("Rein statische Klasse");}

//=============================================================================

}
