/**
 *  Created on	Oct 24, 2005 - Oct 24, 2005
 *  Feb 01, 2006 - Feb 06, 2006	filter unterstuetzen
 *  Jun 06, 2007 - ...			ws:	XTor auch mit explizitem hauptLogFilter
 *  Oct 14, 2008 - Oct 27, 2008	ws: added:	registerPreference(...)
 *  Jun 24, 2009 - Jun 24, 2009	ws:	addes: toString() default Implementation
 * 	Jul 3, 2009 - Jul 3, 2009	ws:	add implements IWriter<LogMsg>
 */
package de.desy.mst.libs.framework.mstapp.logging.appender;

import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import de.desy.mst.libs.framework.mstapp.logging.LogMsg;
import de.desy.mst.libs.framework.mstapp.logging.filter.HauptLogFilter;
import de.desy.mst.libs.framework.mstapp.logging.filter.StdLogFilter;
import de.desy.mst.libs.framework.mstapp.preferences.IPrefManager;
import de.desy.mst.libs.framework.mstapp.preferences.IPreferences;
import de.desy.mst.libs.framework.mstapp.util.xml.AXmlBeschreibung;

/**
 * Abstract Mst Logging Appender
 * @author ws
 * @version 1.0.0
 */
public abstract class ALogAppender 	extends AXmlBeschreibung
									implements ILogAppender, IWriter<LogMsg> {
	private final HauptLogFilter hauptLogFilter;
//=============================================================================
	public ALogAppender(final HauptLogFilter hauptLogFilter) {
		super();
		this.hauptLogFilter = hauptLogFilter;
	}
	public ALogAppender() {this(new HauptLogFilter());}
//=============================================================================

	/**
	 * msg ist erfolgreich gefiltert und muss nur noch rausgeschrieben werden
	 * @param msg never null und erfolgreich durch die filter gekommen
	 */
	@Override public abstract void doWrite(final LogMsg msg);
//=============================================================================

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.appender.ILogAppender#doLog(java.util.List)
	 */
	@Override public final void doLog(final List<LogMsg> msgList) {
		for (final LogMsg msg : msgList) {
			doLog(msg);
		}
	}


	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.appender.ILogAppender#doLog(de.desy.mst.mstapp.logging.LogMsg)
	 */
	@Override public final void doLog(final LogMsg msg) {
		if(getHauptLogFilter().pass(msg)){
			doWrite(msg);
		}
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.appender.ILogAppender#getFilter()
	 */
	@Override public final StdLogFilter getFilter() {
		return getHauptLogFilter();
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.appender.ILogAppender#getBypassFilter()
	 */
	@Override public final StdLogFilter getBypassFilter() {return getHauptLogFilter().getBypassFilter();}
	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.logging.appender.ILogAppender#isBypassFilterEnabled()
	 */
	@Override public boolean isBypassFilterEnabled() {return getHauptLogFilter().isBypassFilterEnabled();}
	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.logging.appender.ILogAppender#setBypassFilterEnabled(boolean)
	 */
	@Override public void setBypassFilterEnabled(final boolean enabled) {getHauptLogFilter().setBypassFilterEnabled(enabled);}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.appender.ILogAppender#registerPreference(de.desy.mst.libs.framework.mstapp.preferences.IPrefManager, de.desy.mst.libs.framework.mstapp.preferences.IPreferences)
	 */
	@Override public void registerPreference(final IPrefManager prefMgr, final IPreferences parent, final String appenderName) {
//		System.out.println(this.getClass().getSimpleName() 
//				        + ".registerPreference(\"" + prefMgr.getClass().getSimpleName() 
//				        + "\", \"" + parent.getTitle() 
//				        + "\", \"" + appenderName + "\").");
		prefMgr.register(new PrefLogging(appenderName, this, prefMgr, parent));
	}

	@Override public String toString() {return getName();}

//=============================================================================

	protected Element getBeschreibungAsElement(Document document, final String name) {
		if(document == null) document = createDocument();
		if(document == null) {return null;}	//in real trouble on a real minor feature

		final Element element = document.createElement(name);
		element.appendChild(getHauptLogFilter().getBeschreibungAsElement(document));
		//element.appendChild(wrapInNamedElement(document, LOG_FORMATTER, 	getLogFormatter().getBeschreibungAsElement(document)));
		return element;
	}

//=============================================================================
 
	private HauptLogFilter getHauptLogFilter() {return this.hauptLogFilter;}
} 
