/**
 * Oct  7, 2008 - Nov  4, 2008	ws:	Creation from scratch
 */
package de.desy.mst.libs.framework.mstapp.logging.appender;

import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.logging.appender.ILogAppender;
import de.desy.mst.libs.framework.mstapp.preferences.IPrefContainer;
import de.desy.mst.libs.framework.mstapp.preferences.IPrefManager;
import de.desy.mst.libs.framework.mstapp.preferences.IPreferences;
import de.desy.mst.libs.framework.mstapp.preferences.helper.APrefClientPanel;
import de.desy.mst.libs.framework.mstapp.preferences.helper.APreferences;

/**
 * Preferences fuer das MstLogging (package private)
 * @author ws
 * @version 1.0.0
 */
class PrefLogging extends APreferences {
	private static final String   TITLE    = "Filter fr: ";
	private static final String[] KEYWORDS = new String[]
	                                         {"Logging", "Protokoll", "Filter"};
//=============================================================================
	private final ILogAppender appender;
	private final IPrefManager prefMgr;
	private final IPreferences prefLoggingBypass;
//=============================================================================
	/**
	 * @param appender not null
	 * @param parent
	 * @param prefMgr not null
	 * @throws NullPointerException, falls appender oder PrefManager null ist
	 */
	public PrefLogging(final String appenderName, final ILogAppender appender, final IPrefManager prefMgr, final IPreferences parent) {
		super(TITLE + appenderName, parent, KEYWORDS);

		if(prefMgr == null)
			throw new NullPointerException("Der PrefManager darf nicht null sein.");

		this.appender = appender;
		this.prefMgr  = prefMgr;
		this.prefLoggingBypass = new APreferences("Bypass Filter " + TITLE + appenderName, this, KEYWORDS[0], KEYWORDS[1], KEYWORDS[2], "Bypass"){
			@Override public APrefClientPanel createPrefClientPanel(final IPrefContainer prefContainer) {
				return new PrefLoggingPanelByPass(prefContainer, appender);
			}
			@Override public void onNewSaveString(final String saveString) {
				PrefLogging.this.appender.getBypassFilter().setFromSaveString(getTitle(), saveString);
			}
		};
		PrefLoggingPanel.managePrefByPassFilter(appender.getFilter(), prefMgr, this.prefLoggingBypass);
	}
//=============================================================================
	/* (non-Javadoc) 
	 * @see de.desy.mst.libs.framework.mstapp.preferences.helper.APreferences#createPrefClientPanel(de.desy.mst.libs.framework.mstapp.preferences.IPrefContainer)
	 */
	@Override public APrefClientPanel createPrefClientPanel(final IPrefContainer prefContainer) {
		return new PrefLoggingPanel(prefContainer, this.appender, this.prefMgr, this.prefLoggingBypass);
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.preferences.helper.APreferences#onNewSaveString(java.lang.String)
	 */
	@Override public void onNewSaveString(final String saveString) {
		if(saveString == null || saveString.isEmpty() || !saveString.contains(getTitle()))
			return;
		Logger.SWFW.info("PrefLogging.onNewSaveString(" + saveString + ") wird ausgefhrt.");
		this.appender.getFilter().setFromSaveString(getTitle(), saveString);
	}
}

