/**
 * Oct  7, 2008 - Oct 17, 2008	ws:	Creation from scratch
 * Nov  4, 2008 - Nov  4, 2008	ws: von APrefClientPanel2 statt APrefClientPanel ableiten
 */
package de.desy.mst.libs.framework.mstapp.logging.appender;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.text.JTextComponent;

import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.logging.filter.FilterModus;
import de.desy.mst.libs.framework.mstapp.logging.filter.StdLogFilter;
import de.desy.mst.libs.framework.mstapp.preferences.IPrefContainer;
import de.desy.mst.libs.framework.mstapp.preferences.IPrefManager;
import de.desy.mst.libs.framework.mstapp.preferences.IPreferences;
import de.desy.mst.libs.framework.mstapp.preferences.helper.APrefClientPanel2;

/**
 * Main Panel for Preferences
 * @author ws
 * @version 1.0.0
 */
class PrefLoggingPanel extends APrefClientPanel2 {
	private static final long serialVersionUID = 1L;
	//-------------------------------------------------------------------------
    /**pp*/ static void managePrefByPassFilter(final StdLogFilter filter, final IPrefManager prefMgr, final IPreferences prefByPassFilter){
    	if(filter == null || prefMgr == null || prefByPassFilter == null)
    		return;

    	if(filter.getModus() == FilterModus.STANDARD)
    		prefMgr.register(prefByPassFilter);
    	else
    		prefMgr.deRegister(prefByPassFilter);
    }
 //=============================================================================
	private final ILogAppender logAppender;
	private final IPrefManager prefManager;
	private final IPreferences prefLoggingBypass;

	private final JTableForLogFilter table;;
//=============================================================================
	/**
	 * @param prefContainer	not null
	 * @param logAppender	not null
	 * @param prefManager
	 * @param prefLoggingBypass
	 * @throws NullPointerException falls container oder appendex Werte null ist
	 */
	public PrefLoggingPanel(final IPrefContainer prefContainer
						  , final ILogAppender logAppender
						  , final IPrefManager prefManager
						  , final IPreferences prefLoggingBypass) {
		super(prefContainer);

		if(logAppender == null )
			throw new NullPointerException("Der Filter darf nicht null sein.");

		this.logAppender = logAppender;
		this.prefManager = prefManager;
		this.prefLoggingBypass = prefLoggingBypass;

		this.table = createTable();

		this.setLayout(new BorderLayout());
		this.add(new JScrollPane(this.table), 			BorderLayout.CENTER);
		this.add(createTextCmp(logAppender.toString()), BorderLayout.SOUTH);

		systemToGui();
	}
	private JTableForLogFilter createTable() {
		final ActionListener listener = new ActionListener(){
			@Override public void actionPerformed(final ActionEvent evt) {
				updateIsChanged();
			}
		};
		return new JTableForLogFilter(getFilterSystem(), listener);
	}
	private JTextComponent createTextCmp(final String text) {
		final JTextComponent textCmp = new JTextPane();	//Pane wegen Zeilenumbruch
		textCmp.setText(text);
		textCmp.setEditable(false);
		textCmp.setToolTipText(text);
		return textCmp;
	}

//=============================================================================
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.preferences.helper.APrefClientPanel#onDispose()
	 */
	@Override public void onDispose() {}

	//------------------------------------------------------------------------

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.preferences.helper.APrefClientPanel2#getSaveString()
	 */
	@Override public String getSaveString() {
		final String saveString = getFilterSystem().getSaveString(getTitle(), super.getSaveString());
		return saveString;
	}
	//-------------------------------------------------------------------------
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.preferences.helper.APrefClientPanel2#guiToSystem()
	 */
	@Override public void guiToSystem(){
		if(!isChanged())
			return;

		getFilterSystem().set(getFilterGUI());
		PrefLoggingPanel.managePrefByPassFilter(getFilterSystem(), this.prefManager, this.prefLoggingBypass);

		Logger.SWFW.info("PrefLoggingPanel just updated '"
						+ getPrefContainer().getTitle() + "' filter to "
						+ getFilterSystem() + ".");
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.preferences.helper.APrefClientPanel2#systemToGui()
	 */
	@Override public void systemToGui() {getTable().setFilter(getFilterSystem());}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.preferences.helper.APrefClientPanel2#isChanged()
	 */
	@Override public boolean isChanged() {
		final StdLogFilter filterGUI    = new StdLogFilter(getFilterGUI());
		final StdLogFilter filterSystem = new StdLogFilter(getFilterSystem());
		final boolean isChanged = !filterGUI.equals(filterSystem);
		return isChanged;
	}

//=============================================================================

	private final JTableForLogFilter getTable () {return this.table;}
	/**pp*/       ILogAppender getLogAppender () {return this.logAppender;}
	private final StdLogFilter getFilterGUI   () {return getTable().getFilterClone();}
	/**pp*/       StdLogFilter getFilterSystem() {return getLogAppender().getFilter();}
}
//=============================================================================
//=============================================================================
//=============================================================================
final class PrefLoggingPanelByPass extends PrefLoggingPanel {
	private static final long serialVersionUID = 1L;
	PrefLoggingPanelByPass(final IPrefContainer prefContainer, final ILogAppender logAppender) {
		super(prefContainer, logAppender, null, null);
	}
	@Override StdLogFilter getFilterSystem() {return getLogAppender().getBypassFilter();}
}
