/**
 * ws:	Jun 27, 2007 - Jul 10, 2007 creation from LogAppenderFile by Refactor
 * ws:	Nov  1, 2007 - Nov  8, 2007 ws: added: registerEmergencyCleanup()
 *	Feb 15, 2008 - Feb 15, 2008 ws: added "@Override"es
 */
package de.desy.mst.libs.framework.mstapp.logging.appender.file;

import java.util.ArrayList;
import java.util.List;

import de.desy.mst.libs.framework.mstapp.logging.ILogFormatter;
import de.desy.mst.libs.framework.mstapp.logging.LogMsg;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.logging.appender.ALogAppenderFormatted;
import de.desy.mst.libs.framework.mstapp.logging.appender.FlushManager;
import de.desy.mst.libs.framework.mstapp.logging.filter.HauptLogFilter;

/**
 * Synchronisierter gepufferter LogAppender.<br>
 * flush muss aufgerufen werden vor Ende des Objects<br>
 * Output als TextString (insbesondere nicht fuer DBs)<br>
 * falls nicht mehr gebraucht: close() aufrufen.
 * @author ws
 * @version 1.0.0
 */
public abstract class ALogAppenderBuffered extends ALogAppenderFormatted{
	private final static int  MAX_BUFF_BF = 5;//MAXimum BUFFersize Before Flush
	private final static long MAX_MS_BF = -1;//MAXimum time in MS Before Flush

//=============================================================================
	/**synchronised on this (getter, write, flush, finalize)*/
	private final List<LogMsg> buffer = new ArrayList<LogMsg>();
	private final FlushManager flushManager;
//=============================================================================
	public ALogAppenderBuffered(final ILogFormatter  formatter
							  , final HauptLogFilter hauptLogFilter ) {
		super(formatter, hauptLogFilter);
		this.flushManager = new FlushManager(this, MAX_BUFF_BF, MAX_MS_BF);
		registerFinalCleanup();
	}
//=============================================================================
	/**pp, implementieren aber nicht aufrufen (hier wirklich rausschreiben)*/
	abstract void flush(final String completeOutput);
	/**pp, wird vor doWrite(final LogMsg msg) aufgerufen*/
	void beforeWrite(final LogMsg msg){;}
	/**pp, wird nach doWrite(final LogMsg msg) aufgerufen*/
	void afterWrite (final LogMsg msg){;}
//=============================================================================
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.appender.ALogAppender#doWrite(de.desy.mst.libs.framework.mstapp.logging.LogMsg)
	 */
	@Override public synchronized final void doWrite(final LogMsg msg) {
		this.beforeWrite(msg);
			final List<LogMsg> theBuffer = getBuffer();
			theBuffer.add(msg);
		this.afterWrite(msg);
		getFlushManager().fireNewBuffersize(theBuffer.size());
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.appender.ILogAppenderKern#flush()
	 */
	@Override public synchronized final void flush() {
		final List<LogMsg> buffer = getBuffer();
		if(buffer.size() > 0){
			final String completeOutput = getLogFormatter().format(buffer);
			buffer.clear();
			flush(completeOutput);
		}
	}
//=============================================================================
	/**@return the flushManager never null*/
	public final FlushManager getFlushManager() {return this.flushManager;}
	/**Falls nicht mehr gebraucht aufrufen*/
	public final void close(){getFlushManager().close();}
//=============================================================================
	/**never null*/ 
	private final synchronized List<LogMsg> getBuffer() {return this.buffer;}

	/*
	 * Warnung falls flush vor exit vergessen.
	 */
	private synchronized final void registerFinalCleanup(){
		try {
			Runtime.getRuntime().addShutdownHook(
					new Thread(){
						@Override public void run() {
							final int bufferSize = getBuffer().size();
							if(bufferSize > 0){
								final String msg =
								   "Letztes Aufrumen des Logging Buffers "
								 + "mit 'bufferSize > 0' (" + bufferSize + ").";
								Logger.SWFW.info(msg);
								close();
								flush();
							}
						}
					}
			);
		} catch (final IllegalStateException  e) {
			/*im Begriff des Shutdowns*/
			close();
		}
	}
}