/**
 * ws:	Jun  4, 2007 - Aug  6, 2007 creation from scratch
 * ws:	Sep 12, 2007 - Sep 13, 2007 class pp -> public, XTor <-, factory methods
 * ws:	Nov  1, 2007 - Nov  5, 2007 ws: cosmetics
 * ws:	Jan 16, 2008 - Jan 16, 2008 ws: fix Charset Problem
 * ws:	Feb  7, 2008 - Feb  7, 2008 ws: disable flush on IO Error
 *	Feb 15, 2008 - Feb 15, 2008 ws: added "@Override"es
 * 	Oct 15, 2008 - Oct 15, 2008	ws:	getName()
 * 	Jan 16, 2009 - Jan 22, 2009	ws:	improve msgBox text if IOProblems
 * 	Jun 22, 2009 - Jun 24, 2009	ws:	adopt to peer invokations
 * 	Feb 23, 2015 - Feb 23, 2015	ws: flush(final String completeOutput) friendlier io error display
 */
package de.desy.mst.libs.framework.mstapp.logging.appender.file;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.concurrent.atomic.AtomicBoolean;

import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import de.desy.mst.libs.framework.mstapp.ApplicationInfo;
import de.desy.mst.libs.framework.mstapp.logging.EinfacherLogFormatter;
import de.desy.mst.libs.framework.mstapp.logging.ILogFormatter;
import de.desy.mst.libs.framework.mstapp.logging.LogMsg;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.logging.filter.HauptLogFilter;
import de.desy.mst.libs.framework.mstapp.logging.filter.ALogTaggedFilter.TagState;
import de.desy.mst.libs.framework.mstapp.security.MstSecurity;

/**
 * Einfachster File LogAppender nur zur Implementation (package private)<br>
 * Mit Thread Synchronisation<br>
 * Kein File/Path Checking<br>
 * Schreib nur bei flush, sonst buffert er.
 *
 * @author ws
 * @version 1.0.0
 */
public class LogAppenderFile extends ALogAppenderBuffered{
	public static class Factory{
		public static LogAppenderFile createStdDateiAppender(final ApplicationInfo info){
			return __LMTagFilePeer.PNachDateiStdLog.createLogAppenderFile(info);
		}
		public static List<LogAppenderFile> createTempDateiAppender(final ApplicationInfo info){
			final ArrayList<LogAppenderFile> list = new ArrayList<LogAppenderFile>(3);
			for (final __LMTagTempFilePeer peer : __LMTagTempFilePeer.getTempFilePeers())
				list.add(peer.createLogAppenderFile(info));
			return list;
		}
	}
	public  final static String FILE_SUFFIX = PfadeProtokoll.FILE_SUFFIX;
	/**pp*/ final static String NAME = "LogAppenderFile";
 //-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
	private static final String DEFAULT_CHARSETNAME = "UTF-8";
//=============================================================================
//=============================================================================


	/**Hat die "Welcher File ist zu nehmen?" Logik nach msgDate*/
	private final IFileTargetsByDate ftbd;
	private final String filePath;
	private final AtomicBoolean schonGeschrieben	= new AtomicBoolean(false);
	private final AtomicBoolean outputToNullDevice	= new AtomicBoolean(false);

	private File   file;

//=============================================================================
	/**
	 * pp
	 * @param ftbd, may not be null
	 * @param filePath, may not be null
	 * @param formatter, may not be null
	 * @param hauptLogFilter, may not be null
	 * @throws NullPointerException, falls any object = null
	 */
	LogAppenderFile(final IFileTargetsByDate ftbd
				  , final String filePath
				  , final ILogFormatter formatter
				  , final HauptLogFilter hauptLogFilter ) {
		super(formatter, hauptLogFilter);

		checkForNull(ftbd, 				"Der IFileTargetsByDate");
		checkForNull(filePath, 			"Der filePath");
		checkForNull(formatter, 		"Der formatter");
		checkForNull(hauptLogFilter,	"Der hauptLogFilter");

		this.ftbd = ftbd;
		this.filePath = filePath;
		Logger.SWFW.info("Neuer Datei Appender '" + hauptLogFilter.getTag()
												  + "'. Pfad: " + filePath);
	}
	/**
	 * pp
	 * @param filePath, may not be null
	 * @param baseName, may be null (ohne .log Suffix und Date Dekoration)
	 * @throws NullPointerException, falls filePath
	 */
	LogAppenderFile(final String filePath, final String baseName) {
		this(new FTWochentag(baseName)
		   , filePath
		   , EinfacherLogFormatter.createNopadNothrowableFormatter()
		   , new HauptLogFilter()
		    );
	}
	/**
	 * @param applicationInfo not null
	 * @throws NullPointerException, falls applicationInfo == null
	 */
	@Deprecated public LogAppenderFile(final ApplicationInfo applicationInfo) {
		this(__LMTagFilePeer.PNachDateiStdLog.calcPath(applicationInfo), null);
	}
//=============================================================================
	@Override public String toString() {
		return getName() + " "  + createFile().getAbsolutePath();
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.util.xml.IXmlBeschreibung#getBeschreibungAsElement(org.w3c.dom.Document)
	 */
	@Override public final Element getBeschreibungAsElement(final Document document) {
		final Element element = getBeschreibungAsElement(document, NAME);
		element.appendChild(createElementWithTextContent(document, "FileName", getFileName()));
		element.appendChild(createElementWithTextContent(document, "FilePath", getFilePath()));
		return element;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.appender.ILogAppender#getName()
	 */
	@Override public String getName() {return "Datei Log " + getFilter().getTag(TagState.ON);}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.appender.file.ALogAppenderBuffered#flush(java.lang.String)
	 */
	@Override public synchronized void flush(final String completeOutput) {
		if(getOutputToNullDevice())
			return;

		try {
			final File file = getFile();
			handlePossiblyObsoleteFile(file);
			final String header = handlePossiblyNewFile( file);
			final String outString = header + completeOutput;

			//Eigentliches Schreiben
			final boolean append = true;
			final OutputStream fos = new FileOutputStream(file, append);
				final OutputStreamWriter osw = new OutputStreamWriter(fos, DEFAULT_CHARSETNAME);
					final BufferedWriter buf = new BufferedWriter(osw);
						buf.write(outString);
					buf.close();
				osw.close();
			fos.close();
			//und fertig
		} catch (final FileNotFoundException e) {
			final String msg = "LogFileAppender.flush '"
							 + getFile().getPath() + "' nicht gefunden";
			Logger.SWFW.warn(msg);
		} catch (final IOException e) {
			this.setOutputToNullDevice(true); //resign after first IOException
			final String msg = "Schreibe kein Protokoll mehr nach '"
							 + getFile().getPath()
							 + "' von PC " + MstSecurity.get().getHostName() 
							 + " (LogFileAppender.flush)"
							 + " auf Grund von IO Problemen. ";
			Logger.SWFW.error(msg, e);
			SwingUtilities.invokeLater(new Runnable(){
				@Override public void run() {
						JOptionPane.showMessageDialog(null
								, msg + e.getMessage()
								, "Protokoll/Logging Io-Problem"
								, JOptionPane.PLAIN_MESSAGE
								 );
					}
				}
			);
		}
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.appender.file.ALogAppenderBuffered#beforeWrite(de.desy.mst.libs.framework.mstapp.logging.LogMsg)
	 */
	@Override void beforeWrite(final LogMsg msg) {
		if(this.getIFileTargetsByDate().needNewOutput(msg.getDate())){
			this.flush();
			this.getIFileTargetsByDate().setDate(msg.getDate());
			this.file = createFile();
			if(!this.schonGeschrieben.getAndSet(true)){
				final String msgS = "Schreibe ein Protokoll nach: '"
								  + getFilePath()
								  + File.separator
								  + getFileName()
								  + "'";
				Logger.SWFW.info(msgS);
			}
		}else{
			//nothing to do
		}
	}
//=============================================================================
	public File createFile() {
		return new File(getFilePath(), getFileName());
	}
	/**never null, File Object auf die aktuelle Schreibdatei*/
	public final File getFile() {return (this.file == null) ? createFile() : this.file;}

	/**pp, @return FileName ohne filePath, never null*/
	final String getFileName() {
		return getIFileTargetsByDate().getName() + FILE_SUFFIX;
	}

	/**pp, @return filePath ohne FileName (Directory only), never null*/
	final String getFilePath() {return this.filePath;}

	/**pp, @return the IFileTargetsByDate, never null*/
	final IFileTargetsByDate getIFileTargetsByDate()	{return this.ftbd;}
//=============================================================================
	private boolean getOutputToNullDevice() {return this.outputToNullDevice.get();}
	private void setOutputToNullDevice(final boolean b)  {this.outputToNullDevice.set(b);}

	private void checkForNull(final Object object, final String preFix) {
		if(object == null){
			throw new NullPointerException(preFix + " darf nicht null sein");
		}
	}
	private void handlePossiblyObsoleteFile(final File file) {
		if(file.exists()){
			final Date lastModifiedDate = new Date(file.lastModified());
			if(getIFileTargetsByDate().istAlt(lastModifiedDate)){
				file.delete();
				file.getAbsoluteFile();
			}
		} 
	}
	private String handlePossiblyNewFile(final File file) throws IOException {
		if(file.exists()){
			return "";
		}
		handlePossiblyMissingParentDirs(file);
		file.createNewFile();
		final boolean withMultiMessageSeparator = true;
		final String header =
						getLogFormatter().getHeader(withMultiMessageSeparator);
		return header;
	}
	private void handlePossiblyMissingParentDirs(final File file) {
		final File parentDir = new File(file.getParent());
		if(!parentDir.exists()){
			parentDir.mkdirs();
		}
	}
}
