/**
 * Jun 18, 2009 - Jun 23, 2009	ws:	Creation from scratch
 * Dec  3, 2009 - Dec  3, 2009	ws:	allow creation(if possible) of new facilities and subsystems
 */
package de.desy.mst.libs.framework.mstapp.logging.appender.file;

import static de.desy.mst.libs.framework.mstapp.logging.LMTag.NachDateiTemp.Ein_Tag;
import static de.desy.mst.libs.framework.mstapp.logging.LMTag.NachDateiTemp.Hundert_Tage;
import static de.desy.mst.libs.framework.mstapp.logging.LMTag.NachDateiTemp.Zehn_Tage;

import java.io.File;
import de.desy.mst.libs.framework.mstapp.ApplicationInfo;
import de.desy.mst.libs.framework.mstapp.logging.LMTag;
import de.desy.mst.libs.framework.mstapp.logging.filter.FilterModus;
import de.desy.mst.libs.framework.mstapp.logging.filter.HauptLogFilter;

/**
 * Temp File Spezifische Infos
 * @author ws
 * @version 1.0.0
 */
class __LMTagTempFilePeer extends __LMTagFilePeer {
	private static final String rootDir = "\\\\tempstorage\\tempdata$\\";
	private static final String DEFAULT_SUBSYSTEM 	= "Informationsdienste";
	private static final String DEFAULT_FACILITY 	= "auxiliary";
	private static final String PROTOKOLL_DIRNAME	= "mst_app";

	private final static __LMTagTempFilePeer[] tmpFilePeers = 	{
							  new __LMTagTempFilePeer(Ein_Tag,      "1_Tage")
							, new __LMTagTempFilePeer(Zehn_Tage,    "10_Tage")
							, new __LMTagTempFilePeer(Hundert_Tage, "100_Tage")
																};
	//synchrone Titel fuer Zeitgesteuerte Datein:
	private static final long now = System.currentTimeMillis();
//-----------------------------------------------------------------------------
	public static __LMTagTempFilePeer[] getTempFilePeers(){return tmpFilePeers.clone();}

//=============================================================================
	private final String topDir;
//-----------------------------------------------------------------------------
	private __LMTagTempFilePeer(final LMTag tag, final String topDir) throws NullPointerException {
		super(tag);
		this.topDir = topDir;
	}
//-----------------------------------------------------------------------------
	/**Output nur via Tag, Bypass ist ausgeschaltet*/
	@Override public HauptLogFilter createHauptLogFilter() {
		final HauptLogFilter hauptLogFilter = new HauptLogFilter(getTag());
		hauptLogFilter.setBlockAnyLevel(true);
		hauptLogFilter.getBypassFilter().setBlockAnyLevel(true);
		hauptLogFilter.getBypassFilter().setModus(FilterModus.NEVER_PASS);//OFF
		return hauptLogFilter;
	}
	@Override public String calcPath(final ApplicationInfo info) {
		//rootDir                topDir facility  subsystem
		//\\tempstorage\tempdata$\1_Tage\auxiliary\Informationsdienste\AppName\mst_app
		final String bisTopDir = rootDir + this.topDir;
		if(!new File(bisTopDir).exists())	//blindly return the unreachable Path
			return bisTopDir + File.separator + info.getAccelerator()
							 + File.separator + info.getSubSystem()
							 + File.separator + info.getAppName()
							 + File.separator + PROTOKOLL_DIRNAME;

		final String bisFacility  = calcPath(bisTopDir,   info.getAccelerator(), DEFAULT_FACILITY);
		final String bisSubsystem = calcPath(bisFacility, info.getSubSystem(),   DEFAULT_SUBSYSTEM);
		return bisSubsystem + File.separator + info.getAppName()
							+ File.separator + PROTOKOLL_DIRNAME;
	}
	/**try to extant Path by dirName, if it does not exist use dirNameDefault*/
	private String calcPath(final String bisTopDir, final String dirName, final String dirNameDefault) {
		final String newPathName = bisTopDir + File.separator + dirName;
		final File file = new File(newPathName);
		if(file.exists())
			return newPathName;
		try {if(file.mkdirs())
				return newPathName;} 
		catch (final SecurityException ex) {/**ignore*/}
		return bisTopDir + File.separator + dirNameDefault;
	} 
	@Override public IFileTargetsByDate createFileTargetsByDate(){
		return new __FTOncePerAppRun(now);
	}
}
