/**
 * Jun  9, 2009 - Jun 29, 2009	ws:	Creation from scratch
 */
package de.desy.mst.libs.framework.mstapp.logging.filter;

import java.util.concurrent.atomic.AtomicReference;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import de.desy.mst.libs.framework.mstapp.ini.PrefStorage;
import de.desy.mst.libs.framework.mstapp.logging.LMTag;
import de.desy.mst.libs.framework.mstapp.logging.LogMsg;

/**
 * Prioritr lassen sich die StdFilterKriterien LogLvl und Kategorie
 * berschreiben. (Positiv mit tag und negativ mit tag.inverted in der LogMsg).
 * <br>
 * <br>In folgender Reihenfolge geht passOnStandardModus vor:
 * <li>passOnStandardModus, falls
 * 		<br> getTag() != null && logMsg.hasTag(getTag())
 * <li>not passOnStandardModus, falls
 * 		<br> getTag() != null && logMsg.hasTag(getTag().invertiert)
 * <li>else:
 * 		<br>passOnStandardModusIgnoreTag(logMsg)
 * @author ws
 * @version 1.0.0
 */
abstract public class ALogTaggedFilter extends ALogFilter {
	private static final String ATTRIBUTNAME_TAG      = "LogMsgTag";
	private static final String ATTRIBUTNAME_TAGSTATE = "LogMsgTagState";
//-----------------------------------------------------------------------------
	/**Allows to switch original LMTag on, off or invert it*/
	public static enum TagState{
		  ON {
			  @Override final LMTag mapTag(final LMTag oriTag){return oriTag;}
			  @Override final TagState invert(){ return TagState.INVERSE;}
		     }
		, OFF{
			@Override final LMTag mapTag(final LMTag oriTag){return null;}
			  @Override final TagState invert(){ return TagState.OFF;}
		}
		, INVERSE{
			  @Override final LMTag mapTag(final LMTag oriTag)	{
				  return oriTag == null ? null : oriTag.getInvertedTag();
			  }
			  @Override final TagState invert(){ return TagState.ON;}
				 }
		;
	abstract LMTag mapTag(final LMTag oriTag);
	abstract TagState invert();
	}
//=============================================================================
	private final LMTag oriTag;
	private final AtomicReference<TagState> refTagState =
									new AtomicReference<TagState>(TagState.ON);
//-----------------------------------------------------------------------------
	public ALogTaggedFilter(final LMTag originalTag) {this.oriTag = originalTag;}
//-----------------------------------------------------------------------------
	abstract protected boolean passOnStandardModusIgnoreTag(final LogMsg logMsg);
//-----------------------------------------------------------------------------
	/**never Null*/public final TagState getTagState()	{return this.refTagState.get();}
	public final void setTagState(final TagState newState) throws NullPointerException{
		if(newState == null)
			throw new NullPointerException("TagState newState darf nicht null sein.");
		this.refTagState.set(newState);
	}
	/**
	 * @param 	state	must not be null
	 * @return	oriState modified by Tag: positive (ON), inverted (INVERSE), null (OFF)
	 * @throws 	NullPointerException ... if state null
	 */
	public LMTag  getTag(final TagState state) throws NullPointerException{
		return state.mapTag(this.oriTag);
	}
	/**null ok*/public LMTag  getTag()	{return getTag(getTagState());}

	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.logging.filter.ALogFilter#passOnStandardModus(de.desy.mst.mstapp.logging.LogMsg)
	 */
	@Override protected boolean passOnStandardModus(final LogMsg logMsg) {
		final LMTag tag = getTag();
		if(tag == null)
			return passOnStandardModusIgnoreTag(logMsg);

		if(logMsg.hasTag(tag))
			return true;

		if(logMsg.hasTag(tag.getInvertedTag()))
			return false;

		return passOnStandardModusIgnoreTag(logMsg);
	}
	@Override public String getSaveString(final String appenderGruppe, final String saveString) {
		if(getTag(TagState.ON) == null)
			return super.getSaveString(appenderGruppe, saveString);

		final String keyTag		= makeKey(appenderGruppe, ATTRIBUTNAME_TAG);
		final String valueTag	= getTag(TagState.ON).getName();

		final String keyState 	= makeKey(appenderGruppe, ATTRIBUTNAME_TAGSTATE);
		final String valueState = getTagState().name();

		final String saveString0 = super.getSaveString(appenderGruppe, saveString);
		final String saveString1 = PrefStorage.serialize(saveString0, keyTag ,  valueTag);
		final String saveString2 = PrefStorage.serialize(saveString1, keyState, valueState);
		return saveString2;
	}
	/**
	 * @param appenderGruppe dieser spezifische Appender (Std Datei Logging mehrere Dateinamen)
	 * @param saveString	 Serialisierung dieses Filters fuer PrefStorage
	 * @param optionalDefault	 wird nicht beachtet
	 */
	@Override protected void setFromSaveString(final String appenderGruppe, final String saveString, final Object optionalDefault) {
		final String keyTag = makeKey(appenderGruppe, ATTRIBUTNAME_TAG);
		final String valueTag = PrefStorage.extract(saveString, keyTag);
		if(valueTag != null && !valueTag.isEmpty()){
			if(LMTag.get(valueTag) != getTag(TagState.ON))
				throw new IllegalArgumentException("Kann alten Tag " + getTag(TagState.ON)
						+ " nicht nachtrglich ndern zu " + valueTag + ".");

			final String keyState = makeKey(appenderGruppe, ATTRIBUTNAME_TAGSTATE);
			final String valueState = PrefStorage.extract(saveString, keyState);
			setTagState(valueState);
		}
		super.setFromSaveString(appenderGruppe, saveString, null);
	}
	/**error tolerant way, if a problem do nothing*/
	private void setTagState(final String value) {
		if(value == null || value.isEmpty())
			return;	//ignore and stay with original
		try {
			this.setTagState(TagState.valueOf(value));
		} catch (final IllegalArgumentException ex) {
			return;	//ignore and stay with original
		}
	}

	@Override public int hashCode() {
		final int prime = 31;
		int result = super.hashCode();
		result = prime * result
				+ ((this.oriTag == null) ? 0 : this.oriTag.hashCode());
		result = prime * result
				+ this.refTagState.get().hashCode();
		return result;
	}
	@Override public boolean equals(final Object obj) {
		if (this == obj)
			return true;
		if (!super.equals(obj))
			return false;
		if (getClass() != obj.getClass())
			return false;
		final ALogTaggedFilter other = (ALogTaggedFilter) obj;
		if (this.oriTag == null) {
			if (other.oriTag != null)
				return false;
		} else if (!this.oriTag.equals(other.oriTag))
			return false;
		if (!this.refTagState.get().equals(other.refTagState.get()))
			return false;
		return true;
	}
	/**
	 * @param  wegWirfFilter, non null, same Tag as this required
	 * @throws NullPointerException, IllegalArgumentException
	 */
	public void set(final ALogTaggedFilter wegWirfFilter) throws IllegalArgumentException{
		if(this.oriTag != wegWirfFilter.oriTag)
			throw new IllegalArgumentException("Kann den ursprnglichen Tag "
					+ this.oriTag + " nicht ndern in "
					+ wegWirfFilter.oriTag+ ".");
 
		this.setTagState(wegWirfFilter.getTagState());
		super.set(wegWirfFilter);
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.logging.filter.ALogFilter#decorateElement(org.w3c.dom.Document, org.w3c.dom.Element)
	 */
	@Override protected void decorateElement(final Document document, final Element element) {
		final LMTag tag = getTag(TagState.ON);
		if(tag != null){
			element.setAttribute(ATTRIBUTNAME_TAG, 		tag          .getName());
			element.setAttribute(ATTRIBUTNAME_TAGSTATE, getTagState().name   ());
		}
	}
}
