/**
 *  Created on  Aug 9, 2006
 */
package de.desy.mst.libs.framework.mstapp.logging.gui;

import java.awt.Color;
import java.awt.Component;

import javax.swing.DefaultListCellRenderer;
import javax.swing.JList;
import de.desy.mst.libs.framework.mstapp.logging.ILogFormatter;
import de.desy.mst.libs.framework.mstapp.logging.LogLvl;
import de.desy.mst.libs.framework.mstapp.logging.LogMsg;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.logging.ServerFormatter;

/**
 * Renders LogMessages mit Hilfe eines ILogFormatters.<br>
 * Adapter zwischen Logging System (ILogFormatter, LogMsg)<br>
 * <b>Warnung: Alle Daten der List/Combobox muessen vom Typ LogMsg sein</b>
 * @author ws
 */
class JListLogCellRenderer extends DefaultListCellRenderer {
	private static final long serialVersionUID = 1L;
	
	private ILogFormatter 	logFormatter	= new ServerFormatter();
	private LogLvl 			emphasizeLevel	= LogLvl.warn; 
	private Color			emphasizeColor  = Color.RED;
	
//=============================================================================
	
	/**not null*/ public ILogFormatter getLogFormatter() {return this.logFormatter;}
	/**not null*/ public void setLogFormatter(final ILogFormatter logFormatter) {
		if(logFormatter == null) {
			Logger.SWFW.warn("Seltsamer gebrauch von setLogFormatter (LogFormatter null) ignore.", new NullPointerException());
			return;
		}
		this.logFormatter = logFormatter;
	}
	
	/**value muss vom Typ LogMsg sein*/
	@Override
	public Component getListCellRendererComponent(final JList<?> list, Object value, final int index, final boolean isSelected, final boolean cellHasFocus) {
		if(value instanceof LogMsg) {
			final LogMsg logMsg = (LogMsg)value;
			value = getLogFormatter().format(logMsg);
			final Component component = super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
			if(logMsg.level.istErnsterAls(getEmphasizeLevel())){
				component.setForeground(getEmphasizeColor());
			}
			return component;
		}else{
			return super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
		}
	}
	/**not null*/ public LogLvl getEmphasizeLevel() {return this.emphasizeLevel;}
	/**not null*/ public void setEmphasizeLevel(final LogLvl emphasizeLevel) {
		if(emphasizeLevel == null) {
			Logger.SWFW.warn("Seltsamer Gebrauch von emphasizeLevel (ist null) ignore.", new NullPointerException());
			return;
		}
		this.emphasizeLevel = emphasizeLevel;
	}
	/**not null*/ public Color getEmphasizeColor() {return this.emphasizeColor;}
	/**not null*/ public void setEmphasizeColor(final Color emphasizeColor) {
		if(emphasizeColor == null){
			Logger.SWFW.warn("Seltsamer Gebrauch von emphasizeColor (ist null) ignore.", new NullPointerException());
		} 
		this.emphasizeColor = emphasizeColor;
	}
	
}
