/**
 *  Created on  Aug 14, 2006 - Aug 21, 2006	ws: created from javax.swing.DefaultListModel
 */
package de.desy.mst.libs.framework.mstapp.logging.gui;

import java.util.List;

import javax.swing.AbstractListModel;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import de.desy.mst.libs.framework.mstapp.util.datastructures.LastNObjects;
import de.desy.mst.libs.framework.mstapp.util.xml.AXmlBeschreibung;
import de.desy.mst.libs.framework.mstapp.util.xml.IXmlBeschreibung;


/**
 * DatenModel der JList fuer das Logging System<br>
 * Implementation mit LastNObjects<br>
 * Geeignet fuer alle LastNObject Darstellungen<br>
 * @author ws
 *
 */
@SuppressWarnings({"rawtypes"})
class LogMsgListModel<E> extends AbstractListModel	{
	public  static final int  	DEFAULT_SIZE     = 100;
	public  static final String	NAME = "LogMsgListModel";
	private static final long serialVersionUID = 1L;
	private LastNObjects<E> buffer;
	public IXmlBeschreibung BESCHREIBUNG = new AXmlBeschreibung(){
		@Override public Element getBeschreibungAsElement(Document document) {
			if(document == null) document = createDocument();
			if(document == null) {return null;}	//in real trouble on a real minor feature
			
			final Element element = document.createElement(NAME);
			element.setAttribute("size", 		Long.toString(getSize()));
			element.setAttribute("capacity", 	Long.toString(getCapacity()));
			return element;
		}
		
	};
	
//=============================================================================	

	public LogMsgListModel(final int size) {
		super();
		this.buffer = new LastNObjects<E>(size);
	}
	public LogMsgListModel() {this(DEFAULT_SIZE);}

//=============================================================================	

	@Override synchronized public int 	getSize() 						{return this.buffer.getSize();}
	@Override synchronized public E		getElementAt(final int index  )	{return this.buffer.getElementAt(index);}
	          synchronized public int 	getCapacity() 					{return this.buffer.getCapacity();}  
	          synchronized public void 	setCapacity	(final int newCapacity) {
		final int oldCapacity = this.buffer.getCapacity();
		if(oldCapacity == newCapacity){
			return;	//nichts zu tun
		}
		this.buffer = new LastNObjects<E>(newCapacity, this.buffer);
		if      (oldCapacity > newCapacity) {
			fireIntervalRemoved(this, newCapacity, oldCapacity - 1);
		}
		else if (oldCapacity < newCapacity) {
			fireIntervalAdded  (this, oldCapacity, newCapacity - 1);
		}
	}

	synchronized public void addElement(final E obj) {
		final boolean sizeChanged = this.buffer.addElement(obj); 
		final int index = this.buffer.getSize() - 1;
		if(sizeChanged){
			fireIntervalAdded(this, index, index);
		}else{
			fireContentsChanged(this, 0, index);
		}
	}

	/**der buffer wird vollstaendig geleert, die Capacity aber erhalten*/
	synchronized public void clear() {
		final int size = this.getSize();
		if(size <= 0){
			return; //nichts zu tun
		}
		this.buffer = new LastNObjects<E> (this.getCapacity());
		fireIntervalRemoved(this, 0, size - 1);
	}
	public List<E>  getEntries() 		{return this.buffer.getEntries();}
	public E getEntry(final int index) 	{return this.buffer.getElementAt(index);}
}
