/**
 * Jan  9, 2008 - Jan 16, 2008	ws:	Creation from scratch/LoggerJFrame extract
 * Jan 28, 2008 - Jan 28, 2008	ws: createSwingLoggerManager with mstApp in ParamList
 */
package de.desy.mst.libs.framework.mstapp.logging.gui;

import java.awt.BorderLayout;
import java.awt.Cursor;

import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import de.desy.mst.libs.framework.mstapp.IMstAppKernel;
import de.desy.mst.libs.framework.mstapp.IOnExit;
import de.desy.mst.libs.framework.mstapp.Main;
import de.desy.mst.libs.framework.mstapp.logging.ILogProvider;
import de.desy.mst.libs.framework.mstapp.logging.LogMsg;
import de.desy.mst.libs.framework.mstapp.logging.filter.HauptLogFilter;

/**
 * Enthaelt ein jPanel fuer das Logging Display
 * @author ws
 * @version 1.0.0
 */
@SuppressWarnings({"rawtypes"})
class LoggingJPanel extends JPanel implements IOnExit{
	private static final long serialVersionUID = 1L;
	//-------------------------------------------------------------------------
	private final JList     jListLoggingEntries;
	private final SwingLoggerManager swingLoggerManager;  
	private final JPanel_LogMsg_MitLabels jPanel_LogMsg_MitLabels;
	private final JPanel_LogMsg_MitTable jPanel_LogMsg_MitTable;
	
//=============================================================================
	/**@throws NullPointerException, falls ein Argument null ist*/
	public LoggingJPanel(final IMstAppKernel mstApp, final ILogProvider logProvider) {
		super();
		final Cursor oldCursor = getCursor();
		setCursor(new Cursor(Cursor.WAIT_CURSOR));
			if( mstApp == null)
				throw new NullPointerException("IMstAppKernel may not be null.");
			if( logProvider == null)
				throw new NullPointerException("ILogProvider may not be null.");
	
			this.jPanel_LogMsg_MitLabels = new JPanel_LogMsg_MitLabels();
			this.jPanel_LogMsg_MitTable  = new JPanel_LogMsg_MitTable();
			this.swingLoggerManager = createSwingLoggerManager(mstApp, logProvider);
			this.jListLoggingEntries = createJListLoggingEntries();
	
			setLayout(new BorderLayout());
				add(createJPanelLoggingEntries(), BorderLayout.CENTER);
				add(createJPanelLoggingMessage(), BorderLayout.SOUTH);
				add(getJPanelLogMsgMitTable(),    BorderLayout.NORTH);
				
			onMessageSelected();
		setCursor(oldCursor);
}
	
//=============================================================================
	
		public void dispose() {
			getSwingLoggerManager().deregister(this.jListLoggingEntries);
			Main.deRegisterForOnExit(this);
		}
		@Override public void onExit() {dispose();}

//=============================================================================
				
		
	final JPanel createJPanelLoggingEntries() {
		final JPanel jPanelLoggingEntries = new JPanel();
		jPanelLoggingEntries.setLayout(new BorderLayout());
		jPanelLoggingEntries.add(createJScrollPaneEntries(), BorderLayout.CENTER);
		return jPanelLoggingEntries;
	}
	private final JScrollPane createJScrollPaneEntries() {
		final JScrollPane jScrollPaneEntries = new JScrollPane();
		jScrollPaneEntries.setViewportView(getJListLoggingEntries());
		return jScrollPaneEntries;
	} 
	final JPanel createJPanelLoggingMessage() {
		final JPanel jPanelLoggingMessage = new JPanel();
		jPanelLoggingMessage.setLayout(new BorderLayout());
		jPanelLoggingMessage.add(getJPanelLogMsg(), BorderLayout.NORTH);
		return jPanelLoggingMessage;
	}
	private JList createJListLoggingEntries() {
		final JList jListLoggingEntries = new JList();
		jListLoggingEntries.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
		jListLoggingEntries.setName("Liste der Logging Messages");
		jListLoggingEntries
				.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
					@Override public void valueChanged(final javax.swing.event.ListSelectionEvent e) {
						onMessageSelected();
					}
				});
		getSwingLoggerManager().register(jListLoggingEntries);
		Main.registerForOnExit(this);
		return jListLoggingEntries;
	}
	private SwingLoggerManager createSwingLoggerManager
				(final IMstAppKernel mstApp, final ILogProvider logProvider) {
		final int capacity = 500;
		final SwingLoggerManager swingLoggerManager =
									new SwingLoggerManager(mstApp
														 , capacity
														 , new HauptLogFilter()
														 , logProvider);
		swingLoggerManager.setEnsureLastElementIsVisible(true);
		return swingLoggerManager;
	}

	
	private void onMessageSelected() {
		final LogMsg msg = getSwingLoggerManager().getSelectedMessage();
		LoggingJPanel.this.onMessageSelected(msg);
	}
	private final void onMessageSelected(final LogMsg msg){
//		Logger.SWFW.info("Neue Message ausgewaehlt: " + new ServerFormatter().format(msg));
//		jLabelLoggingMessage.setText(new ServerFormatter().format(msg));
		getJPanelLogMsg().setMsg(msg);
		getJPanelLogMsgMitTable().setMsg(msg);
	}

	private final SwingLoggerManager getSwingLoggerManager(){
		return this.swingLoggerManager;
	}
	private JPanel_LogMsg_MitLabels getJPanelLogMsg() {
		return this.jPanel_LogMsg_MitLabels;
	}
	private JPanel_LogMsg_MitTable getJPanelLogMsgMitTable() {
		return this.jPanel_LogMsg_MitTable;
	}
	private final JList getJListLoggingEntries() {return this.jListLoggingEntries;}
}
