/**
 * 	Aug 11, 2008 - Aug 25, 2008	ws:	Creation from scratch
 * 	Oct  6, 2008 - Oct  6, 2008	ws:	added getBezeichnung
 *	Jan 15, 2009 - Jan 15, 2009	ws:	serializable
 *	Jul 23, 2009 - Jul 23, 2009	ws:	changed List<L> getMessages() -> IRList<L> ... 
 */
package de.desy.mst.libs.framework.mstapp.logging.gui;

import java.io.File;
import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.desy.mst.libs.framework.mstapp.logging.gui.AFieldFilterGUIFactory.AsJTextArea;
import de.desy.mst.libs.framework.mstapp.util.MstParser;

/**
 * Fuer Tine Text Logs
 * <br> Tine Server Logs siehe LogSource_Clog
 * @author ws
 * @version 1.0.0
 */
class TineStdLogSource extends ALogSource_File<TineStdLogRow, TineStdLogMsgModel> {
	public static TineStdLogSource create(final File file) throws IOException {
		return create(ALogSource_File.calcInputLines(file), file);
	}
	public static TineStdLogSource create(final String[] inputLines) {
		return create(inputLines, (File) null);
	}
	private static TineStdLogSource create(final String[] inputLines, final File file) {
		return new TineStdLogSource(TineStdLogRow.createList(inputLines), file);
	}

//-----------------------------------------------------------------------------
	private List<TineStdLogRow> msgs;
//-----------------------------------------------------------------------------
	private TineStdLogSource(final List<TineStdLogRow> msgs, final File file) {
		super(new TineStdLogMsgModel(), file);
		this.msgs = Collections.unmodifiableList(msgs);
	}
//-----------------------------------------------------------------------------
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.gui.ALogSource#getMessages()
	 */
	@Override public __IRList<TineStdLogRow> getMessages() {return new __ListAdapt<TineStdLogRow>(this.msgs);}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.gui.ALogSource#hasMassiveDecodingProblems()
	 */
	@Override public boolean hasMassiveDecodingProblems() {
		final __IRList<TineStdLogRow> messages = getMessages();
		int countBad = 0;
		for (final SplitTextRow row : messages) {
			if(row.hasDecodingProblem())
				countBad++;
		}
		return countBad != 0 && countBad > 0.1 * messages.size();
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.gui.ALogSource#reloadAllData()
	 */
	@Override public void reloadAllData() throws IOException { 
		final File file = getFile();
		if(file != null) {
			reloadAllData(file);
		}
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.gui.ALogSource#toString()
	 */
	@Override public String toString() {
		final File file = getFile();
		return "Tine Std Log Source ("
		 + ((file == null) ? "" : "', Datei = '" + file.getAbsolutePath())
		 + "') " + System.identityHashCode(this);
	}
	@Override public String getBezeichnung() {
		return getBezeichnung("Tine Std Log");
	}

//-----------------------------------------------------------------------------
	/**@throws NullPointerException for null file*/
	private void reloadAllData(final File file) throws IOException {
		this.msgs = TineStdLogRow.createList(calcInputLines(file));
	}
}//END class TineStdLogSource

//=============================================================================
//=============================================================================
//=============================================================================

class TineStdLogRow extends SplitTextRow{
	public static List<TineStdLogRow> createList(final String[] inputLines) {
		final List<TineStdLogRow> list = new ArrayList<TineStdLogRow>(inputLines.length);
		for (final String line : inputLines) list.add(new TineStdLogRow(line));
		return list;
	}
	private static final String[] split(final String line) {
		final String[] retVal = new String[3];
		retVal[2] = line;	//default
		if(line != null){
			if(line.contains("["))
				splitOnSquareBrackets(line, retVal);
			else if(line.contains(" : "))
				splitOnColon(line,retVal);	//cdi.log structure
		}
		return MstParser.normalize(retVal);
	}
	private static void splitOnSquareBrackets(final String line, final String[] retVal) {
		final int begin1 = line.indexOf("[");
		if(begin1 > -1){
			retVal[0] = line.substring(0, begin1);
			final int end1 = line.indexOf("]", begin1) + 1;
			if(end1 > 0){
				retVal[1] = line.substring(begin1, end1);
				retVal[2] = line.substring(end1);
			}else{
				retVal[1] = "";
				retVal[2] = line.substring(begin1);
			}
		}
	}
	private static void splitOnColon(final String line, final String[] retVal) {
		final int begin1 = line.indexOf(" : ");
		if(begin1 > -1){
			retVal[0] = line.substring(0, begin1);
			retVal[1] = "";
			retVal[2] = line.substring(Math.min(begin1 + 3, line.length() - 1));
		}
	}

//-----------------------------------------------------------------------------
	public TineStdLogRow(final String line) {super(TineStdLogRow.split(line));}
	//-----------------------------------------------------------------------------
	@Override public String getName() {return "TineStdLogRow";}
}//END class TineStdLogRow

//=============================================================================

class TineStdLogMsgModel extends AMsgModel<TineStdLogRow>{
	private static Map<String, ? extends AField<?, TineStdLogRow>> createMap() {
		final Map<String, TineStdLogField<?>> map = new HashMap<String,  TineStdLogField<?>>();
		map.put(TineStdLogField. DATE_FIELD.getName(), TineStdLogField. DATE_FIELD);
		map.put(TineStdLogField.CLASS_FIELD.getName(), TineStdLogField.CLASS_FIELD);
		map.put(TineStdLogField.  MSG_FIELD.getName(), TineStdLogField.  MSG_FIELD);
		return Collections.unmodifiableMap(map);
	}
//-----------------------------------------------------------------------------
	TineStdLogMsgModel() {super(createMap());}
//-----------------------------------------------------------------------------
	@Override public AField<?, TineStdLogRow> getCoreInfoField() {
		return TineStdLogField.MSG_FIELD;
	}

	@Override public AField<Date, TineStdLogRow> getDateField() {
		return null;	//im Augenblick erstes Feld nicht konvertiert
	}
}//END class TineStdLogMsgModel

//=============================================================================

class TineStdLogField<E extends Serializable> extends AField<E, TineStdLogRow >{
	private static final long serialVersionUID = 1L;

	static final TineStdLogField<String>  DATE_FIELD;
	static final TineStdLogField<String> CLASS_FIELD;
	static final TineStdLogField<String>  MSG_FIELD;
	static{
		final boolean isWideEditor = true;
		 DATE_FIELD = new TineStdLogField<String>(0, "Datum",   "Datum im Original Textformat", String.class, "22.07.08 16:41:49.316 CDT", 	!isWideEditor);
		CLASS_FIELD = new TineStdLogField<String>(1, "[.*]" ,   "[.*]", 						String.class, "[IPLRF.3]", 					!isWideEditor);
		  MSG_FIELD = new TineStdLogField<String>(2, "Meldung", "Meldung", 						String.class, "Eine Meldung wwwwwwwwwwwwww", isWideEditor);
	}
	private static AsJTextArea<String, TineStdLogRow, TineStdLogMsgModel> createFFFactory(final String name, final String description, final boolean isWideEditor) {
		return new AFieldFilterGUIFactory.AsJTextArea<String, TineStdLogRow, TineStdLogMsgModel>(name, description, isWideEditor);
	}
	//-----------------------------------------------------------------------------
	public TineStdLogField(final int index, final String name, final String description, final Class<E> clazz, final E valueForPrefSize, final boolean isWideEditor) {
		this(index, name, description, clazz, valueForPrefSize, createFFFactory(name, description, isWideEditor));
	}
	@SuppressWarnings({ "rawtypes", "unchecked" })
	private TineStdLogField(final int index, final String name, final String description, final Class<E> clazz, final E valueForPrefSize, final AFieldFilterGUIFactory filterFactory) {
		super(index, name, description, clazz, valueForPrefSize, filterFactory);
	}
//-----------------------------------------------------------------------------
	@SuppressWarnings("unchecked")
	@Override E get(final TineStdLogRow logRow) {return (E)logRow.get(getIndex());}
}//END class TineStdLogRow


