/**
 * 	Jul 14, 2016 - Jul 19, 2016	ws:	created from scratch
 */
package de.desy.mst.libs.framework.mstapp.logging.gui;

import java.awt.event.ActionListener;
import java.util.concurrent.atomic.AtomicLong;
import javax.swing.Timer;

/**
 * <li>I have or have not GUI update requests in a sequence
 * <li>I need a minimum update interval (other wise the mouse might get stolen)
 * <li>Calling an unneeded update is basically harmless
 * <li>Missing an update is basically harmless
 * <li>I need to do the same regardless of nature and number of update requests accumulated
 * <li>The updates need to run in the EDT Thread
 * <li>I need threadsafety
 * @author ws
 */
class _GUIUpdateThrottle {
	public static final int DEFAULTMINUPDATEINTERVAL = 500;	//500milli seconds
	//=============================================================================================
	public final int minUpdateIntervalMS;
	public final ActionListener update;
	//---------------------------------------------------------------------------------------------
	private final AtomicLong refLastUpdateMS = new AtomicLong(0L);
	private volatile boolean needFreshTimer = true;
	//---------------------------------------------------------------------------------------------
	_GUIUpdateThrottle(final Runnable update) {this(update, DEFAULTMINUPDATEINTERVAL);}
	_GUIUpdateThrottle(final Runnable update, final int minUpdateIntervalMS) {
		this.update = wrap(update);
		this.minUpdateIntervalMS = minUpdateIntervalMS;
	}
	//---------------------------------------------------------------------------------------------
	/**request a potentially throttled update*/
	public final void update() {
		if(this.needFreshTimer)
			scheduleUpdate(this.update);
	}
	//---------------------------------------------------------------------------------------------
	private void scheduleUpdate(final ActionListener update) {
		this.needFreshTimer = false;	//this is the fresh one
		final Timer swingTimer = new Timer(calcDelay(), update);
		swingTimer.setRepeats(false);	//one shot only
		swingTimer.start();
	}
	private int calcDelay() {
		return calcDelay(this.refLastUpdateMS.get()
				       , this.minUpdateIntervalMS
				       , System.currentTimeMillis()
						);
	}
	@Deprecated /**@deprecated For Test only*/ 
	static int calcDelay(final long lastUpdateMS
					   , final long minUpdateIntervalMS
					   , final long currentTimeMillis) {
		final long lDelay = (lastUpdateMS + minUpdateIntervalMS) - currentTimeMillis;
		return lDelay < Integer.MIN_VALUE ? 0 :(lDelay > Integer.MAX_VALUE ? Integer.MAX_VALUE 
		                                		                           : (int)lDelay      );
	}
	/**add the bookkeeping*/
	private final ActionListener wrap(final Runnable update) {
		return  e -> {//next updates should be timed with a fresh timer
					  _GUIUpdateThrottle.this.needFreshTimer = true;
					  _GUIUpdateThrottle.this.refLastUpdateMS.set(System.currentTimeMillis());
					  update.run();	
					 };
	}
}
