/**
 * 	Jun  2, 2009 - Jul  3, 2009	ws:	Creation from scratch
 * 	Nov  2, 2009 - Nov  2, 2009	ws: instrument to see if part of longActionDeadlock
 */
package de.desy.mst.libs.framework.mstapp.logging.map_tinelogsrv;

import java.util.List;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicLong;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import de.desy.mst.libs.framework.mstapp.logging.LMTag;
import de.desy.mst.libs.framework.mstapp.logging.LogLvl;
import de.desy.mst.libs.framework.mstapp.logging.LogMsg;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.logging.appender.ALogAppender;
import de.desy.mst.libs.framework.mstapp.logging.appender.IWriter;
import de.desy.mst.libs.framework.mstapp.logging.filter.HauptLogFilter;
import de.desy.mst.libs.framework.mstapp.logging.filter.StdLogFilter;
import de.desy.mst.libs.framework.mstapp.util.threads.Sleep;

/**
 * Appender um standard MstLog's im Tine Logging/Protokoll Server abzulegen
 * <br> Es werden HeartBeat System Messages am Anfang, Ende und dazwischen
 * im HeartBeatIntervall geschrieben.
 * @author ws
 * @version 1.0.0
 * @param <a>
 */
class ___TineLogAppender extends ALogAppender{
	private final static String NAME = "TineLogAppender";
	private static final long EINTAG_IN_MS = 1000L * 3600L * 24L;

	private static final String HB_THREADNAME = "TineLogAppenderHeartBeat";
	private static final long   HB_DEFAULT_INTERVALL_MS = EINTAG_IN_MS;
	private static final long   HB_LOOP_WAIT_TOCHECK_MS = 1000L;
	private static HauptLogFilter createTineHauptLogFilter() {
		final HauptLogFilter hauptLogFilter = new HauptLogFilter(Logger.Any,  LogLvl.fatal, LMTag.NachSrvTineLog);
		hauptLogFilter.getBypassFilter().set( new StdLogFilter  (Logger.SWFW, LogLvl.error, LMTag.NachSrvTineLog));
		return hauptLogFilter;
	}

//=============================================================================
	private final List<IWriter<ClsLogInfo>> writers;
	private final ___MapEntireMsg mapper;
	private final __SystemInfoFactory systemInfoFactory;
	private final __HeartBeat heartBeat;
	private final AtomicBoolean refIsInShutdown = new AtomicBoolean(false);
//=============================================================================
	/**
	 * @param logMsg    not null
	 * @param writers 	not null
	 * @throws NullPointerException, if any paremeter null
	 */
	___TineLogAppender(final LogMsg logMsg, final List<IWriter<ClsLogInfo>> writers) throws NullPointerException{
		this(logMsg, HB_DEFAULT_INTERVALL_MS, writers);
	}
	/**
	 * @param logMsg       			not null
	 * @param deltaHBMS	   			heartBeat Intervall in Milli Sekunden
	 * @param infoAppender 			not null
	 * @throws NullPointerException, if any paremeter null or list contains null elements
	 */
	___TineLogAppender(final LogMsg logMsg, final long deltaHBMS, final List<IWriter<ClsLogInfo>> writers) throws NullPointerException{
		super(createTineHauptLogFilter());
		if(writers == null)
			throw new NullPointerException("writers darf nicht null sein.");

		this.writers = writers;

		this.mapper = new ___MapEntireMsg(logMsg.getPcName());
		this.systemInfoFactory = new __SystemInfoFactory("", logMsg);
		this.heartBeat = new __HeartBeat(logMsg.date.getTime(), deltaHBMS);

		manageSystemMessages();
	}

//=============================================================================
	@Override public void doWrite(final LogMsg msg) {doWrite(map(msg));}
	public void doWriteNextSystemMessage() {doWrite(createNextSystemMessage());}

	@Override public String getName() {return NAME;}
	@Override public void flush() 	{/**not buffered => NOP*/}
	@Override public Element getBeschreibungAsElement(final Document document) {
		return getBeschreibungAsElement(document, NAME);
	}
//=============================================================================
	private final ___MapEntireMsg getMapper()				{return this.mapper;}
	private final __SystemInfoFactory getSystemInfoFactory(){return this.systemInfoFactory;}
	private final __HeartBeat getHeartBeat() 				{return this.heartBeat;}
	private final List<IWriter<ClsLogInfo>> getWriters()	{return this.writers;}

	private final ClsLogInfo map(final LogMsg msg)	  {return getMapper().map(msg);}
	private final void doWrite(final ClsLogInfo info) {
		for (final IWriter<ClsLogInfo> writer : getWriters())
			writer.doWrite(info);
	}

	private void manageSystemMessages() {
		manageNextPotentialHBs();	//erste dann die HeartBeats
		registerFinalCleanup();		//letzte System Message
	}
	private __ClsLogInfoSystem createNextSystemMessage() {
		final long msNow = System.currentTimeMillis();
		final long nHB   = getHeartBeat().getHBCount();
		final __ClsLogInfoSystem info = getSystemInfoFactory().create(msNow, nHB);
		return info;
	}
	private void manageNextPotentialHBs() {
		final Thread thread = new Thread(createHBRunnable());
		thread.setName(HB_THREADNAME);
		thread.setDaemon(true);
		thread.start();
	}
	private Runnable createHBRunnable() {
		return new Runnable(){
			@Override public void run(){
				final AtomicLong nextHBZeit = new AtomicLong(updateForNextHeartBeat());
				while(continueRun()){
					doWriteNextSystemMessage();
					nextHBZeit.set(updateForNextHeartBeat());

					waitForHBorShutdown(nextHBZeit);	
					
					final long nextHBin = nextHBZeit.get() - System.currentTimeMillis();
					final String msg = "__TineLogAppender nextHB in " + nextHBin + "[ms].";
					Logger.SWFW.debug(msg);
				}
			}
			private boolean continueRun() {return !___TineLogAppender.this.refIsInShutdown.get();}
			private void waitForHBorShutdown(final AtomicLong nextHBZeit) {
				while(System.currentTimeMillis() < nextHBZeit.get() && continueRun())
					Sleep.sleep(HB_LOOP_WAIT_TOCHECK_MS);
			}
			private long updateForNextHeartBeat() {
				final long msNow = System.currentTimeMillis();
				return getHeartBeat().updateForNextHeartBeat(msNow);
			}
		};
	}

	/**at end of app there will be a system message*/
	private final void registerFinalCleanup(){
		Runtime.getRuntime().addShutdownHook(
				new Thread(){@Override public void run() {
					___TineLogAppender.this.refIsInShutdown.set(true);	//Stops HB Manage Loop
					doWriteNextSystemMessage();
				}
				}); 
	}
}//END class TineLogAppender

