/**
 * Dec 18, 2007 - Dec 19, 2007	ws:	Creation from scratch
 * Jun  3, 2008 - Jun 26, 2008	ws: catch Exception not only RunTimeException
 * 									cosmetics
 * Aug  1, 2008 - Aug  5, 2008	ws:	added: hasMassiveDecodingProblems()
 */
package de.desy.mst.libs.framework.mstapp.logging.reader;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.concurrent.atomic.AtomicLong;

import de.desy.mst.libs.framework.mstapp.logging.EinfacherLogDeformatter;
import de.desy.mst.libs.framework.mstapp.logging.EinfacherLogFormatter;
import de.desy.mst.libs.framework.mstapp.logging.LogMsg;
import de.desy.mst.libs.framework.mstapp.logging.Logger;

/**
 * Basis Implementation eines Datei Lesers <br>
 * Eine Datei pro objekt
 * @author ws
 * @version 1.0.0
 */
class LogReader extends BufferedReader{
	private static final String CHARSETNAME = "UTF-8";
	private static final int MAX_LOG_BAD_MSG_LINES_TO_SHOW = 10;
//=============================================================================
	public static LogReader createFileReader(final File file)throws IOException{
		final FileInputStream fis = new FileInputStream(file);
		final InputStreamReader isr = new InputStreamReader(fis, CHARSETNAME);
		return new LogReader(isr);
	}
//=============================================================================



//=============================================================================
	private boolean isClosed;
	private final String header;
	private final EinfacherLogDeformatter deformatter;
	private final AtomicLong nHeaderLinesR = new AtomicLong(0);
	private final AtomicLong nMessagesRead = new AtomicLong(0);
	private boolean hasMoreMessages = true;
	private long nBadMsgLines = 0L;
//=============================================================================
	public LogReader(final Reader in) throws IOException {
		super(in);
		this.isClosed = false;
		this.header = readHeader();
		this.deformatter = new EinfacherLogDeformatter(new EinfacherLogFormatter());
		this.deformatter.deformatSpaltenTitel(this.header);
	}
//=============================================================================
	@Override public void close() throws IOException {
		super.close();
		this.isClosed = true;
	}
	public boolean isClosed() {return this.isClosed;}
	public long getNumberOfMessagesRead() {return this.nMessagesRead.get();}

	@Override public String readLine() throws IOException {
		final String line = super.readLine();
		this.hasMoreMessages  = (line != null);
		return line;
	}
	/**erste Zeile des Readers*/
	public String getHeader() {return this.header;}
	/**reads the next Msg until msg == null
	 * @throws IOException */
	public LogMsg readMsg() throws IOException {
		final String line = readLine();
		if(line != null)
			this.nMessagesRead.incrementAndGet();
		try {
			final LogMsg msg = this.deformatter.deformatMessageEntry(line);
			return msg;
		} catch (final OutOfMemoryError e) {
			throw e;
		} catch (final Throwable e) {
			this.nBadMsgLines ++;
			if(this.nBadMsgLines <= MAX_LOG_BAD_MSG_LINES_TO_SHOW){
				final LogMsg msg = this.deformatter.createBadDeformatMsg(line, e);
				Logger.doLog(msg);
				if(this.nBadMsgLines == MAX_LOG_BAD_MSG_LINES_TO_SHOW) {
					Logger.SWFW.warn("Schlecht formatierte Meldungen werden ab jetzt fuer diese Quelle ignoriert.");
					if(getNumberOfMessagesRead() == this.nBadMsgLines){
						Logger.SWFW.warn("Da alle Meldungen schlecht, stelle lesen ein.");
						this.hasMoreMessages = false;
						return null;
					}
				}
				return msg;
			}
			return readMsg();	//ignoriere den Eintrag
		}
	} 
	/**liest Min(nMsgs, vorhandene Messages[Zeilen]), Messages werden nicht deformatiert*/
	public void skipMsgs(long nMsgs) throws IOException {
		while(nMsgs-- > 0 && readLine() != null)
				this.nMessagesRead.incrementAndGet();
	}
	public boolean hasMoreMessages() {return this.hasMoreMessages;}
	public boolean hasMassiveDecodingProblems() {
		return (this.deformatter.hasBadHeader())
			|| (this.nBadMsgLines >  0 && this.nBadMsgLines == getNumberOfMessagesRead())
			|| (this.nBadMsgLines > 50 && this.nBadMsgLines >= 0.2 * getNumberOfMessagesRead())
			;
	}
//=============================================================================
	private String readHeader() throws IOException {
		if(		this.nMessagesRead.get() > 0
		   || 	this.nMessagesRead.get() != 0){
			return this.header;
		}
		final String header = readLine();
		this.nHeaderLinesR.incrementAndGet();
		return header;
	}
}
