/**
 *  ???????????? - ????????????	ws:	Created from scratch
 *  Nov 29, 2007 - Nov 29, 2007	ws: added Maschinen sorted
 */
package de.desy.mst.libs.framework.mstapp.menu;

import javax.swing.ButtonGroup;
import javax.swing.JMenu;
import javax.swing.JRadioButtonMenuItem;

import de.desy.mst.libs.framework.mstapp.mode.IMaschineObserver;
import de.desy.mst.libs.framework.mstapp.mode.IMaschinenKonfigurationObserver;
import de.desy.mst.libs.framework.mstapp.mode.Maschine;
import de.desy.mst.libs.framework.mstapp.mode.MaschinenKonfiguration;
import de.desy.mst.libs.framework.mstapp.IMstAppKernel;

public class MenusMaschine implements IMaschineObserver, IMaschinenKonfigurationObserver{
	private static class MaschineMenu{
		private final Maschine  maschine;
		private final JRadioButtonMenuItem menuItem;
		public MaschineMenu(final Maschine maschine, final JRadioButtonMenuItem menuItem) {
			super();
			this.maschine = maschine;
			this.menuItem = menuItem;
		}
		public void onMaschineChange(final Maschine neueMaschine) {
			this.menuItem.setSelected(this.maschine.equals(neueMaschine));
		}
		public void onMaschinenKonfigurationChange(final MaschinenKonfiguration neueMaschinenKonfiguration) {
//			this.menuItem.setEnabled(neueMaschinenKonfiguration.istErlaubt(this.maschine));
			this.menuItem.setVisible(neueMaschinenKonfiguration.istErlaubt(this.maschine));
		}
	}
	private final  JMenu jMenuMaschine;
	private final        ButtonGroup            buttonGroupMaschine          = new ButtonGroup();
	private final  MaschineMenu[] 		  mm;
	private	final  IMstAppKernel mstApp;
	public MenusMaschine(final IMstAppKernel mstApp) {
		super();
		this.mstApp = mstApp;
		this.mm = createMMArray();
		this.jMenuMaschine = createJMenuMaschine();
		this.mstApp.getMaschineMgr().registerIMaschinenKonfigurationObserver(this);
		this.mstApp.getMaschineMgr().registerIMaschineObserver(this);
	}
	private MaschineMenu[] createMMArray() {
		final MaschineMenu[] mm = new MaschineMenu[countValidMaschines()];
		int i = 0;
		for (final Maschine maschine : Maschine.valuesSorted()) {
			if(!maschine.istMaschine()){continue;}
			mm[i] = createMM(maschine);
			i++;
		}
		return mm;
	}
	private MaschineMenu createMM(final Maschine maschine) {
		final JRadioButtonMenuItem jRadioButtonMenuItem = new JRadioButtonMenuItem(maschine.getLangenNamen());
		final MaschineMenu maschineMenu = new MaschineMenu(maschine, jRadioButtonMenuItem);
		this.buttonGroupMaschine.add(jRadioButtonMenuItem);
		jRadioButtonMenuItem.addActionListener(new java.awt.event.ActionListener() {
			@Override public void actionPerformed(final java.awt.event.ActionEvent e) {
				getMstApp().getMaschineMgr().setMaschine(maschine);
			}
		});
		return maschineMenu;
	}
	private int countValidMaschines() {
		int validMaschines = 0;
		for (final Maschine maschine : Maschine.valuesSorted()) {
			if(maschine.istMaschine()){
				validMaschines++;
			}
		}		
		return validMaschines;
	}
	public int anzahlEnabledMaschines(){
		int anzahl = 0;
		for (final MaschineMenu maschineMenu : this.mm) {
			if(maschineMenu.menuItem.isEnabled()) anzahl++;
		}
		return anzahl; 
	}
	public JRadioButtonMenuItem[] getJRadioButtonMenuItemMaschine() {
		final JRadioButtonMenuItem[] items = new JRadioButtonMenuItem[this.mm.length];
		for (int i = 0; i < items.length; i++) {
			items[i] = this.mm[i].menuItem;
		}
		return items;
	}
	public ButtonGroup getButtonGroupMaschine() {
		return this.buttonGroupMaschine;
	}
	private final IMstAppKernel getMstApp() {
		return this.mstApp;
	}
	@Override public void onMaschineChange(final Maschine neueMaschine) {
		for (final MaschineMenu maschineMenu : this.mm) {
			maschineMenu.onMaschineChange(neueMaschine);
		}
		getJMenuMaschine().setText("Maschine(" +  neueMaschine.langerName +")");
	}
	@Override public void onMaschinenKonfigurationChange(final MaschinenKonfiguration neueMaschinenKonfiguration) {
		for (final MaschineMenu maschineMenu : this.mm) {
			maschineMenu.onMaschinenKonfigurationChange(neueMaschinenKonfiguration);
		}
		getJMenuMaschine().setEnabled(this.anzahlEnabledMaschines() > 1);
	}
	
	public JMenu getJMenuMaschine() {
		return this.jMenuMaschine;
	}
	public JMenu createJMenuMaschine() {
		final JMenu jMenuMaschine = new JMenu();
		jMenuMaschine.setText("Maschine");
		for (final MaschineMenu maschineMenu : this.mm) {
			jMenuMaschine.add(maschineMenu.menuItem);
		} 
		return jMenuMaschine;
	} 
}
