/**
 *	Feb 15, 2008 - Feb 15, 2008 ws: added "@Override"es
 * 	Aug  6, 2009 - Aug  6, 2009	ws:	Eclipse asked for removing unnessary instanceof, so I did
 * 	Aug 18, 2009 - Aug 18, 2009	kh, ws:	addMenu(..) refaktoriert & null item Problem bei separator fixed
 * 	Sep  9, 2009 - Sep  9, 2009	ws:	add(JMenu ...) kopiere auch isVisible(), isEnabled()
 *	Dec 15, 2009 - Dec 23, 2009	ws:	add: getClickCounter() for debugging
 *									add: createStandardMouseListener() mit Feldvariable
 *									add: applyStandardMouseListener(...)
 */
package de.desy.mst.libs.framework.mstapp.menu;

import java.awt.Component;
import java.awt.Container;
import java.awt.event.ItemListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.text.JTextComponent;

import de.desy.mst.libs.framework.mstapp.io.MstPrintText;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.mode.MaschineMgr;
import de.desy.mst.libs.framework.mstapp.util.observables.setvalue.GuiSetSelectedManager;

@SuppressWarnings({"rawtypes"})
public class PopUpMenuMgr extends JPopupMenu implements IPopupMenuMgr{
	public static class StandardMouseAdapter extends MouseAdapter {
		final IPopupMenuMgr popup;
		/**@param popup, may not be null*/
		public StandardMouseAdapter(final IPopupMenuMgr popup) {
			if(popup == null)
				throw new NullPointerException();
			this.popup = popup;
		}
		/**Windows*/ @Override public void mouseReleased(final MouseEvent evt) {handleIt(evt);}
		/**Linux ?*/ @Override public void mousePressed (final MouseEvent evt) {handleIt(evt);}
		private void handleIt(final MouseEvent evt) {
			if(evt.isPopupTrigger())
				this.popup.showMenu(evt.getComponent(), evt.getX(), evt.getY());
		}
	}
	private static final long serialVersionUID = 1L;
//=============================================================================
	//private Deklarationen
	private final List<JComponent> menuList = new LinkedList<JComponent>();
	private final ArrayList<GuiSetSelectedManager> selMgrList = new ArrayList<GuiSetSelectedManager>();
	private final ArrayList<JMenuItem> oldMenuCleanup = new ArrayList<JMenuItem>();
	private final Map<JComponent, List<JComponent>> componentsWithAdditionalPopup = new HashMap<JComponent, List<JComponent>>();
	private final AtomicInteger clickCounter = new AtomicInteger(0);
	private final MouseListener mouseListener = new StandardMouseAdapter(this);
//=============================================================================
	//private Methoden alphabetisch
	private void addBeforePopup(final Component invoker){
		for (final JComponent item : this.menuList) {
			if(item instanceof JSeparator){
				this.add(new JSeparator());
			}else{
				if(item instanceof JMenu){
					addMenu((JMenu)item, this);
				}else{
					addMenuItem((JMenuItem)item);
				}
			}
		}
		if (this.componentsWithAdditionalPopup.containsKey(invoker) == true){
			this.add(new JSeparator());
			for (final JComponent item : this.componentsWithAdditionalPopup.get(invoker)) {
				if(item instanceof JSeparator){
					this.add(new JSeparator());
				}else{
					if(item instanceof JMenu){
						addMenu((JMenu)item, this);
					}else{
						addMenuItem((JMenuItem)item);
					}
				}
			}
		}
	} 
	private void addMenu(final JMenu oriMenu, final JComponent cmp){
		if(oriMenu == null)
			return;
		final JMenu newMenu = new JMenu(oriMenu.getText());
		newMenu.setVisible(oriMenu.isVisible());
		newMenu.setEnabled(oriMenu.isEnabled());
		for (int i = 0; i < oriMenu.getItemCount(); i++){
			final JMenuItem oriItem = oriMenu.getItem(i);
			try {
				final JMenuItem newItem = (oriItem == null) ? null : oriItem.getClass().newInstance();
				if (newItem == null || oriItem == null){
					newMenu.addSeparator();
				}else if(  newItem instanceof JRadioButtonMenuItem
						|| newItem instanceof JCheckBoxMenuItem){
					addSelectionMgmt(oriItem, newItem);
				}else if(  newItem instanceof JMenu){
					addMenu((JMenu)oriItem, newMenu);
				}else{
					newItem.setText(oriItem.getText());
					setMenuItemProperties(oriItem, newItem);
					newMenu.add(newItem);
				}
			}
			catch (final InstantiationException e) {Logger.SWFW.warn(e);}
			catch (final IllegalAccessException e) {Logger.SWFW.warn(e);}
		}
		cmp.add(newMenu);
	}
	private void addMenuItem(final JMenuItem menu){
		if(menu == null){return;}
		JMenuItem newMenu;
		try {
			newMenu = menu.getClass().newInstance();
		} catch (final InstantiationException e) {
			Logger.SWFW.warn(e);
			return;
		} catch (final IllegalAccessException e) {
			Logger.SWFW.warn(e);
			return;
		}
		newMenu.setText(menu.getText());

//		Muss als erstes stehen, da auch die beiden anderen auch MenuItems sind.
		if (   newMenu instanceof JRadioButtonMenuItem
			|| newMenu instanceof JCheckBoxMenuItem){
			addSelectionMgmt(menu, newMenu);
		}
		setMenuItemProperties(menu, newMenu);
		this.add(newMenu);
	}
	private void addSelectionMgmt(final JMenuItem oldMenu, final JMenuItem newMenu) {
		GuiSetSelectedManager guiSelMgr = null;
		try{
			//Test auf vorhandenen Listener
			final ItemListener itemListener = oldMenu.getItemListeners()[0];
			if(itemListener instanceof GuiSetSelectedManager){
				guiSelMgr = (GuiSetSelectedManager) itemListener;
			}else{
				Logger.SWFW.warn("Das Menue '" + oldMenu.getText()
						                       + "' hat einen itemListener der keine Instance von '"
						                       + GuiSetSelectedManager.class.getName() + "' ist.");
				return;
			}
		}catch(final Exception e){
			//Kein Listener vorhanden. Also einen erzeugen
			guiSelMgr = new GuiSetSelectedManager(oldMenu.isSelected());
			oldMenu.addItemListener(guiSelMgr);
			this.oldMenuCleanup.add(oldMenu);
			guiSelMgr.add(oldMenu);
			this.selMgrList.add(guiSelMgr);
		}
		newMenu.addItemListener(guiSelMgr);
		guiSelMgr.add(newMenu);
	}
	private void cleanupSeparators(){

		for (int i = 0; i < this.menuList.size(); i++) {
			if (this.menuList.get(i) instanceof JSeparator){
				if (i < this.menuList.size()-1){
					if (this.menuList.get(i + 1) instanceof JSeparator){
						this.menuList.remove(i);
					}
				}
			}
		}
		if (this.menuList.get(0) instanceof JSeparator){
			this.menuList.remove(0);
		}
		if (this.menuList.get(this.menuList.size()-1) instanceof JSeparator){
			this.menuList.remove(this.menuList.size()-1);
		}
	}
	private void removeBeforePopup(){
		for (final GuiSetSelectedManager selMgr : this.selMgrList) {
			selMgr.removeAll();
		}
		this.selMgrList.clear();

		for (final JMenuItem oldMenuItem : this.oldMenuCleanup) {
			final ItemListener[] itemListeners = oldMenuItem.getItemListeners();
			oldMenuItem.removeItemListener(itemListeners[itemListeners.length - 1]);
		}
		this.oldMenuCleanup.clear();

		this.removeAll();
	}
	private void setMenuItemProperties(final JMenuItem menu, final JMenuItem newMenu){
		newMenu.setVisible(menu.isVisible());
		newMenu.setEnabled(menu.isEnabled());
		newMenu.setBackground(menu.getBackground());
		newMenu.setForeground(menu.getForeground());
		newMenu.setBorder(menu.getBorder());
		newMenu.setFont(menu.getFont());
		newMenu.setOpaque(menu.isOpaque());
		try{
			newMenu.addActionListener(menu.getActionListeners()[0]);
		}catch(final Exception e){
		}
		try{
			newMenu.setIcon(menu.getIcon());
		}catch(final Exception e){
		}
	}


	//ffentliche Methoden alphabetisch
	/* (non-Javadoc)
	 * @see javax.swing.JPopupMenu#addSeparator()
	 */
	@Override public void addSeparator(){
		this.menuList.add(new JSeparator());
	}
	@Override public void deRegister(final JMenuItem item){
		for (int i = 0; i < this.menuList.size(); i++) {
			if(item.equals(this.menuList.get(i))){
				this.menuList.remove(i);
			}
		}
		cleanupSeparators();
	}
	@Override public void deRegister(final List<JMenuItem> items) {
		for (final JMenuItem item : items) {
			this.deRegister(item);
		}
	}
	@Override public void deRegisterCopmponentForAdditionalPopup(final JComponent component){
		this.componentsWithAdditionalPopup.remove(component);
		cleanupSeparators();
	}
	@Override public void register(final JMenuItem item){
		this.menuList.add(item);
		cleanupSeparators();
	}
	@Override public void register(final List<JMenuItem> items) {
		for (final JMenuItem item : items) {
			this.register(item);
		}
	}
	@Override public void registerCopmponentForAdditionalPopup(final JComponent component, final JMenuItem item){
		List<JComponent> popup;
		if(this.componentsWithAdditionalPopup.containsKey(component) == false){
			popup = new LinkedList<JComponent>();
		}else{
			popup = this.componentsWithAdditionalPopup.get(component);
		}
		popup.add(item);
		this.componentsWithAdditionalPopup.put(component, popup);
	}
	@Override public void registerCopmponentForAdditionalPopup(final JComponent component, final List<JMenuItem> items) {
		final List<JComponent> popup = new LinkedList<JComponent>();
		for (final JMenuItem item : items) {
			popup.add(item);
		}
		this.componentsWithAdditionalPopup.put(component, popup);
	}
	@Override public void showMenu(final Component invoker, final int x, final int y){
		getClickCounter().incrementAndGet();
		removeBeforePopup();
		addBeforePopup(invoker);
		this.show(invoker, x, y);
	}
	@Override public MouseListener getStandardPopUpMouseListener() {return this.mouseListener;}
	@Override public void registerListForPrintContent(final JList jList, final MaschineMgr mgr, final String title) {
		registerCopmponentForAdditionalPopup(jList, MstPrintText.getJMenuItemListeDrucken(jList, mgr, title));

	}
	@Override public void deRegisterListForPrintContent(final JList jList) {
		this.deRegisterCopmponentForAdditionalPopup(jList);

	}
	@Override public final AtomicInteger getClickCounter(){
		return this.clickCounter;
	}
	@Override public final void applyStandardMouseListener(final Component cmp, final boolean recursive) {
		if(cmp == null)
			return;
		if(noMouseListenerPresentOrAtMostTooltiplistener(cmp) && included(cmp))
			cmp.addMouseListener(getStandardPopUpMouseListener());
		if(recursive && cmp instanceof Container)
			for (int iCmp = 0; iCmp < ((Container)cmp).getComponentCount(); iCmp++)
				if(((Container)cmp).getComponent(iCmp) != cmp)
					applyStandardMouseListener(((Container)cmp).getComponent(iCmp), recursive);
	}
	private boolean noMouseListenerPresentOrAtMostTooltiplistener(final Component cmp) {
		final MouseListener[] lsts = cmp.getMouseListeners();
		return (lsts        == null)
			|| (lsts.length ==    0)
			|| (lsts.length ==    1 && lsts[0] instanceof javax.swing.ToolTipManager);
	}
	private boolean included(final Component cmp) {
//		final String className = cmp.getClass().getName();
//		if(className.startsWith("javax.swing.plaf.") && className.contains("UI"))
//			return false;
//		return true;
		return cmp instanceof JPanel
			|| cmp instanceof JLabel
			|| cmp instanceof JFrame
			|| cmp instanceof JTextComponent
			;
	}
}
