/**
 * ?????????  - ??????????  Wst: creation from scratch
 * 2007-02-06 - 2007-02-15	ws: Anschluss an System
 */

package de.desy.mst.libs.framework.mstapp.mode;

import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.preferences.IPrefContainer;
import de.desy.mst.libs.framework.mstapp.preferences.helper.APrefClientPanelEinfach;
import de.desy.mst.libs.framework.mstapp.util.gui.GUIFactory;

import java.awt.Dimension;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.border.TitledBorder;
import javax.swing.BorderFactory;

/**
 * @author wst, ws
 * version 1.0.0
 */
public class PrefServerChoicePanel extends APrefClientPanelEinfach{
	private static final long serialVersionUID = 4568663259196637290L;
	private final ServerAnwahlMgr serverAnwahlMgr;
	private JPanel jPnlServerAnwahl  = null;
	private JRadioButton[] jRButtons = null;
	private ButtonGroup buttonGroup  = null;
	private ServerAnwahl server;
	
	private final IServerAnwahlObserver observer = new IServerAnwahlObserver(){
		@Override public void onServerAnwahlChange(final ServerAnwahl anwahl) {
			initServerAnwahlFromSystem();
		}
	};
	
//=============================================================================
	
	/**
	 * @param prefContainer	  != null
	 * @param serverAnwahlMgr != null
	 * @throws NullPointerException falls einer der Parameter null
	 */
	public PrefServerChoicePanel(final IPrefContainer prefContainer,
			                     final ServerAnwahlMgr serverAnwahlMgr) {
		super(prefContainer);
		if(serverAnwahlMgr == null) {throw new NullPointerException("serverAnwahlMgr");}
		this.serverAnwahlMgr = serverAnwahlMgr;
		this.server = serverAnwahlMgr.getAnwahl();
		initialize();
		register();
	}
	
//=============================================================================
		
	private void initialize() {
		this.setSize(126, 200);
		this.add(getJPnlServerAnwahl(), null);
	}

	private JPanel getJPnlServerAnwahl() {
		if (this.jPnlServerAnwahl == null) {
			this.jPnlServerAnwahl = new JPanel();
			this.jPnlServerAnwahl.setLayout(new BoxLayout(this.jPnlServerAnwahl,
					BoxLayout.Y_AXIS));
			this.jPnlServerAnwahl.setBorder(
					BorderFactory.createTitledBorder(
					BorderFactory.createLineBorder(Color.gray, 1),
					"Server Anwahl", TitledBorder.DEFAULT_JUSTIFICATION,
					TitledBorder.DEFAULT_POSITION, null, null
					                                )
					                  );

			this.buttonGroup = new ButtonGroup();
			final JRadioButton[] butArr = getJRButtons();
			for (final JRadioButton button : butArr) {
				this.buttonGroup.add(button);
				this.jPnlServerAnwahl.add(button);
			}

			this.jPnlServerAnwahl.setPreferredSize(new Dimension(300, 100));
		}
		return this.jPnlServerAnwahl;
	}
 
	private final JRadioButton[] getJRButtons() {
		if (this.jRButtons == null) {
			final ServerAnwahl[] anwahlen = ServerAnwahl.VALUES;
			this.jRButtons = new JRadioButton[anwahlen.length];
			for (int i = 0; i < anwahlen.length; i++) {
				this.jRButtons[i] = createJRButton(anwahlen[i]);
			}
		}
		return this.jRButtons;
	}

	private JRadioButton createJRButton(final ServerAnwahl anwahl) {
		final JRadioButton jRButton = new JRadioButton();
		jRButton.setText(anwahl.name);
		jRButton.setFont(GUIFactory.createButtonFont());
		jRButton.addActionListener(new ActionListener() {
			@Override public void actionPerformed(final ActionEvent e) {
				setServer(anwahl);
			}
		});
		return jRButton;
	}

	public void setButtonGroupToSelectedValue() {
		final ServerAnwahl server = getServer();
		try {
			(getJRButtons()[server.index]).setSelected(true);
		} catch (final Exception e) {
			final String msg = "PrefServerChoicePanel: Kann die ServerAnwahl '" 
				             + server + "' Index = '" + server.index 
				             + "' nicht setzen. Ignorier die Aktion.";
			Logger.SWFW.error(msg, e);
		}
	}

//=============================================================================	
	
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.preferences.IPrefClient#onReset()
	 */
	@Override public void onReset() {initServerAnwahlFromSystem();}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.preferences.IPrefClient#onSet()
	 */
	@Override public void onSet() throws Exception {
		getServerAnwahlMgr().setServerAnwahl(getServer());
		initServerAnwahlFromSystem();
		updateIsChanged();
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.preferences.helper.APrefClientPanelMitPersistenz#onDispose()
	 */
	@Override public void onDispose() {deregister();}

//=============================================================================
	
	private ServerAnwahlMgr getServerAnwahlMgr() {return this.serverAnwahlMgr;}
	private void initServerAnwahlFromSystem() { 
		this.server = getOriginalServerAnwahl();
		updateIsImplemented();
		setButtonGroupToSelectedValue();
		updateIsChanged();
	}

	private ServerAnwahl getOriginalServerAnwahl() {
		return getServerAnwahlMgr().getAnwahl();
	}

	private ServerAnwahl getServer() {return this.server;}
	private void setServer(final ServerAnwahl server) {
		this.server = server;
		updateIsChanged();
	}
	
	private void updateIsChanged() {getPrefContainer().setChanged(isChanged());}
	private boolean isChanged() {
		return getServer() != getOriginalServerAnwahl();
	}
	private void updateIsImplemented() {
		final JRadioButton[] buttons  = getJRButtons();
		final ServerAnwahl[] anwahlen = ServerAnwahl.VALUES;
		for (int i = 0; i < buttons.length; i++) {
			buttons[i].setEnabled(isImplemented(anwahlen[i]));
		}
	}
	private boolean isImplemented(final ServerAnwahl serverAnwahl) {
		return getServerAnwahlMgr().getImplAnwahlen().isInList(serverAnwahl);
	}

	private void register() {
		getServerAnwahlMgr().registerAnwahlObserver(getObserver());
	}
	private void deregister() {
		getServerAnwahlMgr().deregisterAnwahlObserver(getObserver());
	}
	private final IServerAnwahlObserver getObserver() {return this.observer;}
}
