/**
 * 	Mar 30, 2015 - Apr 28, 2015	ws:	created from scratch
 */
package de.desy.mst.libs.framework.mstapp.mode.input;

import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.tabellendaten.entry.OPModeACLEntryFactory;
import de.desy.mst.libs.framework.mstapp.tabellendaten.entry.OPModeACLEntry;
import de.desy.tine.client.TLink;
import de.desy.tine.dataUtils.TDataType;
import de.desy.tine.definitions.TAccess;
import de.desy.tine.definitions.TErrorList;
import de.desy.tine.types.NAME32;
import de.desy.tine.types.TCompoundDataObject;

/**
 * Public entry point to Andreas OPModeACL Tine Service
 * @author ws
 * Ueberfluessigen Code (Klassen und Interfaces) entfernt.
 * Paralleliserung (callable) und error handling (return null)
 * wird nach Ruecksprache mit ws durch den aufrufenden Code verarbeitet  
 * @author al
 */
public class OPModeACLEntryFactory_Tine extends OPModeACLEntryFactory {
	private final static String TINE_SERVER_ADDESS = "/COMMON/ClientDisplayMode/";
	public static final int OAZ_NR_TINESERVER = -112;//A == 1, L == 12
	public static final String ACLNAME_TINESERVER = "FROM TINE SERVER (generic)";
	public static final OPModeACLEntryFactory_Tine INSTANCE = new OPModeACLEntryFactory_Tine();
	//=============================================================================================
	/**
	 * True, wenn dem Konstruktur die Klasse TLink bekannt ist.
	 * Ist die TLink Klasse nicht bekannt, werden in keinem Fall Operation Modes 
	 * vom Tine Server ausgelesen
	 */
	private final boolean hasTlinkClassString;
	/**
	 * creates the instance of factory
	 */
	private OPModeACLEntryFactory_Tine() {
		boolean hasTlinkClassString = false;
		try {
			TLink.class.toString();
			hasTlinkClassString = true;
		} catch (final Exception e) {
			hasTlinkClassString = false;
			Logger.SWFW.warn("Application hasn't defined class TLink. No operation modes will be defined by tine server.");
		}
		this.hasTlinkClassString = hasTlinkClassString;
	}
	//---------------------------------------------------------------------------------------------
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.tabellendaten.entry.OPModeACLEntryFactory#getOPModeACLEntry(java.lang.String, java.lang.String, java.lang.String)
	 * @return null in any case of constructor or function error
	 */
	@Override public final OPModeACLEntry getOPModeACLEntry(
		final String pcUser,
		final String pcName,
		final String appName
	) {
		if (this.hasTlinkClassString) {
			final TCompoundDataObject[] inputData = new TCompoundDataObject[]{new NAME32(pcUser + "@" + pcName)};
			final TDataType din = new TDataType(inputData);
			final int[] modes = new int[3];
			final TDataType dout = new TDataType(modes);
			final TLink reader = new TLink(OPModeACLEntryFactory_Tine.TINE_SERVER_ADDESS + appName + "/Summary", dout, din, TAccess.CA_READ);
			final int terror = reader.execute();
			if (terror == TErrorList.success) {
				final int maxOpMode = modes[1];
				final int defaultOpMode = modes[0];
				final int maxTestServerOpMode = modes[2];
				return new OPModeACLEntry(
					OAZ_NR_TINESERVER,
					pcUser, pcName, appName,
					maxOpMode, defaultOpMode, maxTestServerOpMode,
					OPModeACLEntryFactory_Tine.ACLNAME_TINESERVER
				);
			} else {
				Logger.SWFW.warn("Can not retrieve the OPModeACLEntry from tine: " + reader.getError(terror));
				return null;
			}
		} else {
			return null;
		}
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.tabellendaten.entry.OPModeACLEntryFactory#usesOldSQLWay()
	 */
	@Override 
	@Deprecated
	public boolean usesOldSQLWay() { return false;}
}
