/**
 *	ws: Sep 29, 2006 - Sep 29, 2006	creation from scratch
 *	ws: May 15, 2007 - May 15, 2007	replaced isCheckedForSave() with 
 *											 getSpeicherMode()
 *
 */
package de.desy.mst.libs.framework.mstapp.preferences;

/**
 * Fuer die Interaktion zwischen dem IPreferenceContainer und dem 
 * PreferencePanel. Man wird benachrichtigt, wenn einer der Knpfe "abbrechen",
 * "zurcksetzen", "anwenden" oder "OK" gedrckt wurde.<br>
 * Typische Implementation: Direkt im PreferencePanel.
 * 
 * @author  ws
 * @version 1.0.0
 */
public interface IPrefClient {
	

	/**
	 * Falls beim Versuch, die vom User gewhlten Einstellungen zu setzen, 
	 * etwas schief geht, muss eine Exception geworfen werden, damit das 
	 * FrameWork den User darueber in einheitlicher Weise informieren kann. 
	 * Insbesondere sollte der String in der Exception in deutsch und 
	 * informativ fuer einen Endbenutzer sein.<br>
	 * Wird bei "anwenden" und bei "OK" aufgerufen.
	 */
	public void onSet() throws Exception;
	
	/**
	 * Hier soll  das Panel wieder auf die urspruenglichen Werte zurueckgesetzt
	 * werden, damit der Benutzer noch mal von vorne beginnen kann.<br>
	 * Wird bei "abbrechen" und bei "zurcksetzen" aufgerufen.
	 */
	public void onReset();
	
	/**
	 * Macht nur Sinn, wenn isCheckedForSave() wahr ist.
	 * Hier stellt der Entwickler einen String zur Verfuegung mit dem er
	 * beim erneuten Registrieren eines IPreferences das Setting so wieder
	 * herstellen kann. Das Rahmenwerk speichert den String lediglich PC- und 
	 * User-spezifisch und gibt ihn, falls er auf diesem PC unter diesem User 
	 * abgespeichert wurde, beim Registrieren von IPreferences an IPreferences
	 * zurueck. 
	 * (Framework basiertes persistentes Konfigurationsmanagement).  
	 * @see	de.desy.mst.libs.framework.mstapp.preferences.IPreferences
	 * Wird bei "OK" abgefragt.
	 */
	public String getSaveString();
	
//  replaced with: "public PrefSpeicherMode getSpeicherMode();"	
//	/** 
//	 * Gibt an, ob abgespeichert werden soll; wenn der User  ankreuzen kann,
//	 * ob seine gewhlten Einstellungen beim nchsten Programstart wieder 
//	 * eingestellt werden sollen, wird der Wert der CheckBox zurckgegeben;
//	 * sonst muss der Entwickler das entscheiden.
//	 * Wird bei "OK" abgefragt.
//	 */
//	public boolean isCheckedForSave();
	
	/**
	 * Hier bitte den gewuenschten Speichermode zurueckgeben 
	 * @see de.desy.mst.libs.framework.mstapp.preferences.PrefSpeicherMode
	 */
	public PrefSpeicherMode getSpeicherMode();
}
