/*
 *	ws: May 15, 2007 - May 16, 2007	replaced isCheckedForSave() with
 *											 getSpeicherMode()
 *									extr. fireOnSet damit auch bei apply
 *										  gespeichert wird
 *	Feb 15, 2008 - Feb 15, 2008 ws: added "@Override"es
 * 	Sep 26, 2008 - Sep 26, 2008	ws: adopted to Wrapperless PrefFrame
 * 	Sep 29, 2008 - Oct  7, 2008	ws:	replaced reference by PrefManager
 *	Oct 15, 2008 - Oct 15, 2008 ws: added: getTitle()
 * 	May  5, 2009 - May  5, 2009	ws: Fix usability issue on CanceledByUserException
 * 	Apr 11, 2011 - Apr 11, 2011	ws: Force fix Libera/Pjotr/MDISchmidt problem in cancel handler
 */
package de.desy.mst.libs.framework.mstapp.preferences.implementation;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.SwingConstants;
import javax.swing.BorderFactory;
import javax.swing.border.BevelBorder;

import de.desy.mst.libs.framework.mstapp.ini.PrefStorage;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.preferences.IPrefClient;
import de.desy.mst.libs.framework.mstapp.preferences.IPrefContainer;
import de.desy.mst.libs.framework.mstapp.preferences.IPreferences;
import de.desy.mst.libs.framework.mstapp.preferences.PrefSpeicherMode;
import de.desy.mst.libs.framework.mstapp.util.CanceledByUserException;

//TODO with AJPanelForUserInput
/**
 * Hauptpanel im Preferences Frame. Dies ist der Container fuer den
 * IPreferencesClient und das PreferencesPanel des Entwicklers.
 *
 * @author  wst
 * @version 1.0.0
 */
public class PrefContainer extends JPanel implements IPrefContainer{
	private static final long serialVersionUID = 1L;
//-----------------------------------------------------------------------------
	private final PrefManager prefMgr;

	private final JLabel jLblTitle;
	private final NiceButton jButApply;
	private final NiceButton jButCancel;
	private final NiceButton jButOK;
	private final NiceButton jButReset;
	private final JPanel jPnlPref;

	private IPreferences actualIPref = null;
//=============================================================================
	public PrefContainer(final PrefManager prefMgr) {
		super(new BorderLayout());

		this.prefMgr = prefMgr;

		this.jLblTitle  = createJLblTitle();
		this.jButApply  = createJButApply();
		this.jButCancel = createJButCancel();
		this.jButOK     = createJButOK();
		this.jButReset  = createJButReset();
		this.jPnlPref   = createJPnlPref();

		this.setSize(400, 300);
		this.add(getJLblTitle(),				BorderLayout.NORTH);
		this.add(getJPnlPref(),					BorderLayout.CENTER);
		this.add(createJPnlStandardButtons(),	BorderLayout.SOUTH);

		setChanged(false);
	}
//=============================================================================
	@Override public boolean isChanged() {return getJButOK().isEnabled();}
	@Override public void setChanged(final boolean isChanged) {
		if(isChanged() == isChanged)
			return;
		getJButApply().setEnabled(isChanged);
		getJButOK().   setEnabled(isChanged);
		getJButReset().setEnabled(isChanged);
	}
	//-------------------------------------------------------------------------
	protected void onSetError(final Exception ex) {
		if(ex instanceof CanceledByUserException){
//			PrefContainer.this.setChanged(true);
			return;
		}
		final String msg0 = "Leider konnte die gewaehlte Konfiguration nicht" + " gesetzt werden.";
		final String msgE = msg0 + " Die Fehlermeldung lautet: '" + ex.getMessage() + "'";
		Logger.SWFW.warn(msg0, ex);
		JOptionPane.showMessageDialog(this, msgE);
	}
	//-------------------------------------------------------------------------
	IPreferences getActualIPref() {
		if(this.actualIPref == null)
			setActualIPref(getPrefMgr().getRoot());
		return this.actualIPref;
	}

	void setActualIPref(final IPreferences newActualIPref) {
		final IPreferences oldActualIPref = this.actualIPref;//NICHT DEN GETTER!!!
		removeActualIPref(oldActualIPref);
		if(newActualIPref != null)
			getJPnlPref().add(newActualIPref.getPrefComponent(this), BorderLayout.CENTER);
		this.actualIPref = newActualIPref;
		getJPnlPref().repaint();
	}
	void setTitle(final String title) {getJLblTitle().setText(title);}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.preferences.IPrefContainer#getTitle()
	 */ 
	@Override public String getTitle() {return getJLblTitle().getText();}
	//-------------------------------------------------------------------------
	private IPrefClient getClient () {return getActualIPref().getPrefClient();}
	private PrefManager getPrefMgr() {return this.prefMgr;}

	private JLabel  getJLblTitle () {return this.jLblTitle;}
	/**pp*/ JButton getJButApply () {return this.jButApply;}	//pp for test only
	private JButton getJButCancel() {return this.jButCancel;}
	private JButton getJButOK    () {return this.jButOK;}
	private JButton getJButReset () {return this.jButReset;}
	private JPanel  getJPnlPref  () {return this.jPnlPref;}

	private void removeActualIPref(final IPreferences oldActualIPref) {
		getJPnlPref().removeAll();
		if (oldActualIPref != null)
			oldActualIPref.onDispose();	//ws: added
		this.setChanged(false);
	}

	private JLabel createJLblTitle() {
		final JLabel jLblTitle = new JLabel();
		jLblTitle.setFont(new Font("Dialog", Font.BOLD, 18));
		jLblTitle.setHorizontalAlignment(SwingConstants.CENTER);
		jLblTitle.setAlignmentX(.5f);
		jLblTitle.setPreferredSize(new Dimension(400,60));
		return jLblTitle;
	}
	private NiceButton createJButApply() {
		final NiceButton jButApply = new NiceButton();
		jButApply.setText("anwenden");
		jButApply.addMouseListener(new MouseAdapter() {
			@Override public void mouseClicked(final MouseEvent evt){
				try{
					if(jButApply.isEnabled())
						onSet();
				} catch(final Exception ex) {onSetError(ex);}
			}
		});
		return jButApply;
	}
	private NiceButton createJButCancel() {
		final NiceButton jButCancel = new NiceButton();
		jButCancel.setText("abbrechen");
		jButCancel.addMouseListener(new MouseAdapter() {
			@Override public void mouseClicked(final MouseEvent evt){
				try {
					if(!jButCancel.isEnabled())
						return;
					getClient().onReset();
					onClosing();
				} catch (final Exception ex) {
					Logger.SWFW.error("Unexpected Exception in PrefContainer Cancel Handler.", ex);
				}
			}
		});
		return jButCancel;
	}
	private NiceButton createJButOK() {
		final NiceButton jButOK = new NiceButton();
		jButOK.setText("OK");
		jButOK.addMouseListener(new MouseAdapter() {
			@Override public void mouseClicked(final MouseEvent evt){
				if(!jButOK.isEnabled())
					return;
				try{onSet(); onClosing();
				}catch(final Exception ex){onSetError(ex);}
			}
		});
		return jButOK;
	}
	private NiceButton createJButReset() {
		final NiceButton jButReset = new NiceButton();
		jButReset.setText("zurcksetzen");
		jButReset.addMouseListener(new MouseAdapter() {
			@Override public void mouseClicked(final MouseEvent evt) {
				if(!jButReset.isEnabled())
					return;
				PrefContainer.this.getClient().onReset();
				PrefContainer.this.setChanged(false);
			}
		});
		return jButReset;
	}

	private JPanel createJPnlStandardButtons() {
		final JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
		panel.setPreferredSize(new Dimension(400,30));
		panel.add(getJButCancel(), null);
		panel.add(getJButReset(), null);
		panel.add(Box.createGlue());
		panel.add(getJButApply(), null);
		panel.add(getJButOK(), null);
		return panel;
	}

	private JPanel createJPnlPref() {
		final JPanel jPnlPref = new JPanel();
		jPnlPref.setLayout(new BorderLayout());
		jPnlPref.setPreferredSize(new Dimension(400,280));
		return jPnlPref;
	}


	private void onSet() throws Exception {
		getClient().onSet();
		if (getClient().getSpeicherMode() == PrefSpeicherMode.APPLOCAL) {
			final String saveString = getClient().getSaveString();
//			Logger.SWFW.debug("PrefPanelContainer SaveString: " + saveString);
			final PrefStorage storage = getPrefMgr().getStorage();
			storage.setPrefString(getActualIPref(), saveString);
			storage.store();
		}
		this.setChanged(false);
	}
	private void onClosing() {
		final IPreferences iPref = getActualIPref();
		if(iPref != null)
			iPref.onDispose(); //nur wenn onSet() klappt
		getPrefMgr().getPreferencesFrame().setVisible(false);
	}
}

//=============================================================================
//=============================================================================
//=============================================================================

class NiceButton extends JButton{
	private static final long serialVersionUID = 1L;
	public NiceButton(){
		super();
		setBackground(new Color(210,210,210));
		setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
		setFont(new Font("Dialog", Font.BOLD, 12));
		setPreferredSize(new Dimension(85,25));
		setMaximumSize(new Dimension(85,25));
		setMinimumSize(new Dimension(85,25));
	}
}
