/**
 *	Feb 15, 2008 - Feb 15, 2008 ws: added "@Override"es
 * 	Sep 24, 2008 - Oct  7, 2008	ws: refactor, add add(prf) remove(prf)
 * 									responsive to dynamic changes
 * 	Oct 22, 2008 - Oct 22, 2008	ws:	cleaner exit via windows button, added escape support
 * 	Jan  8, 2009 - Jan  8, 2009	ws:	title with calcAnnotatedTitle()
 * 	Jul 14, 2009 - Jul 14, 2009	ws:	fixed some Eclipse 3.5 warnings
 */

package de.desy.mst.libs.framework.mstapp.preferences.implementation;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.HeadlessException;
import java.awt.KeyEventDispatcher;
import java.awt.KeyboardFocusManager;
import java.awt.event.KeyEvent;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTree;
import javax.swing.WindowConstants;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import de.desy.mst.libs.framework.mstapp.Main;
import de.desy.mst.libs.framework.mstapp.MstApp;
import de.desy.mst.libs.framework.mstapp.MstFrame;
import de.desy.mst.libs.framework.mstapp.ParamDecoder;
import de.desy.mst.libs.framework.mstapp.preferences.IPreferences;

/**
 * Hauptframe fr die Preferences
 * @author wst, ws
 * version 1.0.0
 */
public class PrefFrame extends MstFrame {
	private static final long serialVersionUID = 1L;
	private static final int  ESCAPE_KEY_CODE  = 27;
	private final KeyEventDispatcher keyEventDispatcher = new KeyEventDispatcher(){
		@Override public boolean dispatchKeyEvent(final KeyEvent evt) {
			//System.out.println(" keyCode = " + evt.getKeyCode() + ", Source = " + evt.getSource());
			if(evt.getKeyCode() != ESCAPE_KEY_CODE)
				return false;	//do not handle it here

			if(!(evt.getSource() instanceof Component))
				return false;	//do not handle it here

			if(!isInParentChain((Component) evt.getSource()))
				return false;	//do not handle it here

			PrefFrame.this.setVisible(false);
			return true;
		}
	};
	public  static void main(final String[] args) {new PrefFrame().setVisible(true);}
//=============================================================================
	private JPanel jContentPane = null;
	private PrefContainer jPrefContainer = null;
	private JScrollPane jPnlTree = null;
	private JTree jTree = null;
	private JSplitPane jSplitPane = null;
	private final PrefManager prefManager;
	private final Map<IPreferences,TNode> nodes;
	private final DefaultTreeModel model;
//=============================================================================
	public PrefFrame() throws HeadlessException {
		this(new PrefManager(new MstApp(new Main("Dummy"), new ParamDecoder(new String[]{"faked"})), null));
	}
	public PrefFrame(final PrefManager prefManager){
		super(prefManager.getMstApp());

		this.prefManager = prefManager;

		this.nodes = new HashMap<IPreferences,TNode>();
		this.model = new DefaultTreeModel(null);

		initialize();

		KeyboardFocusManager.getCurrentKeyboardFocusManager()
							.addKeyEventDispatcher(this.keyEventDispatcher);}
//=============================================================================
	private boolean isInParentChain(final Component source) {
		if(this == source)
			return true;
		if(source == null || source == source.getParent())
			return false;
		return isInParentChain(source.getParent());
	}
	/**pp*/ final PrefManager getPrefManager() {return this.prefManager;}

	private JPanel getJContentPane() {
		if (this.jContentPane == null) {
			this.jContentPane = new JPanel();
			this.jContentPane.setLayout(new BorderLayout());
			this.jContentPane.add(getJSplitPane(), BorderLayout.CENTER);
		}
		return this.jContentPane;
	}

	/**pp*/  PrefContainer getPrefContainer() {
		if (this.jPrefContainer == null) {
			this.jPrefContainer = new PrefContainer(getPrefManager());
			this.jPrefContainer.setPreferredSize(new Dimension(200,300));
			this.jPrefContainer.setMaximumSize(new Dimension(200,300));
			this.jPrefContainer.setMinimumSize(new Dimension(200,300));
		}
		return this.jPrefContainer;
	}

 
	private JScrollPane getJPnlTree() {
		if (this.jPnlTree == null) {
			this.jPnlTree = new JScrollPane(getJTree());
			this.jPnlTree.setPreferredSize(new Dimension(200,100));
			this.jPnlTree.setMaximumSize(new Dimension(200,100));
			this.jPnlTree.setMinimumSize(new Dimension(200,100));
		}
		return this.jPnlTree;
	}

	private JSplitPane getJSplitPane() {
		if (this.jSplitPane == null) {
			this.jSplitPane = new JSplitPane();
			this.jSplitPane.setRightComponent(getPrefContainer());
			this.jSplitPane.setLeftComponent(getJPnlTree());
		}
		return this.jSplitPane;
	}

	private JTree getJTree() {
		if (this.jTree == null) {
			final PrefGeneral root = getPrefManager().getRoot();
			this.model.setRoot(this.nodes.get(root));
			this.jTree = new JTree(this.model);
			this.jTree.putClientProperty("JTree.lineStyle", "Angled");
			this.jTree.setShowsRootHandles(false);
			this.jTree.setFont(new Font("Dialog", Font.PLAIN, 11));
			this.jTree.addTreeSelectionListener(new TreeSelectionListener(){
				@Override public void valueChanged(final TreeSelectionEvent evt){
					final TNode node = (TNode) evt.getPath().getLastPathComponent();
					getPrefContainer().setTitle(node.toString());
					getPrefContainer().setActualIPref(node.getUserObject());
				}
			});
		}
		return this.jTree;
	}

/**
 * hier werden die Baumknoten initialisiert und fr die IPreferences, die noch
 * kein Parent haben, wird die root, also das Objekt von PrefGeneral zum Parent
 * gemacht
 *
 */
	private void initialize() {
		this.setSize(575, 402);
		this.setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);
		initialize(getPrefManager().getIPreferences());
		this.setContentPane(getJContentPane());
	}

	private void initialize(final List<IPreferences> allPreferences ) {
		this.setTitle(Main.calcAnnotatedTitle("Preferences"));
		for (final IPreferences preference : allPreferences) {
			initialize(preference);
		}
		for (final IPreferences preference : allPreferences)
			synchWithTreeModel(preference);
	}
	private void initialize(final IPreferences preference) {
		final TNode iNode = new TNode(preference);
		this.nodes.put(preference, iNode);
		assureParent(preference);
	}
	private void assureParent(final IPreferences preference) {
		final IPreferences root = getPrefManager().getRoot();
		if (preference != root && preference.getParent() == null)
			preference.setParent(root);
	}
	private void synchWithTreeModel(final IPreferences preference) {
		final IPreferences iPParent = preference.getParent();
		if (iPParent == null)
			return;

		final TNode iParentNode = this.nodes.get(iPParent);
		final TNode node = this.nodes.get(preference);
		this.model.insertNodeInto(node, iParentNode, iParentNode.getChildCount());
	}

	/**pp*/void add(final IPreferences preference){
		initialize(preference);
		synchWithTreeModel(preference);
	}
	/**pp*/void remove(final IPreferences preference){
		//Map<IPreferences,TNode> nodes
		final TNode node = this.nodes.remove(preference);
		if(node != null)
			if(this.model != null)
				this.model.removeNodeFromParent(node);
			else
				node.removeFromParent();
	}

	/**startet PrefFrame mit der Root*/
	public void setPrefSelectionToRoot(){
		setPrefSelection(getPrefManager().getRoot());
	}
	/**startet PrefFrame nicht mit der Root (PrefGeneral) sondern mit iP*/
	public void setPrefSelection(final IPreferences iP) {
		if(!this.nodes.containsKey(iP))
			return;
		final TNode node = this.nodes.get(iP);
		final TreeNode[] pathToRoot = this.model.getPathToRoot(node);
		final TreePath treePath = new TreePath(pathToRoot);
		getJTree().setSelectionPath(treePath);
		getPrefContainer().setActualIPref(iP);	//MaWa Problem 2008-10-06
	}

	//-------------------------------------------------------------------------

	/* (non-Javadoc)
	 * @see java.awt.Window#dispose()
	 */
	@Override public void dispose() {
		cleanUpPrefContainer();
		super.dispose();
	}
	/* (non-Javadoc)
	 * @see java.awt.Window#setVisible(boolean)
	 */
	@Override public void setVisible(final boolean b) {
		if(b == false)
			cleanUpPrefContainer();

		super.setVisible(b);
	}
	private void cleanUpPrefContainer() {
		final IPreferences iPref = getPrefContainer().getActualIPref();
		if(iPref != null)
			iPref.onDispose();
	}

//==============================================================================
//==============================================================================
//==============================================================================

	/**
	 * nur fuer nodes vom UserTyp IPreferences<br>
	 * toString is preference.getTitle()
	 */
	private static class TNode extends DefaultMutableTreeNode{
		private static final long serialVersionUID = 1L;
		public TNode(final IPreferences preferenc) {super(preferenc);}
		@Override public String toString() {
			return getUserObject() == null ? null : getUserObject().getTitle();
		}
		@Override public IPreferences getUserObject() {
			return (IPreferences) super.getUserObject();
		}
		@Override public void setUserObject(final Object iPreference) {
			if(iPreference instanceof IPreferences)
				super.setUserObject(iPreference);
			else
				throw new IllegalArgumentException("nicht vom Typ IPreferences");
		}
	}// END: class TNode

//==============================================================================
//==============================================================================
//==============================================================================
}


