/**
 *  Created on 	Sep  2, 2005 - Sep 14, 2005
 * 				Oct 25, 2005 - ...			ws: new logging system
 * 				Dec 14, 2005 - ...			ws: Konstanten FIELD_*
 * 				Aug 30, 2006 - Aug 31, 2006	ws: added getDeviceDetailKey(userKey), RESERVED_DEV_NO_FOR_DEFAULT = -61
 */
package de.desy.mst.libs.framework.mstapp.tabellendaten.entry;

import de.desy.mst.libs.framework.mstapp.logging.Logger;

/**
 * Devices bestehen aus 
 * Nr   (int  Index)   und
 * Name (String Key)
 * und sind immutable
 * Nr ist der primaere Key, iterator ist danach sortiert
 * @author ws
 * @version 1.0.0
 */
public class Device extends AEntry2<Integer, String> {
	public static final String FIELD_DEV_GR_NR	= "devGrNr";	//KEEP IN SYNCH WITH ACTUAL FIELD NAME
	public static final String FIELD_DEV_NO 	= "devNo";		//KEEP IN SYNCH WITH ACTUAL FIELD NAME
	public static final String FIELD_DEV_NAME	= "devName";	//KEEP IN SYNCH WITH ACTUAL FIELD NAME
	public static final int RESERVED_DEV_NO_FOR_DEFAULT = -61;	//Negative DevNo's kommen nicht vor und sind fuer Defaults (devNo = *) reserviert

	public final int    devGrNr;	//Device Gruppen Nummer
	public final int    devNo;		//Device Nummer >= 0 enforced
	public final String devName;	//never null

//=============================================================================
	
	/**
	 * @param devGrNr 			Device Gruppen Nummer
	 * @param devNo   >= 0		Device Nummer 	(eindeutig innerhalb einer Device Gruppe)
	 * @param devName <> null	Device Name 	(eindeutig innerhalb einer Device Gruppe)
	 * @throws IllegalArgumentException, if devNo < 0
	 * @throws NullPointerException,     if devName == null
	 */
	public Device(final int devGrNr, final int devNo, final String devName) {
		super();
		if(devNo   <     0){throw new IllegalArgumentException("Device Xtor: devN0 darf nicht kleiner Null sein.");}
		if(devName == null){throw new NullPointerException("Device Xtor: devName darf nicht null sein.");}
		this.devGrNr = devGrNr ;
		this.devNo   = devNo;
		this.devName = devName;
		if(this instanceof IDeviceGenauEinerGruppe){handleIDGEG((IDeviceGenauEinerGruppe)this);}
	}

//=============================================================================
	
	public int 		getDevGrNo(){return this.devGrNr;}
	public int 		getDevNo()	{return this.devNo;}
	public String 	getDevName(){return new String(this.devName);}
	public DeviceDetailKey getDeviceDetailKey(final int userKey) {
		return new DeviceDetailKey(getDevGrNo(), getDevNo(), userKey);
	}

//=============================================================================
	
	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.tabellendaten.entry.AEntry#inSeinenWertenGleich(de.desy.mst.mstapp.tabellendaten.entry.AEntry)
	 */
	@Override
	public boolean inSeinenWertenGleich(final AEntry<Integer> otherEntry) {
		if(otherEntry instanceof Device){
			final Device other = (Device)otherEntry;
			return (this.devGrNr == other.devGrNr)
			    && (this.devNo   == other.devNo)
			    && (this.devName.equals(other.devName));
		}
		return false;
	}
	
	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.tabellendaten.entry.AEntry2#getKey2()
	 */
	@Override public String getKey2() {return getDevName();}
	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.tabellendaten.entry.AEntry#getKey()
	 */
	@Override public Integer getKey() {return Integer.valueOf(getDevNo());}
//=============================================================================

	/**handleIDeviceGenauEinerGruppe*/
	private void handleIDGEG(final IDeviceGenauEinerGruppe iDGEG) {
		final int devGrNoSoll = iDGEG.getDevGrNoSoll();
		if(this.devGrNr != devGrNoSoll){
			final String msg = "Schlechte Device Gruppen Nr.: " 
				+ "Erwartet " + devGrNoSoll + ". "
				+ "Bekommen " + this.devGrNr     + ". ";
			Logger.SWFW.error(msg);
		}
	}
} 
