/**
 *  Created on  Aug 28, 2006 - Aug 31, 2006	ws	
 *	Feb 15, 2008 - Feb 15, 2008 ws: added "@Override"es
 */
package de.desy.mst.libs.framework.mstapp.tabellendaten.entry;

import de.desy.mst.libs.framework.mstapp.tabellendaten.CompositeKey;

/**
 * Zusammengesetzter Schlssel fuer eine Device-Detail-Tabelle<br>
 * userKey (Typ Integer) ist der user spezifische Schluessel (die ersten beiden Schluessel
 * sind devGrNr und devNo)<br>
 * Immutable Class, lexikalisch sortiert mit Value orientierter Semantik
 * @author ws
 *
 */
public class DeviceDetailKey implements Comparable<DeviceDetailKey>{
	private final CompositeKey key;
//	=============================================================================	
	public static DeviceDetailKey createDefaultKey(final DeviceDetailKey key) {
		return new DeviceDetailKey(key.getDevGrNr(), getMagicKeyValue(), key.getUserKey());
	}
	public static DeviceDetailKey createSuperDefaultKey(final DeviceDetailKey key) {
		return new DeviceDetailKey(key.getDevGrNr(), getMagicKeyValue(), getMagicKeyValue());
	}	
//=============================================================================	
	public DeviceDetailKey(final int devGrNr, final int devNo, final int userKey) {
		this.key = new CompositeKey(Integer.valueOf(devGrNr)	//0
								  , Integer.valueOf(devNo)		//1
								  , Integer.valueOf(userKey));	//2
	}
	public DeviceDetailKey(final DeviceDetail deviceDetail) {
		this(deviceDetail.getDevGrNr(), deviceDetail.getDevNo(), deviceDetail.getUserKey());
	}
		
//=============================================================================	
	public final int getDevGrNr(){return ((Integer) this.key.getKey(0)).intValue();}
	public final int getDevNo  (){return ((Integer) this.key.getKey(1)).intValue();}
	public final int getUserKey(){return ((Integer) this.key.getKey(2)).intValue();}
	public final boolean isDefaultKey()		{return this.getDevNo()		== DeviceDetailKey.getMagicKeyValue();}
	public final boolean isSuperDefaultKey(){return this.isDefaultKey() 
		                                        && (this.getUserKey()	== DeviceDetailKey.getMagicKeyValue());}
//-----------------------------------------------------------------------------	
	/** @see de.desy.mst.libs.framework.mstapp.tabellendaten.CompositeKey#compareTo(java.lang.Object)*/
	@Override public int compareTo(final DeviceDetailKey other)	{return this.key.compareTo((other).key);}
	/** @see de.desy.mst.libs.framework.mstapp.tabellendaten.CompositeKey#equals(java.lang.Object)*/
	@Override public boolean equals(final Object other)			{return this.key.equals(((DeviceDetailKey)other).key);}
	/** @see de.desy.mst.libs.framework.mstapp.tabellendaten.CompositeKey#hashCode()*/
	@Override public int hashCode()				{return this.key.hashCode();}
	/** @see de.desy.mst.libs.framework.mstapp.tabellendaten.CompositeKey#toString()*/
	@Override public String toString()			{return this.key.toString();}
	 
//	=============================================================================	
	private static int getMagicKeyValue() {return Device.RESERVED_DEV_NO_FOR_DEFAULT;}
}
