/**
 * ws:	Aug 16, 2007 - Aug 16, 2007 creation from scratch
 */
package de.desy.mst.libs.framework.mstapp.tabellendaten.entry;

import de.desy.mst.libs.framework.mstapp.tabellendaten.AlleTabellen;
import de.desy.mst.libs.framework.mstapp.tabellendaten.Tabelle;

/**
 * Ein Eintrag (eine Zeile) in der Info Tabelle (Info Tabellen habe jeweils
 * nur dieselbe InfoGruppenNummer pro Tabelle <br>
 * Dies ist eine immutable Class (absolut konstant und threadsicher)
 * @author ws
 * @version 1.0.0
 */
public class InfoEntry extends AEntry<Integer> {
	private static final String TABID_BASE_NAME = "InfoEntryXML1";
//=============================================================================
	/**Info-Gruppen-Id (Tabellenkonstant) */ public final int igid;	
	/**Info-Id         (Tabelleneindeutig)*/ public final int iid;
	/**kurzer Text        */ public final String kurz;
	/**langer Text        */ public final String lang;
	/**Wert wie definiert */ public final int int1;
	/**Wert wie definiert */ public final int int2;
	/**Wert wie definiert */ public final int int3;
	/**Wert wie definiert */ public final int int4;
	/**Wert wie definiert */ public final int int5;
//=============================================================================
	public final static Tabelle<Integer, InfoEntry> getInfoEntryTabelle(final int igid){
		final String tabId = calcTabID(igid);
		return AlleTabellen.getTabelle(tabId);
	}
	public static String calcTabID(final int igid) {
		return TABID_BASE_NAME + "+" + igid;
	}
//=============================================================================
	public InfoEntry(final int    igid, final int    iid
				   , final String kurz, final String lang
				   , final int int1, final int int2, final int int3
				   , final int int4, final int int5
				    ) {
		super();
		this.igid = igid;
		this.iid  = iid;
		this.kurz = kurz;
		this.lang = lang;
		this.int1 = int1;
		this.int2 = int2;
		this.int3 = int3;
		this.int4 = int4;
		this.int5 = int5;
	}
	public InfoEntry(final int igid,    final int iid
			       , final String kurz, final String lang
			       , final int int1, final int int2, final int int3
			       , final int int4, final int int5
			       , final Object ...unsupportedExtraParams
			        ) {
		this(igid, iid, kurz, lang, int1, int2, int3, int4, int5);
		super.handleUnsupportedExtraParams(this, unsupportedExtraParams);
	}
//=============================================================================
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.tabellendaten.entry.AEntry#getKey()
	 */
	@Override public Integer getKey() {return Integer.valueOf(getIid());}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.tabellendaten.entry.AEntry#inSeinenWertenGleich(de.desy.mst.libs.framework.mstapp.tabellendaten.entry.AEntry)
	 */
	@Override public boolean inSeinenWertenGleich(final AEntry<Integer> otherEntry) {
		if( this.equals(otherEntry)) {return true;}
		
		if(!(otherEntry instanceof InfoEntry)){return false;}
		final InfoEntry entry = (InfoEntry) otherEntry;
		return entry.igid == this.igid
		    && entry.iid  == this.iid
		    && entry.kurz.equalsIgnoreCase(this.kurz)
		    && entry.lang.equalsIgnoreCase(this.lang)
		    && entry.int1  == this.int1
		    && entry.int2  == this.int2
		    && entry.int3  == this.int3
		    && entry.int4  == this.int4
		    && entry.int5  == this.int5
		    ;
	}
//=============================================================================
	/**Info-Gruppen-Id (Tabellenkonstant) */
	public final int getIgid() {return this.igid;}
	/**Info-Id         (Tabelleneindeutig)*/
	public final int getIid()  {return this.iid; }
	/**kurzer Text        */public final String getKurz() {return this.kurz;}
	/**langer Text        */public final String getLang() {return this.lang;}
	/**Wert wie definiert */public final int getInt1() {return this.int1;}
	/**Wert wie definiert */public final int getInt2() {return this.int2;}
	/**Wert wie definiert */public final int getInt3() {return this.int3;}
	/**Wert wie definiert */public final int getInt4() {return this.int4;}
	/**Wert wie definiert */public final int getInt5() {return this.int5;}
} 
 