/**
 *	Aug 27, 2009 - Aug 27, 2009	ws:	creation from AJDialogWithCloseOnEscapeOKApplyCancel 
 */
package de.desy.mst.libs.framework.mstapp.util.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.border.BevelBorder;

/**
 * Organises the OK, Cancel, ... mechanics for User Input Panels
 * <br> user components are added via this.add(component)
 * <br> user components should setDirty if something is changed
 * <br> onSet and onReset must be supplied
 * <br> a reference the enclosing Window (JFrame, JDialog, ...) is needed
 * @author ws
 * @version 1.0.0
 */
public abstract class AJPanelForUserInput extends JPanel {
	private static final long serialVersionUID = 1L;
//=============================================================================
	private final Window  enclosingWindow;
	private final JButton okButton 		= createOKButton();
	private final JButton applyButton 	= createAnwendenButton();
	private final JButton resetButton	= createResetButton();
	private final JButton cancelButton	= createCancelButton(); 
	private final JComponent[] dirtySensitiveComponents = {this.okButton, this.applyButton, this.resetButton};
//=============================================================================
	/**
	 * @param enclosingWindow if(not null) {it will be closed onOK, onCancel} else {this.setVisible(false)}
	 */
	public AJPanelForUserInput(final Window enclosingWindow) {
		this.enclosingWindow = enclosingWindow;
		
		this.setLayout(new BorderLayout());
		super.add(createPnlStdBtns(), BorderLayout.SOUTH);
		resetDirty();
	}
//=============================================================================
	/**Model State := GUI State // not Dirty*/
	abstract public void onSet  (final ActionEvent evt);
	/**GUI State := Model State // not Dirty*/
	abstract public void onReset(final ActionEvent evt);
	
	/**Components are added this way, other add functions circumvent it: do not use*/
	@Override public Component add(final Component comp) {
		super.add(comp, BorderLayout.CENTER);
		return comp;
	}
	/**Muss bei GUI State != Model State gesetzt werden // im Zweifel dirty*/
	public final void        setDirty() {setDirty(true);}
	/**@return only false is reliable*/
	public final boolean mightBeDirty() {return this.dirtySensitiveComponents[0].isEnabled();}
//=============================================================================
	private final void resetDirty() {setDirty(false);}
	/**
	 * nur in exakt dieser Klasse darf isDirty == false gesetzt werden,
	 * da mglicherweise mehrstufige Unterklassen, die sich nicht kennen
	 */
	private final void setDirty(final boolean isDirty){
		for (final JComponent cmp : this.dirtySensitiveComponents) 
			cmp.setEnabled(isDirty);
	}
	
	private final Window getEnclosingWindow() {return this.enclosingWindow;}
	
	private final JButton getOkButton    () {return this.okButton;}
	private final JButton getApplyButton () {return this.applyButton;}
	private final JButton getResetButton () {return this.resetButton;}
	private final JButton getCancelButton() {return this.cancelButton;}
	
	private JButton createOKButton() {
		final JButton but = new NiceButton("OK", new ActionListener() {
			@Override public void actionPerformed(final ActionEvent evt) {onSet(evt);dispose();}
		});
		return but;
	}
	private JButton createAnwendenButton() {
		final JButton but = new NiceButton("anwenden", new ActionListener() {
			@Override public void actionPerformed(final ActionEvent evt) {onSet(evt); resetDirty();}
		});
		return but;
	}
	private JButton createResetButton() {
		final JButton but = new NiceButton("zurcksetzen", new ActionListener() {
			@Override public void actionPerformed(final ActionEvent evt) {onReset(evt); resetDirty();}
		});
		return but;
	}
	private JButton createCancelButton() {
		final JButton but = new NiceButton("abbrechen", new ActionListener() {
			@Override public void actionPerformed(final ActionEvent evt) {onReset(evt);dispose();}
		});
		return but;
	}
	
	private JPanel createPnlStdBtns() {
		final JPanel pnlStdBtns = new JPanel();
		pnlStdBtns.setLayout(new BoxLayout(pnlStdBtns, BoxLayout.X_AXIS));
		final int prefHeight = 2 * getOkButton().getPreferredSize().height;
		pnlStdBtns.add(Box.createVerticalStrut(prefHeight));

		pnlStdBtns.add(getCancelButton());
		pnlStdBtns.add(getResetButton());

		pnlStdBtns.add(Box.createGlue());

		pnlStdBtns.add(getApplyButton());
		pnlStdBtns.add(getOkButton());
		return pnlStdBtns;
	}
	private void dispose() {
		if(getEnclosingWindow() != null)
			getEnclosingWindow().dispose();
		else
			this.setVisible(false);
	}

//=============================================================================
	/**von Wst in PrefContainer*/
	private class NiceButton extends JButton{
		private static final long serialVersionUID = 1L;
		public NiceButton(final String text, final ActionListener lst){
			super.setText(text);
			super.addActionListener(lst);
			setBackground(new Color(210,210,210));
			setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
			setFont(new Font("Dialog", Font.BOLD, 12));
			setPreferredSize(new Dimension(85,25));
			setMaximumSize(new Dimension(85,25));
			setMinimumSize(new Dimension(85,25));
		}
	}//END CLASS: NiceButton
}
