/**
 * Oct 21, 2008 - Oct 21, 2008	ws:	Creation from scratch
 * Jul 16, 2009 - Jul 16, 2009	ws:	set image in initialize()
 * Sep 25, 2009 - Sep 25, 2009	ws: IconImage Handling modified
 */
package de.desy.mst.libs.framework.mstapp.util.gui;

import java.awt.Component;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.Image;
import java.awt.KeyEventDispatcher;
import java.awt.KeyboardFocusManager;
import java.awt.Window;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JDialog;
import javax.swing.JFrame;

import de.desy.mst.libs.framework.mstapp.Main;

/**
 * ganz normaler Dialog, der aber dispose aufruft wenn escape gerdrueckt wird.
 * <br> Bei hinzufuegen weiterer Konstruktoren die KeyboardFocusManager Zeilen
 * hinzufuegen.
 * @author ws
 * @version 1.0.0
 */
public class JDialogWithCloseOnEscape extends JDialog {
	private static final long serialVersionUID = 1L;
	private static final int  ESCAPE_KEY_CODE  = 27;
	//-------------------------------------------------------------------------
	private final KeyEventDispatcher keyEventDispatcher = new KeyEventDispatcher(){
		@Override public boolean dispatchKeyEvent(final KeyEvent evt) {
			//System.out.println(" keyCode = " + evt.getKeyCode() + ", Source = " + evt.getSource());
			if(evt.getKeyCode() != ESCAPE_KEY_CODE)
				return false;	//do not handle it here

			if(!(evt.getSource() instanceof Component))
				return false;	//do not handle it here

			if(!isInParentChain((Component) evt.getSource()))
				return false;	//do not handle it here

			JDialogWithCloseOnEscape.this.dispose();	//Source can only be part of Dialog
			return true;
		}
	};
//=============================================================================
	public JDialogWithCloseOnEscape(final Dialog owner, final String title, final boolean modal) {
		super(owner, title, modal);
		initialize(owner);
	}
	public JDialogWithCloseOnEscape(final JFrame owner, final String title, final boolean modal) {
		super(owner, title, modal);
		initialize(owner);
	}
	public JDialogWithCloseOnEscape(final Frame owner, final String title, final boolean modal) {
		super(owner, title, modal);
		initialize(owner);
	}
//=============================================================================
	/* (non-Javadoc)
	 * @see java.awt.Window#dispose()
	 */
	@Override public void dispose() {
		KeyboardFocusManager.getCurrentKeyboardFocusManager()
							.removeKeyEventDispatcher(this.keyEventDispatcher);
		super.dispose();
	}
//=============================================================================
	private void initialize(final Window owner) {
		KeyboardFocusManager.getCurrentKeyboardFocusManager()
							.addKeyEventDispatcher(this.keyEventDispatcher);
		this.setIconImages(getIconImages(owner));
	}
	/**vesuche aus owner zu holen, sonst system image*/
	private List<Image> getIconImages(final Window owner) {
		if(owner == null)
			return getSystemImages();
		final List<Image> iconImages = owner.getIconImages();
		if(iconImages == null || iconImages.isEmpty())
			return getSystemImages();
		return iconImages;
	}
	private List<Image> getSystemImages() {
		final ArrayList<Image> iconImages = new ArrayList<Image>();
		iconImages.add(Main.getAppImage());
		return iconImages; 
	}
	private boolean isInParentChain(final Component source) {
		if(this == source)
			return true;
		if(source == null || source == source.getParent())
			return false;
		return isInParentChain(source.getParent());
	}
}
