/**
 * May 29, 2008 - May 29, 2008	ws:	Creation from scratch
 * Jun 30, 2008 - Jul  2, 2008	ws: mstApp.png -> mstapp11.png (AL Konvention)
 * 									toLowerCase() of resourceName
 * Aug 15, 2008 - Aug 15, 2008	ws: appName -> alName (nur Bezeichnung)
 */
package de.desy.mst.libs.framework.mstapp.util.gui;

import java.awt.Image;
import java.awt.Toolkit;
import java.net.URL;
import de.desy.mst.libs.framework.mstapp.logging.Logger;

/**
 * Hilfsklasse um das Standard Image Icon zu erzeugen
 * (/resources/webstart/appName.png).
 * <br>createIconImage ohne Exception, aber vielleicht null als Rueckgabe-Wert
 *     Dann allerdings mit Logger.SWFW.warn(...).
 * @author ws
 * @version 1.0.0
 */
public class MstImage {
	private static final String ROOT 				= "/resources/";
	private static final String WEBSTART 			= "webstart/";
	private static final String SUFFIX = ".png";
	private static final String MSTAPP_ICON_NAME	= "mstapp11" + SUFFIX;
	final String alName;
	private final Class<?> classFromJar;
//=============================================================================
	/**Resources in jars can be found optimally sometimes only with class from the corresponding .jar*/
	public MstImage(final String alName, final Class<?> classFromJar) {
		super();
		this.alName = alName.replace(" ", "");
		this.classFromJar = (classFromJar == null) 	? this.getClass()
													: classFromJar;
	}
//=============================================================================
	/**appSpezifisches nach appName, sonst system Image, sonst null*/
	public Image createImage() {
		final Image image = createImage(WEBSTART + this.alName + SUFFIX);
		if(image != null)
			return image;
		return createImageSystem();
	}
	/**sieht unter "/resources/" nach und liefert evtl. null*/
	public Image createImage(final String resourceName) {
		final String completeName = ROOT + resourceName.toLowerCase();
		try {
			final URL url = this.classFromJar.getResource(completeName);
			if(url == null)
				throw new NullPointerException("URL ist null.");

			final Image image = Toolkit.getDefaultToolkit().getImage(url);
			Logger.SWFW.debug("MstImage.createImage ldt: '"
							 + completeName + "'.");
			return image;
 
		} catch (final Exception ex) {
			final String msg = "MstImage.createImage(" + completeName
							 + ") kann nicht erzeugt werden. Meldung: "
							 + ex.getMessage();
			Logger.SWFW.warn(msg);
			return null;
		}
	}
//=============================================================================
	/**pp*/Image createImageSystem() {
		return createImage(WEBSTART + MSTAPP_ICON_NAME);
	}
}
