/**
 *  Created on	May 17, 2006 - May 17, 2006 from SetEnabledManager
 * 	Oct 16, 2006 - 	Oct 16, 2006	ws:	removed Guilleaumes (-> Markus Walla) 
 */
package de.desy.mst.libs.framework.mstapp.util.observables.setvalue;

import javax.swing.AbstractButton;

/**
 * Typesafe observable for objects that have a 'setSelected(boolean)' method
 *  
 * 'setSelected(String)' is the call back function/event
 * 
 * Support for any component like 	Button, Canvas, Checkbox, Choice, Container,
 * 									Label, List, Scrollbar, TextComponent
 *									is already implemented.
 *  
 * If you need more kinds of objects i.e. classes
 * 1) provide a SPECIFIC (not for any Object) method add(NewClass newClass),
 * 2) provide a testcase in SetSelectedManagerTest
 * AND 
 * 3) run the test successfully
 *
 * Implemented with the non typesafe setvalue class
 * 
 * @author ws
 * @version 1.0.0
 */
public class SetSelectedManager extends SetValueManager{
	/**
	 * @param startValue	a default boolean start with.
	 * 						null will be replaced by empty string "" 
	 */
	public SetSelectedManager(final boolean startValue) {
		super(Boolean.valueOf(startValue), "setSelected");			
		this.forceParameterTypes(new Class[]{boolean.class});   //need the class of native type not its corresponding class
	}
	 
	/**
	 * @param startValue	all registered clients will be receive a method call 
	 * 						to setSelected(startValue)
	 */
	public void setSelected(final boolean startValue) {
		this.setValue(Boolean.valueOf(startValue));					 
	}
	
	/**
	 * @param 	item	a component for automated setSelected(boolean) like
	 * 					<li>JButton
	 * 					<li>JMenuItem (inclusive: JCheckBoxMenuItem, JMenu, JRadioButtonMenuItem)
	 * 					<li>JToggleButton
	 */
	public void add(final AbstractButton item) {
		//item.setSelected(true);
		this.addObject(item);
	}	

}
