/**
 *  	Sep  8, 2005 - Sep 9, 2005	ws	Created on
 * 		Oct 25, 2005 - 				ws:	...	new logging system
 * 		Oct 25, 2006 - Oct 26, 2006	ws: IWerteInfo extrahiert
 * 		Jun 12, 2008 - Jun 12, 2008	ws: small fix
 */
package de.desy.mst.libs.framework.mstapp.util.reflection;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import de.desy.mst.libs.framework.mstapp.logging.Logger;

/**
 * This class works on public properties of any Object named here Entry E.
 * Static properties are ignored.
 * Helper for Tabellen um Spalten von Entries zu konstruieren.
 * @author ws
 * @version 1.0.0 
 */
@SuppressWarnings({"rawtypes"})
public class PublicFields<E> implements ISpaltenInfo<E>{
	public final Class theClass;		//classUnderInspection
	public final List<Field> fieldList;	//never null, unmodifiable, maybe empty
	
//=============================================================================
	
	public PublicFields(final Class inspection) {
		this.theClass = inspection;
		this.fieldList = Collections.unmodifiableList(calcFieldList());
	}
	
//=============================================================================
		
	/**
	 * @param fieldName
	 * @return null or Field
	 */
	final Field getField(final String fieldName) {
		for (final Field field : this.fieldList) {
			if(field != null && field.getName().equals(fieldName)) {return field;}
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.util.reflection.ISpaltenInfo#getSpalte(java.lang.String, java.lang.Iterable)
	 */
	@Override public List<Object> getSpalte(final String id
								, final String spaltenName
								, final Iterable<E> entryIterator) {
		final Field field = getField(spaltenName);
		final List<Object> spalte = new ArrayList<Object>();
		for (final E entry : entryIterator) {
			Object object = null;
			try { 
				if(entry != null && field != null){object = field.get(entry);}
			} catch (final IllegalArgumentException e) {
				final String msg = createErrMsgForFieldRetrieval(id, spaltenName, entry);
				Logger.SW.error(msg);
			} catch (final IllegalAccessException e) {
				final String msg = createErrMsgForFieldRetrieval(id, spaltenName, entry);
				Logger.SW.error(msg);
			}
			spalte.add(object);
		}
		return spalte;  
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.util.reflection.ISpaltenInfo#getSpaltenNamen()
	 */
	@Override public List<String> getSpaltenNamen() {
		final ArrayList<String> namensListe = new ArrayList<String>(this.fieldList.size());
		for (final Field field : this.fieldList) {
			namensListe.add(field.getName());
		}
		return namensListe;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.util.reflection.ISpaltenInfo#isTrivial()
	 */
	@Override public boolean isTrivial() {return (this.theClass == null);}

//=============================================================================
	
	private List<Field> calcFieldList(){
		final List<Field> fieldList = new ArrayList<Field>();
		final Field[] fieldArray = (this.theClass == null) ? new Field[]{} : this.theClass.getFields();
		if(fieldArray != null) {
			for (int i = 0; i < fieldArray.length; i++) {
				final Field field   = fieldArray[i];
				if(isMyTypeOfField(field)){fieldList.add(field);}
			}
		}
		return fieldList;
	}

	private boolean isMyTypeOfField(final Field field) {
		try{
			final int modifiers = field.getModifiers();
			return 		Modifier.isPublic(modifiers)
					&& !Modifier.isStatic(modifiers);
		}
		catch(final Exception e){
			Logger.SWFW.error("Probleme Felder einer Klasse zu analysieren", e);
			return false;	//im Zweifel Ignorieren
		}
	}

	private final String createErrMsgForFieldRetrieval(final String id
													 , final String spaltenName
													 , final E entry) {
		return "Kann Spalte '" + spaltenName 
		           + "' nicht fuellen fuer entry '" + entry
		           + "' mit Id '" + id 
		           +"'.";
	}
}
