/**
 * 	Jul  6, 2010 - Jul  7, 2010	ws:	created from scratch
 * 	Aug 30, 2011 - Aug 30, 2011	ws:	added boolean isDesyIpAddress()
 */
package de.desy.mst.libs.info.appadmin;

import static de.desy.mst.libs.info.appadmin._StringEntry.*;

import java.net.InetAddress;
import java.util.logging.Level;

/**
 * Immutable singleton class
 * @author ws
 */
class _HostInfo {
	public final String hostAddress;
	public final String hostName;
	public final String computerUser;
	public final _StringEntry hostAddressEntry;
	public final _StringEntry hostNameEntry;
	public final _StringEntry computerUserEntry;
	
	//-------------------------------------------------------------------------
	private _HostInfo(final String address, final String name, final String user) {
		this.hostAddress  = address;
		this.hostName     = name;
		this.computerUser = user;
		
		this.hostAddressEntry  = new __StringEntry(KEY_HOSTADDRESS,  address); 
		this.hostNameEntry     = new __StringEntry(KEY_HOSTNAME,     name   ); 
		this.computerUserEntry = new __StringEntry(KEY_COMPUTERUSER, user   ); 
	}
	//-------------------------------------------------------------------------
	@Override public int hashCode() {
		final int p = 31;
		int r = 1;
		r = p * r + ((this.computerUser == null) ? 0 : this.computerUser.hashCode());
		r = p * r + ((this.hostAddress  == null) ? 0 : this.hostAddress .hashCode());
		return r;
	}
	@Override public boolean equals(final Object obj) {
		if (this == obj) return true;
		if (obj == null) return false;
		if (getClass() != obj.getClass()) return false;
		
		final _HostInfo other = (_HostInfo) obj;
		if (this.computerUser == null) {
			if (other.computerUser != null)
				return false;
		} else if (!this.computerUser.equals(other.computerUser))
			return false;
		if (this.hostAddress == null) {
			if (other.hostAddress != null)
				return false;
		} else if (!this.hostAddress.equals(other.hostAddress))
			return false;
		if (this.hostName == null) {
			if (other.hostName != null)
				return false;
		} else if (!this.hostName.equals(other.hostName))
			return false;
		return true;
	}
	//-----------------------------------------------------------------------------
	public boolean isDesyIPAddress(final String ipAddress) {
		return ipAddress != null && ipAddress.startsWith(DESY_IPADDRESS_PREFIX);
	}
	public boolean isDesyIPAddress(){return this.isDesyIPAddress(this.hostAddress);}
	
//=============================================================================
	
	private static final String HOSTNAME    = "HostName";
	private static final String HOSTADDRESS = "HostAddress";
	private static final String USER 		= "user.name";

	private static final String DESY_IPADDRESS_PREFIX = "131.169.";

	private static final _HostInfo inst = create();	

	//-------------------------------------------------------------------------

	public static _HostInfo getInstance() {return inst ;}
	
	//-------------------------------------------------------------------------

	private static _HostInfo create() {
		return new _HostInfo(calcHostAddress(), calcHostName(), calcUserName());
	}
//	private static _HostInfo createDummy() {
//		return new _HostInfo(unknownWhat(HOSTADDRESS)
//						   , unknownWhat(HOSTNAME   )
//						   , unknownWhat(USER       )
//						    );
//	}

	private static String calcHostAddress(){
		try {
			return InetAddress.getLocalHost().getHostAddress();
		} catch (final Exception ex){
			return warnAndReturnDefault(ex, HOSTADDRESS);
		}
	}
	private static String calcHostName(){
		try {
			return InetAddress.getLocalHost().getHostName();
		} catch (final Exception ex){
			return warnAndReturnDefault(ex, HOSTNAME);
		}
	}
	private static String calcUserName(){
		try {
			return System.getProperty(USER, unknownWhat(USER));
		} catch (final Exception ex){
			return warnAndReturnDefault(ex, USER);
		}
	}
	
	private static String warnAndReturnDefault(final Exception ex
											 , final String what) { 
		_Logging.log(Level.WARNING, "Can not determine " + what + ".", ex); 
		return unknownWhat(what);
	}
	private static String unknownWhat(final String what) {
		return "?" + what + "?";
	}
	
//=============================================================================
		
	private final class __StringEntry extends _StringEntry {
		__StringEntry(final String key, final String value) {
			super(key, value);
		}
	}
}
