/**
 *	Feb  3, 2010 - Mar 16, 2010	ws:	creation from scratch 
 */
package de.desy.mst.libs.info.appadmin;

import java.io.PrintStream;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.concurrent.atomic.AtomicLong;
import java.util.logging.Level;

/**
 * Single point wrapper for Logging
 * <br> might get in future version explicit logging mechanism from AppAdminContext
 * @author ws
 * @version 1.0.0
 */
class _Logging {
	private static final AtomicLong refCount;
	private static final List<_ILogger> logger;
//-----------------------------------------------------------------------------
	static {
		refCount = new AtomicLong(0);
		logger = new CopyOnWriteArrayList<_ILogger>();
		_Logging.logger.add(new ConsoleLogger());
	}
	
//-----------------------------------------------------------------------------	
	/**returns always null for syntactic sugar (abnormal end of function can return log(...)*/
	public static final <E> E log(final Level lvl, final String msg, final Throwable thr) {
		try{
			_Logging.refCount.incrementAndGet();
			for (final _ILogger iLogger : _Logging.logger) 
				iLogger.log(lvl, msg, thr);
		}catch(final Throwable thr_){
			Level.WARNING.intValue();/*silently resigning (statement for breakpoint only)*/
		}
		return null;
	}
//-----------------------------------------------------------------------------	
	/**for test purposes only*/
	static final long _getCount() {return refCount.get();}
//-----------------------------------------------------------------------------
	private static class ConsoleLogger implements _ILogger{
		private final AtomicLong refCount;
		public ConsoleLogger() {
			this.refCount = new AtomicLong(0);
		}
		@SuppressWarnings({ "unchecked" }) 
		public <E> E log(final Level lvl, final String msg, final Throwable thr) {
			final long cnt = this.refCount.incrementAndGet();
			if(cnt > 0 && cnt <= AppAdminContext.getInstance().getMaxConsoleLoggings())
				return (E) doLog(lvl, msg, thr);
			return null;
		}
		private Object doLog(final Level lvl, final String msg, final Throwable thr) {
			final boolean isSevere = (lvl != null 
					&& lvl.intValue() >= Level.WARNING.intValue());
			final PrintStream printStream = isSevere ? System.err : System.out;
			printStream.println("AppAdmin._Logging " + lvl + " Msg: " + msg);
			if(thr != null)
				thr.printStackTrace(printStream);
			return null;
		}
	}
}
interface _ILogger{
	public <E> E log(final Level lvl, final String msg, final Throwable thr);
}
