/**
 *	Feb  2, 2010 - Mar 11, 2010	ws:	creation from scratch 
 *	Jul  7, 2010 - Jul  9, 2010	ws:	add _HostInfo infos, testEntry with Name
 *										, _Msg( _ExternalSystemEntry,  Set<_Entry<?>>) {
 */
package de.desy.mst.libs.info.appadmin;

import static de.desy.mst.libs.info.appadmin._StringEntry.*;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.logging.Level;

/**
 * package private general Message for publication to the Message Server 
 * as payload in a Message Server Message etc.
 * <p>
 * immutable and therefore threadsafe class
 * @author ws
 * @version 1.0.0
 */
final class _Msg implements Iterable<_Entry<?>>{
	private static final int N_GENERAL_ENTRIES = 7;	//Test, Nr, Time, RunId = 3 HostInfos
	//-------------------------------------------------------------------------
	static final String _TEST_MSG_FILTER = _TestEntry._KEY + " = '" + Boolean.TRUE + "'";
	static final _Msg _createTestMessage(final String name) {
		return new _Msg(_TestEntry.TE_TRUE, new _NameEntry(name));
	}
	
//=============================================================================
	
	private final Map<String, _Entry<?>> entries;
//-----------------------------------------------------------------------------
	/**
	 * creates a _Msg for sending system and extra entries
	 * @param extraEntries
	 */
	public _Msg(final _Entry<?> ... extraEntries) {
		final int initialCapacity = _Msg.N_GENERAL_ENTRIES + extraEntries.length;
		this.entries = new HashMap<String, _Entry<?>>(initialCapacity);
		
		put(        _TestEntry.TE_FALSE     );
		put( _NrGeneratorEntry.create()     );	//next number in seq 1,2,3...
		put(        _TimeEntry.create()     );	//now
		put(       _RunIDEntry.RUNID_ENTRY  );	//unique to this virt machine run
		
		final _HostInfo hostInfo = _HostInfo.getInstance();
		put( hostInfo.hostAddressEntry  );
		put( hostInfo.hostNameEntry     );
		put( hostInfo.computerUserEntry );
		
		put_ReplaceOnlyIfTypeOK(getEntries(), extraEntries);
	}
	/**for external system use only*/
	_Msg(final _ExternalSystemEntry sysEnty, final Set<_Entry<?>> entrySet) {
		final int initialCapacity = entrySet.size() + 1;
		this.entries = new HashMap<String, _Entry<?>>(initialCapacity);
		
		put( sysEnty);
		for (final _Entry<?> entry : entrySet) 
			put( entry);
	}
//-----------------------------------------------------------------------------
	public final Iterator<_Entry<?>> iterator() {return getEntries().values().iterator();}
	
	@Override public String toString() {
		final StringBuilder sb = new StringBuilder();
		final Map<String, _Entry<?>> entries = getEntries();
		sb.append("_Msg(");
		toStringBuilder(sb, entries);
		sb.append(")");
		return sb.toString();
	}
	
	/**@return value or null if problem casting/getting*/
	@SuppressWarnings("unchecked") 
	public <E> E get(final String key, final E[] prototypeType) {
		try{
			final _Entry<E> entry = (_Entry<E>) getEntries().get(key);
			if(prototypeType.getClass().getComponentType().equals(entry.getClassType()))
				return entry.getValue();
			return null;
		}
		catch(final Throwable th){return null;}
	}
	public String getString(final String key){return get(key, new String[0]);}
	public Long   getLong  (final String key){return get(key, new   Long[0]);}
	
	/**@return System.currentTimeMillis() at message creation*/
	public final Long getTimeMS(){return get(_TimeEntry._KEY, new Long[0]);}
	/**number of the message (together with runId globally unique per Message)*/
	public final Long getNr(){return get(_NrGeneratorEntry._KEY, new Long[0]);}
	/**number of the message (together with runId globally unique per Message)*/
	public final String getRunId(){return getString(_RunIDEntry._KEY);}
	public final String getHostAddress (){return getString(KEY_HOSTADDRESS );}
	public final String getHostName    (){return getString(KEY_HOSTNAME    );}
	public final String getComputerUser(){return getString(KEY_COMPUTERUSER);}
	public final boolean isTestMsg(){
		final Boolean retVal = get(_TestEntry._KEY, new Boolean[0]);
		return retVal != null ? retVal : false;
	}
	public void setTestMsg(final boolean isTestMsg) {		
		getEntries().put(_TestEntry._KEY, _TestEntry.get(isTestMsg));
	}
	@Deprecated /**only for creating testcases*/ void setTimeMs(final long newTimeMS){
		getEntries().put(_TimeEntry._KEY, new _TimeEntry(newTimeMS));
	}
	public boolean contains(final _Entry<?> entry) {
		return getEntries().containsValue(entry);
	}
	public boolean containsByValue(final _Entry<?> entry) {
		if(contains(entry))
			return true;
		
		for (final _Entry<?> e : getEntries().values()) 
			if(_Entry.equalByValue(entry, e))
				return true;
		
		return false;
	}
	//-----------------------------------------------------------------------------
	private final Map<String, _Entry<?>> getEntries() {return this.entries;}
	
	private void put(final _Entry<?> entry) {
		if(entry != null)
			getEntries().put(entry.getKey(), entry);
	}
	private void put_ReplaceOnlyIfTypeOK(final Map<String, _Entry<?>> entries, final _Entry<?>... extraEntries) {
		for (final _Entry<?> entry : extraEntries)
			if(entry != null && entry.getKey() != null)
				put_ReplaceOnlyIfTypeOK(entries, entry);
	}
	private _Entry<?> put_ReplaceOnlyIfTypeOK(final Map<String, _Entry<?>> entries, final _Entry<?> newEntry) {
		final String key = newEntry.getKey();
		if(!entries.containsKey(key) || entries.get(key).getClassType().equals(newEntry.getClassType()))
			return entries.put(key, newEntry);
		
		final _Entry<?> oldEntry = entries.get(key);
		_Logging.log(Level.WARNING, complainMsg(key, oldEntry.getClassType(), newEntry), null);
		return oldEntry;	//there exists an incompatible different one, so keep 
							//otherwise getNr(), getRunID(), ... will crash
	}
	private String complainMsg(final String key, final Class<?> oldClassType, final _Entry<?> newEntry) {
		return "AppAdmin.put_ReplaceOnlyIfTypeOK(key = " + key + ", classType = " 
			 + (oldClassType == null ? "null" : oldClassType.getSimpleName()) + ", ...) encountered incompatible new entry: '" 
			 + newEntry + "' keep the original entry to enforce type safety";
		
	}
	private void toStringBuilder(final StringBuilder sb, final Map<String, _Entry<?>> entries) {
		final String entrySep = ", ";
		for (final _Entry<?> entry : entries.values()) 
			sb.append(entry).append(entrySep);
		
		if(entries.size() > 0)	//remove the entrySep
			sb.setLength(sb.length() - entrySep.length());
	}
}
