/**
 * 	Aug 5, 2010 - Aug 6, 2010	ws:	created from scratch
 *	Aug 16, 2010 - Aug 16, 2010	ws:	logging ohne stack trace
 *
 */

package de.desy.mst.libs.info.appadmin;

import java.util.concurrent.atomic.AtomicInteger;
import java.util.logging.Level;

import javax.jms.ExceptionListener;
import javax.jms.JMSException;
import javax.jms.MessageListener;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueReceiver;
import javax.jms.QueueSession;
import javax.jms.Session;

import com.sun.messaging.QueueConnectionFactory;

/**
 * @author ws
 */
class _MsgReaderJMSQueue {
	private static final AtomicInteger refCountOpenConnections = new AtomicInteger(0);
	//=========================================================================
	private final String queueName;
	private final QueueConnection connection;
	private final QueueReceiver   receiver;
	//-------------------------------------------------------------------------
	_MsgReaderJMSQueue(final String queueName
					 , final MessageListener msgListener
					 , final String filter)  throws JMSException {
		
		this.queueName  = queueName;
		this.connection = createConnection();
		this.receiver =  createReceiver(queueName, filter);
		getReceiver().setMessageListener(msgListener);
		_Logging.log(Level.FINE, calcMsg(refCountOpenConnections.incrementAndGet(), "Created"), null);
	}
	//-------------------------------------------------------------------------
	void close() {
		try{
			getReceiver  ().close();
			getConnection().close();
			_Logging.log(Level.FINE, calcMsg(refCountOpenConnections.decrementAndGet(), "Closed "), null);
		}
		catch (final Throwable th){/*Ignore all problems at this stage.*/}
	}
	//-------------------------------------------------------------------------
	private String calcMsg(final int cnt, final String what) {
		return "_MsgReaderJMSQueue: " + what + " (" + cnt + ") JMS Subscriber on queue '" + this.queueName + "'.";
	}
	private final QueueConnection getConnection() {return this.connection;}
	private final QueueReceiver  getReceiver() {return this.receiver;}
	private QueueConnection createConnection() {
		final AppAdminContext instance = AppAdminContext.getInstance();
		try {
			final QueueConnectionFactory tcf = new QueueConnectionFactory();
			tcf.setProperty(instance.getJMSMQ_URL_KEY(), instance.getJMSMQ_URL());
			
			final QueueConnection queueConnection = tcf.createQueueConnection();
			
			//proper closing to save server resources
			Runtime.getRuntime().addShutdownHook(new Thread(new Runnable() {
				public void run() {
					try{queueConnection.stop();}
					catch (final Throwable th){/*Ignore all problems at this stage.*/}
				}
			}));
			
			//readers do not get feedback on connection exceptions
			final ExceptionListener exceptionListener = new ExceptionListener() {
				public void onException(final JMSException ex) {
					_Logging.log(Level.FINE, "Unexpected JMS Problem", ex);
				}
			};
			queueConnection.setExceptionListener(exceptionListener);
			
			queueConnection.start();
			return queueConnection;
		} catch (final Throwable th) {
			final String msg = "Can not create a topicConnection. Original Message: " + th.getMessage();
			_Logging.log(Level.SEVERE, msg, null);
			return null;
		}
	}
	private QueueReceiver createReceiver(final String queueName
									   , final String filter) throws JMSException {
		final QueueSession session = getConnection().createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
		final Queue queue = session.createQueue(queueName);
		final QueueReceiver receiver  = session.createReceiver(queue, filter);
		return receiver;
	}
}
