/**
 *	Feb 12, 2010 - Mar  9, 2010	ws:	creation from scratch 
 * 	Jul 12, 2010 - Jul 13, 2010	ws:	copied from prj appadminanalysis.MsgReaderJMS
 * 									some natural refactoring
 * 	Aug 12, 2010 - Aug 12, 2010	ws:	fixed some Nullpointer stuff with null connection
 *	Aug 16, 2010 - Aug 16, 2010	ws:	logging ohne stack trace
 *
 */
package de.desy.mst.libs.info.appadmin;

import java.util.logging.Level;

import javax.jms.ExceptionListener;
import javax.jms.JMSException;
import javax.jms.MessageListener;
import javax.jms.Session;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicSession;
import javax.jms.TopicSubscriber;
import com.sun.messaging.TopicConnectionFactory;

import de.desy.mst.libs.info.appadmin.AppAdminContext;
import de.desy.mst.libs.info.appadmin._Logging;

/**
 * Reads Messages from Topic "topicName" filtered by "filter" and notifies
 * the "msgListener"
 * @author ws
 * @version 1.0.0
 */
class _MsgReaderJMSTopic {
	private final TopicConnection 	connection;
//-----------------------------------------------------------------------------	
	public _MsgReaderJMSTopic(final String topicName
							, final MessageListener msgListener
							, final String filter) throws JMSException{
		this.connection = createConnection();
		
		final TopicSubscriber subscriber = createSubscriber(topicName, filter);
		if(subscriber != null)
			subscriber.setMessageListener(msgListener);
		_Logging.log(Level.FINE, "Created JMS Subscriber on topic '" + topicName + "'.", null);		
	}
//-----------------------------------------------------------------------------
	public void stop() {
		try{   if(getConnection() != null) getConnection().stop();   }
		catch (final Throwable th){/*Ignore all problems at this stage.*/}
	}
//-----------------------------------------------------------------------------	
	private TopicConnection getConnection() {return this.connection;}
	private TopicConnection createConnection() {
		final AppAdminContext instance = AppAdminContext.getInstance();
		try {
			final TopicConnectionFactory tcf = new TopicConnectionFactory();
			tcf.setProperty(instance.getJMSMQ_URL_KEY(), instance.getJMSMQ_URL());
			
			final TopicConnection topicConnection = tcf.createTopicConnection();
			
			//proper closing to save server resources
			Runtime.getRuntime().addShutdownHook(new Thread(new Runnable() {
				public void run() {
					try{topicConnection.stop();}
					catch (final Throwable th){/*Ignore all problems at this stage.*/}
				}
			}));
			
			//readers do not get feedback on connection exceptions
			final ExceptionListener exceptionListener = new ExceptionListener() {
				public void onException(final JMSException ex) {
					_Logging.log(Level.FINE, "Unexpected JMS Problem", ex);
				}
			};
			topicConnection.setExceptionListener(exceptionListener);
			
			topicConnection.start();
			return topicConnection;
		} catch (final Throwable th) {
			final String msg = "Can not create a topicConnection. Original Message: " + th.getMessage();
			_Logging.log(Level.SEVERE, msg, null);
			return null;
		}
	}
	private TopicSubscriber createSubscriber(final String topicName
										   , final String filter) throws JMSException {
		if(getConnection() == null)
			return null;
		
		final TopicSession session = getConnection().createTopicSession(false, Session.AUTO_ACKNOWLEDGE);
		final Topic topic = session.createTopic(topicName);
		final TopicSubscriber subscriber  = session.createSubscriber(topic, filter, true);
		return subscriber;
	}
}
