/**
 * 
 * Last changed: $Date$
 * By: $Author$
 * URL: $HeadURL$
 * $Revision$ 
 *
 */
package de.desy.mst.libs.pintlk.badgerviewer;

import java.net.InetAddress;
import java.net.UnknownHostException;

import de.desy.mst.libs.framework.mstapp.logging.Logger;
/** Die Namen aller vorhandenen ConsoleServer
 * @author labudda
 */
enum CopyOfConsoleName {
	ZZonDO_1("K17 B", "BKRXPCSCON02"),
	ZZonDO_2("K17 A", "BKRXPCSCON01"),
	//ZZonDO_2("Office Test", "MCSXPALASKA"),
	ZZonDO_3("K17 C", "BKRXPCSCON03"),
	ZZonPitz("Name already missing", "MCSXPALASKA"),
	
	
	none("'Keine gueltige Konsole definiert'", null);
	
	private final String display;
	private final String defaultClientHost;	// Name des Hosts, der den Server per default benutzt. Kann null sein, wenn dem Server kein default Host zugeordnet ist
	private final boolean isHostOnsameSubnet;
	

	CopyOfConsoleName(final String display, final String hostname) {
		this.display = "Videokonsole " + display;
		this.defaultClientHost = hostname;
		boolean isHostOnsameSubnet;
		;
		try {
			final java.net.InetAddress address = java.net.InetAddress.getByName(hostname);
			final int subnet = CopyOfConsoleName.getSubnet(address, -this.ordinal());
			isHostOnsameSubnet = (subnet == CopyOfConsoleName.getLocalHostSubnet());
		} catch (UnknownHostException e) {
			Logger.Netzwerk.error(e);
			isHostOnsameSubnet = false;
		}
		this.isHostOnsameSubnet = isHostOnsameSubnet;
	}
	
	/**
	 * @return subnet of local host
	 * @throws UnknownHostException 
	 */
	private static int getLocalHostSubnet() throws UnknownHostException {
		final java.net.InetAddress address = java.net.InetAddress.getLocalHost();
		return CopyOfConsoleName.getSubnet(address, Integer.MIN_VALUE);
	}
	
	/**
	 * @param address
	 * @param errorValue
	 * @return subnet part of ip address
	 */
	private static int getSubnet (final java.net.InetAddress address, final int errorValue) {
		final String addressItems[] = address.getHostAddress().split("\\.");
		if (addressItems.length == 4) {
			try {
				final Integer subnet = Integer.valueOf(addressItems[2]);
				return subnet;
			} catch (NumberFormatException e) {
				Logger.SWApp.error(e.getMessage());
				return errorValue;
			}
		} else {
			Logger.SWApp.warn(address.getHostAddress() + ": format not expected");
			return errorValue;
		}
	}

	/** Dies ist der Versuch aus einem defaultClientHost den Namen eines gueltigen consol servers zu ermitteln
	 * @return - Name of server based on defaultClientHost
	 */
	public static CopyOfConsoleName getDefault() {
		try {
			final String hostname = InetAddress.getLocalHost().getHostName();
			for (CopyOfConsoleName server : CopyOfConsoleName.values()) {
				if (hostname.equalsIgnoreCase(server.defaultClientHost)) {
					return server;
				} else {
					// try next
				}
			}
			Logger.SWApp.warn("No defaut console server detected by " + hostname);
		} catch (UnknownHostException ex) {
			Logger.Netzwerk.error("Can not determine Hostname", ex);
		}
		return CopyOfConsoleName.none;
	}

	/**
	 * @return the display
	 */
	public final String getDisplay() {
		return display;
	}
	
	/**
	 * @param name
	 * @return Name einer Konsole oder Default. Never null
	 */
	public static CopyOfConsoleName getConsoleNameOf(final String name) {
		for (CopyOfConsoleName consoleName : CopyOfConsoleName.values()) {
			if (consoleName.name().equalsIgnoreCase(name)) {
				return consoleName;
			} else {
				// try next
			}
		}
		return CopyOfConsoleName.none;
	}

	/**
	 * @return true, wenn sich der Host dieser Applikation im selben Subnetz wie die Konsole befindet 
	 */
	public boolean isDisplayableOnLocalHost(){
		return this.isHostOnsameSubnet;
	}
}
