/**
 * Anzeige eines DACHS Devices 
 * EventDispatchthread only
 * Last changed: $Date$
 * By: $Author$
 * URL: $HeadURL$
 * $Revision$ 
 *
 */
package de.desy.mst.libs.pintlk.badgerviewer.display;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.DefaultListCellRenderer;
import javax.swing.JComponent;
import javax.swing.JList;

import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.pintlk.badgerviewer.network.DachsEntry;
import de.desy.mst.libs.pintlk.badgerviewer.network.IDevice;
import de.desy.mst.libs.ser.tclientlib.TineLinkException;

/**
 * @author labudda
 *
 */
public class DeviceDisplay extends JComponent {
	//private class DachsEntryRenderer extends  DefaultListCellRenderer {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private volatile IDevice device;
	private final javax.swing.DefaultListModel<DachsEntry> listModel;
	private final javax.swing.JLabel listTitle;
	private final javax.swing.JLabel adminMessage;
	private final javax.swing.Timer timer;
	@SuppressWarnings({ "rawtypes", "unchecked" })
	public DeviceDisplay(){
		this.listModel = new javax.swing.DefaultListModel<DachsEntry>();
		this.listTitle = new javax.swing.JLabel();
		this.adminMessage = new javax.swing.JLabel();
		this.adminMessage.setToolTipText("Vom DACHS Team definierter Text");
		final javax.swing.JList list = new javax.swing.JList(this.listModel);
		list.setCellRenderer(new DefaultListCellRenderer() {
			private static final long serialVersionUID = 1L;

			/* (non-Javadoc)
			 * @see javax.swing.DefaultListCellRenderer#getListCellRendererComponent(javax.swing.JList, java.lang.Object, int, boolean, boolean)
			 */
			public Component getListCellRendererComponent(JList srclist, Object value, int index, boolean isSelected, boolean cellHasFocus) {
				final javax.swing.JLabel label;
				if (value instanceof DachsEntry) {
					final DachsEntry entry = (DachsEntry)value;
					label = (javax.swing.JLabel) super.getListCellRendererComponent(srclist, entry.getText(), index, isSelected, cellHasFocus);
					label.setFont(label.getFont().deriveFont(18.0f));
					entry.decorate(label);
					return label;
				} else {
					final String invalidValue = "_debug: not a valid dachs value. Bitte in das logbuch drucken";
					label = (javax.swing.JLabel) super.getListCellRendererComponent(list, invalidValue, index, isSelected, cellHasFocus);
					label.setFont(label.getFont().deriveFont(24.0f));
					de.desy.mst.libs.framework.mstapp.util.gui.AppColor._Service_Req.set(label);
				}
				return label;
			}

		});
		final javax.swing.JScrollPane listScroller = new javax.swing.JScrollPane(list);
		
		this.setLayout(new java.awt.GridBagLayout());
		this.add(
			this.listTitle,
			new java.awt.GridBagConstraints(
				0, 0,
				1, 1,
				1.0, 0.0,
				GridBagConstraints.CENTER,
				GridBagConstraints.BOTH,
				new java.awt.Insets(0,0,0,0),
				0,0
			)
		);
		this.add(
			listScroller,
			new java.awt.GridBagConstraints(
				0, 1,
				1, 1,
				1.0, 1.0,
				GridBagConstraints.CENTER,
				GridBagConstraints.BOTH,
				new java.awt.Insets(0,0,0,0),
				0,0
			)
		);
		this.add(
			this.adminMessage,
			new java.awt.GridBagConstraints(
				0, 2,
				1, 1,
				1.0, 1.0,
				GridBagConstraints.CENTER,
				GridBagConstraints.BOTH,
				new java.awt.Insets(0,0,0,0),
				0,0
			)
		);
		this.timer = new javax.swing.Timer(
			1000,
			new ActionListener(){

				public void actionPerformed(ActionEvent e) {
					DeviceDisplay.this.checkDevice();
					
				}
			}
		);
	}
	/**
	 * @param device
	 */
	public DeviceDisplay(IDevice device) {
		this();
		this.setDevice(device);
	}
	/**
	 * 
	 */
	private void checkDevice() {
		if (this.device == null) {
			// nbt
		} else {
			try {
				this.adminMessage.setText(this.device.getDeviceName());
				java.util.AbstractList<DachsEntry> entries = device.getEntries();
				//this.listModel.setSize(entries.size());
				int index = 0;
				for (DachsEntry entry : entries) {
					if (entry == null) throw new NullPointerException("Dachs entry mus not be null");
					//this.listModel.setElementAt(entry.getText(), index);
					if (index >= this.listModel.getSize()) {
						this.listModel.addElement(entry);
					} else {
						this.listModel.setElementAt(entry, index);
					}
					index++;
				}
				for (int toRemove = this.listModel.getSize()- 1; toRemove >= index; toRemove--) {
					this.listModel.remove(toRemove);
				}
			} catch (TineLinkException e) {
				Logger.Netzwerk.warn("Zugaenge der Tuer (" + this.device.getDeviceName() + ") : " + e.getMessage());
				this.listModel.clear();
				DachsEntry errorItem = new DachsEntry(e.getMessage());
				this.listModel.addElement(errorItem);
			}
			try {
				//this.listTitle.setText("Aktiver Beschleuniger: " + this.device.getNameOfAccelerator());
				this.listTitle.setText(": " + this.device.getNameOfAccelerator());
			} catch (TineLinkException e) {
				Logger.Netzwerk.warn("Tuername: " + e.getMessage());
			}
		}
		
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.pintlk.badgerviewer.network.IServersClient#setConnectionParameter(de.desy.mst.libs.pintlk.badgerviewer.network.TineParameter)
	 */
	public void setDevice(IDevice device) {
		this.device = device;
		if (device == null) {
			this.timer.stop();
		} else {
			this.timer.start();
		}
		
	}
	/**
	 * @return Komponente, in der der Name des Beschleunigers angezeigt wird, auf den sich die DACHS Eintraege beziehen  
	 */
	public Component getTitleDisplay() {
		return this.listTitle;
	}
}
