/**
 * 
 */
package de.desy.mst.libs.ser.launcher;

import java.util.StringTokenizer;

import de.desy.mst.libs.ser.launcher.cache.CacheFactory;
import de.desy.mst.libs.ser.launcher.cache.ICacheItem;
import de.desy.mst.libs.ser.launcher.cache.IStartable;
import de.desy.mst.libs.ser.launcher.cache.InvalidProcess;

/**
 * @author labudda
 *
 */
class NativeStartable implements IStartable {
	private final static java.util.logging.Logger logger = java.util.logging.Logger.getLogger(NativeStartable.class.getName());
	private final String launchName;
	private final ICacheItem cachedItem;
	/**
	 * @param launchName
	 * @param firstItem
	 */
	NativeStartable(String launchName, CacheFactory firstItem) {
		this.cachedItem = firstItem;
		this.launchName = launchName;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.ser.launcher.cache.IStartable#start()
	 */
	@Override
	public Process start() {
		
		if (EOptions.IgnoreLaunch.isDefined()) {
			NativeStartable.logger.info("Ignore launch of: " + launchName);
			return InvalidProcess.Null;
		} else {
			NativeStartable.logger.info("Will launch '" + launchName + "'");
			try {
				if (EOptions.UseOpen.isDefined()) {
					final java.io.File launchable = new java.io.File(launchName);
					// starts anything, based on extionsion
					java.awt.Desktop.getDesktop().open(launchable);
					return InvalidProcess.Null;
				} else {
					/* Uber den Aufruf als Kommand Array wird verhindert, 
					dass die exec Methode nach Argumenten sucht 
					und das Kommando aufbricht in Kommando und Argumente
					Dies fuehrt ansonsten immer dann zu einem Fehler, 
					wenn der Pfad zu einem Kommando Leerzeichen enthaelt
					*/ 
					final String[] commandArgs;
					if (EOptions.MainArgument.isDefined()) {
				        StringTokenizer st = new StringTokenizer(EOptions.MainArgument.getValue(""));
				        String[] argArray = new String[st.countTokens()];
				        for (int i = 0; st.hasMoreTokens(); i++) {
				            argArray[i] = st.nextToken();
				        }
						commandArgs = new String[argArray.length + 1];
						commandArgs[0] = launchName;
						for (int arg = 0; arg< argArray.length; arg++) {
							commandArgs[arg + 1] = argArray[arg];
							NativeStartable.logger.info("Detected  argument '" + commandArgs[arg + 1] + "'");
						}
					} else {
						/* Uberden Aufruf als Kommand Array wird verhindert, 
						dass die exec Methode nach Argumenten sucht 
						und das Kommando aufbricht in Kommando und Argumente
						Dies fuehrt ansonsten immer dann zu einem Fehler, 
						wenn der Pfad zu einem Kommando Leerzeichen enthaelt
						*/ 
						commandArgs = new String[]{launchName};
						// final String commandArgs = launchName;
					}
					final java.io.File workingDirectory;
					if (this.cachedItem == null) {
						workingDirectory = null;
					} else {
						workingDirectory = new java.io.File(this.cachedItem.getCachedItemFolder());
					}
					return Runtime.getRuntime().exec(
						commandArgs, 
						null, 
						workingDirectory
					);
				}
			} catch (java.io.IOException e) {
				NativeStartable.logger.warning(e.toString());
				return InvalidProcess.Null;
			}
		}
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.ser.launcher.cache.IStartable#getLocalPath()
	 */
	@Override
	public String getLocalPath() {
		return this.cachedItem.getCachedItemPath();	}

}
