/**
 * 
 * Last changed: $Date$
 * By: $Author$
 * URL: $HeadURL$
 * $Revision$ 
 *
 */
package de.desy.mst.libs.ser.launcher.cache;

import java.util.StringTokenizer;

import de.desy.mst.libs.ser.launcher.EOptions;

/**
 * @author labudda
 *
 */
class Startable implements IStartable {
	private final static java.util.logging.Logger logger = java.util.logging.Logger.getLogger(Startable.class.getName());
	private final ICacheItem cachedItem;
	

	/**
	 * @param cachedItem
	 */
	public Startable(final ICacheItem cachedItem) {
		this.cachedItem = cachedItem;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.ser.launcher.cache.IStartable#start()
	 */
	public java.lang.Process start() {
		final String launchName = this.cachedItem.getCachedItemPath();
		if (EOptions.IgnoreLaunch.isDefined()) {
			Startable.logger.info("Ignore launch of: " + launchName);
			return InvalidProcess.Null;
		} else {
			Startable.logger.info("Launch: " + launchName);
			try {
				if (EOptions.UseOpen.isDefined()) {
					final java.io.File launchable = new java.io.File(launchName);
					// starts anything, based on extionsion
					java.awt.Desktop.getDesktop().open(launchable);
					return InvalidProcess.Null;
				} else {
					/* Uber den Aufruf als Kommand Array wird verhindert, 
					dass die exec Methode nach Argumenten sucht 
					und das Kommando aufbricht in Kommando und Argumente
					Dies fuehrt ansonsten immer dann zu einem Fehler, 
					wenn der Pfad zu einem Kommando Leerzeichen enthaelt
					*/ 
					final String[] commandArgs;
					if (EOptions.MainArgument.isDefined()) {
				        StringTokenizer st = new StringTokenizer(EOptions.MainArgument.getValue(""));
				        String[] argArray = new String[st.countTokens()];
				        for (int i = 0; st.hasMoreTokens(); i++) {
				            argArray[i] = st.nextToken();
				        }
						commandArgs = new String[argArray.length + 1];
						commandArgs[0] = launchName;
						for (int arg = 0; arg< argArray.length; arg++) {
							commandArgs[arg + 1] = argArray[arg];
						}
					} else {
						/* Uberden Aufruf als Kommand Array wird verhindert, 
						dass die exec Methode nach Argumenten sucht 
						und das Kommando aufbricht in Kommando und Argumente
						Dies fuehrt ansonsten immer dann zu einem Fehler, 
						wenn der Pfad zu einem Kommando Leerzeichen enthaelt
						*/ 
						commandArgs = new String[]{launchName};
						// final String commandArgs = launchName;
					}
					return Runtime.getRuntime().exec(
						commandArgs, 
						null, 
						new java.io.File(this.cachedItem.getCachedItemFolder())
					);
				}
			} catch (java.io.IOException e) {
				Startable.logger.warning(e.toString());
				return InvalidProcess.Null;
			}
		}
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.ser.launcher.cache.IStartable#getLocalPath()
	 */
	@Override
	public String getLocalPath() {
		return this.cachedItem.getCachedItemPath();
	}
}
