/**
 * 
 */
package de.desy.mst.libs.ser.launcher.cache;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.nio.file.FileVisitResult;
import java.nio.file.Path;
import java.nio.file.attribute.BasicFileAttributes;

/**
 * Im Gegensatz zum Cache Item wird die Applikation nicht direkt als heruntergeladene Applikation gestartet.
 * Das Cache Item wird erst im Cache entpackt, um es aus dem entpackten Verzeichnis zu starten 
 * @author labudda
 *
 */
class ZipedCacheItem extends CacheItem  {
	/**
	 * @param url
	 * @throws MalformedURLException
	 */
	ZipedCacheItem(URL url) throws MalformedURLException {
		super(url);
		if (this.isAvailable()) {
			final java.util.logging.Logger log = java.util.logging.Logger.getLogger(this.getClass().getName());
			final java.nio.file.Path zipfile = java.nio.file.Paths.get(super.getCachedItemPath());
			final java.nio.file.Path unzipTo = zipfile.getParent();
			try {
				final java.nio.file.FileSystem zfs = java.nio.file.FileSystems.newFileSystem(zipfile, null);

				final java.nio.file.Path root = zfs.getRootDirectories().iterator().next();
				
				java.nio.file.Files.walkFileTree(root, new java.nio.file.SimpleFileVisitor<java.nio.file.Path>(){
					
					/* (non-Javadoc)
					 * @see java.nio.file.SimpleFileVisitor#preVisitDirectory(java.lang.Object, java.nio.file.attribute.BasicFileAttributes)
					 */
					@Override
					public FileVisitResult preVisitDirectory(Path dir, BasicFileAttributes attrs) throws IOException {
						java.nio.file.Path destination = java.nio.file.Paths.get(unzipTo.toString(), dir.toString());
						if (java.nio.file.Files.exists(destination)) {
							if (java.nio.file.Files.isDirectory(destination)) {
								// folder already available
							} else {
								log.severe(destination.toString() + ": deleteing file to create folder with same name");
								java.nio.file.Files.delete(destination);
								java.nio.file.Files.createDirectory(destination);
							}
						} else {
							java.nio.file.Files.createDirectory(destination);
						}
						return super.preVisitDirectory(dir, attrs);
					}

					/* (non-Javadoc)
					 * @see java.nio.file.SimpleFileVisitor#visitFile(java.lang.Object, java.nio.file.attribute.BasicFileAttributes)
					 */
					@Override
					public FileVisitResult visitFile(Path file, BasicFileAttributes attrs) throws IOException {
						final FileVisitResult result = super.visitFile(file, attrs); 
						if ( result == FileVisitResult.CONTINUE) {
							java.nio.file.Path destination = java.nio.file.Paths.get(unzipTo.toString(), file.toString());
							log.finer("will extract " + file.toString() + " to " + destination.toString());
							final java.nio.file.Path destFolder = destination.getParent();
							if (java.nio.file.Files.exists(destFolder)) {
								// folder already available
							} else {
								java.nio.file.Files.createDirectory(destFolder);
							}
							java.nio.file.Files.copy(file, destination, java.nio.file.StandardCopyOption.REPLACE_EXISTING);
							if (java.nio.file.Files.exists(destination)) {
								log.finest("extracted " + file.toString() + " to " + destination.toString());
							} else {
								log.severe("unable to extract " + file.toString() + " to " + destination.toString());
							}
							return FileVisitResult.CONTINUE;
						} else {
							return result;
						}
					}
					
				});
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		} else {
			// TODO: what todo on not available
		}
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.ser.launcher.cache.CacheItem#getStartable()
	 */
	@Override
	public IStartable getStartable() {
		// TODO Auto-generated method stub
		return super.getStartable();
	}

	
	
}
