/*
 * Created on Dec 11, 2007
 *
 */
package de.desy.mst.libs.ser.tclientlib;

import java.lang.reflect.InvocationTargetException;
import javax.swing.SwingUtilities;

/**
 * An abstract group listener class for GUI Applications.
 * 
 * This class is a special implementation of IReaderGroupListener. It allows
 * thread-safe callbacks in GUI applications. It overwrites the notifyUpdate()
 * method and puts the listener into the AWT queue for later invocation. Users
 * have to implement onUpdate() instead of notifyUpdate().
 * 
 * @author jwlg
 */
public abstract class AwtGroupListener implements IReaderGroupListener, Runnable {

    TineReaderGroup group = null;
    private boolean isSynchronous = false;

    /**
     * Default Constructor. By default, onUpdate() is called asynchronously.
     * 
     */
    public AwtGroupListener() {
    }

    /**
     * Constructor. Allows to call on Update synchronously or asynchronously. If
     * isSynchronous is set to true, SwingUtils.invokeAndWait is called instead
     * of SwingUtils. This means that the calling thread waits until onUpdate()
     * is finished.
     * 
     * @param isSynchronous
     *            true if synchronous.
     */
    public AwtGroupListener(boolean isSynchronous) {
        this.isSynchronous = isSynchronous;
    }

    public void notifyUpdate(TineReaderGroup group) {
        this.group = group;
        if (isSynchronous) {
            try {
                SwingUtilities.invokeAndWait(this);
            } catch (InterruptedException e) {
                //
            } catch (InvocationTargetException e) {
                //
            }
        } else {
            SwingUtilities.invokeLater(this);
        }
    }

    public void run() {
        onUpdate(group);
    }

    public abstract void onUpdate(TineReaderGroup group);

}
