/*
 * Created on Nov 18, 2008
 *
  */
package de.desy.mst.libs.ser.tclientlib.connection;

/**
 * A counter that counts up to a maximum value.
 * 
 * The counter contains a value which can be incremented
 * up to a maximum value. When the counter reaches
 * the maximum value, the counter is expired
 * and does not increment any more.
 * The expired state can be released by calling reset()
 * which sets the value to 0.
 * A counter is disabled when the maximum value is 0.
 * In disabled state, the counter does not increment
 * its value and is always expired.
 */
public class LimitedCounter {
    
    private int maxValue;
    private int value = 0;
    
    
    /**
     * Constructor.
     * @param maxValue The maximum value.
     */
    public LimitedCounter(int maxValue) {
        this.maxValue = maxValue;
    }
    
    /**
     * Sets the value to 0.
     */
    public void reset() {
        value = 0;
    }
    
    /**
     * Increments the value.
     */
    public void increment() {
        if (value < maxValue) {
            value ++;
        }
    }
    
    /**
     * Returns true if the maximum value is reached.
     */
    public boolean isExpired() {
        return (value >= maxValue);
    }

    
    /**
     * Returns the maximum value.
     */
    public int getMaxValue() {
        return maxValue;
    }


    /**
     * Returns the current value.
     */
    public int getValue() {
        return value;
    }

    /**
     * Assigns the maximum value.
     */
    public void setMaxValue(int maxValue) {
        this.maxValue = maxValue;
    }
    
    /**
     * Returns true if the counter is active.
     * 
     * The counter is active if a maximum value > 0
     * is assigned.
     * If the counter is not active, it is always expired
     * and cannot be incremented.
     */
    public boolean isActive() {
        return (maxValue > 0);
    }

}
