package de.desy.mst.libs.ser.tclientlib.writers;
import de.desy.tine.dataUtils.TDataType;
import de.desy.mst.libs.ser.tclientlib.ATineWriter;
import de.desy.mst.libs.ser.tclientlib.TineLinkException;
import de.desy.mst.libs.ser.tclientlib.TLinkAddress;

/**
 * A long writer.
 *
 * The LongWriter writes a scalar value of type long
 * to a TINE server.
 *
 */
public class LongWriter extends ATineWriter {
    
    private static final int ARRAY_SIZE = 1;
    private long[] outputBuffer = new long[ARRAY_SIZE]; // Data buffer for TINE
    
    /**
     * Constructor.
     * @param linkAddress The TINE address
     */
    public LongWriter(TLinkAddress linkAddress) {
        super(linkAddress);
    }

    
    /**
     * Constructor.
     * @param fullDevicePath The full TINE path of
     * the property. It must be in the form 
     * <code>
     * /context/server/device/property
     * </code>.
     */
    public LongWriter(String fullDevicePath) {
        super(fullDevicePath);
    }
    
    /**
     * Writes a value to the server.
     * If the write operation was not successful,
     * a TineLinkException is thrown.
     * @throws TineLinkException
     */
    public synchronized void setValue(long newValue) throws TineLinkException {
        outputBuffer[0] = newValue;
        if (writeData(createTDataType()) != 0) {
            throw new TineLinkException(getLinkAddress(), getLinkStatus(), getErrorMessage());
        }
    }
    
    private TDataType createTDataType() {
        TDataType result = null;
        result = new TDataType(outputBuffer);
        return result;
    }
    
    
}
