/*
 * (c) by DESY 2016 - Group MCS 1, Hamburg, Germany
 * 
 * For more information about the license you should have a look at the
 * following web page:
 * 
 *         http://www.desy.de/legal/license/index_eng.html
 * 
 * If no concrete license can be found at this web page or the connection is
 * broken, then the following license is relevant:
 * 
 * Copyright (C) 2016 DESY.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301  USA
 */
package de.desy.mst.libs.util.helper;

import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;

/**
 * Developer information for version annotation. This annotation can be used
 * in the version annotation.
 * 
 * @author <a href="marcus.walla(at)desy.de">Marcus Walla</a>
 * @version $Id: ADeveloper.java 73197 2016-09-15 11:59:24Z walla $
 */
@Retention(RetentionPolicy.RUNTIME)
public @interface ADeveloper 
{
	/**
     * Returns the developer name, normally with its first- and surname. 
     * @return full name of the developer
     */
	String fullName();
	
    /**
     * Returns the e-mail address of the developer. 
     * @return e-mail address
     */
	String email();

    /**
     * Returns building and room separated with a slash, example: 30/517. 
     * @return room
     */
	String room();
	
    /**
     * Phone number. 
     * @return phone number
     */
	String phone();
	
    /**
     * Returns the group. 
     * @return DESY group
     */
	String group();
}
/* end of file */