/*
 * (c) by DESY 2016 - Group MCS 1, Hamburg, Germany
 * 
 * For more information about the license you should have a look at the
 * following web page:
 * 
 *         http://www.desy.de/legal/license/index_eng.html
 * 
 * If no concrete license can be found at this web page or the connection is
 * broken, then the following license is relevant:
 * 
 * Copyright (C) 2016 DESY.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301  USA
 */
package de.desy.mst.libs.util.helper;

import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;

/**
 * Application version specification. Provides different annotation styled 
 * entries which give the user information about the application and the 
 * developers.
 * 
 * @author <a href="marcus.walla(at)desy.de">Marcus Walla</a>
 * @version $Id: AVersion.java 73197 2016-09-15 11:59:24Z walla $
 */
@Retention(RetentionPolicy.RUNTIME)
public @interface AVersion 
{
		/**
         * Returns the application name.
         * @return application name
         */
		String application();
        
		/**
         * Returns the mailing list e-mail address. By default the 
		 * developer@mcs.desy.de e-mail address is returned.
         * @return mailing list for questions and features
         */
		String mailinglist() default "developer@mcs.desy.de";
		
        /**
         * Returns the major version number. By default the number 1 is returned.
         * @return major version number
         */
		int major() default 1;
		
        /**
         * Returns the minor version number. By default the number 0 is returned.
         * @return minor version number
         */
		int minor() default 0;
		
        /**
         * Returns the revision number. By default the number 0 is returned.
         * @return revision version number
         */
		int revision() default 0;
        
        /**
         * Returns the build ID which is an increment number created during the
         * build and deployment process.
         * @return build ID
         */
        String buildId() default "0";
        
        /**
         * Returns the release date.
         * @return release date
         */
		String releaseDate() default "01.01.1970";
		
        /**
         * Developers.
         * @return returns a collection of developers. Per default one developer
         *      must be defined in the version specification.
         */
		ADeveloper[] developers();
}
/* end of file */