/*
 * (c) by DESY 2016 - Group MCS 1, Hamburg, Germany
 * 
 * For more information about the license you should have a look at the
 * following web page:
 * 
 *         http://www.desy.de/legal/license/index_eng.html
 * 
 * If no concrete license can be found at this web page or the connection is
 * broken, then the following license is relevant:
 * 
 * Copyright (C) 2016 DESY.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301  USA
 */
package de.desy.mst.libs.util.helper;

import java.lang.annotation.Annotation;

/**
 * Annotation exception. When a failure with an annotation occurs. 
 * @author <a href="marcus.walla(at)desy.de">Marcus Walla</a>
 * @version $Id: AnnotationException.java 73197 2016-09-15 11:59:24Z walla $
 */
public class AnnotationException extends RuntimeException {
	// --- CLASS FIELDS --------------------------------------------------------
	private static final long serialVersionUID = 2171976564513295743L;

	// --- INSTANCE FIELDS -----------------------------------------------------
	private final Class<?> clazz;
	private final Class<? extends Annotation> expectedClazz;
	
	// --- CONSTRUCTOR ---------------------------------------------------------
	/**
	 * @param clazz Class which causes this exception.
	 * @param expectedClazz Expected annotation class. 
	 * @param message Message.
	 */
	public AnnotationException(final Class<?> clazz, final Class<? extends Annotation> expectedClazz, final String message) {
		super(message);
		
		this.clazz = clazz;
		this.expectedClazz = expectedClazz;
	}
	/**
	 * @param clazz Class which causes this exception.
	 * @param expectedClazz Expected annotation class. 
	 * @param cause Causing exception.
	 */
	public AnnotationException(final Class<?> clazz, final Class<? extends Annotation> expectedClazz,  final Throwable cause) {
		super(cause);
		
		this.clazz = clazz;
		this.expectedClazz = expectedClazz;
	}
	/**
	 * @param clazz Class which causes this exception.
	 * @param expectedClazz Expected annotation class. 
	 * @param message Message.
	 * @param cause Causing exception.
	 */
	public AnnotationException(final Class<?> clazz, final Class<?  extends Annotation> expectedClazz,  final String message, final Throwable cause) {
		super(message, cause);
		
		this.clazz = clazz;
		this.expectedClazz = expectedClazz;
	}
	
	// --- INSTANCE METHODS ----------------------------------------------------
	public final Class<?> getCausingClass() { return this.clazz; }
	public final Class<? extends Annotation> getExpectedClass() { return this.expectedClazz; }
	@Override public String toString() { return "unexpected annotation parameter: " + getCausingClass().getSimpleName() +", expected was " + getExpectedClass().getSimpleName(); }
}
/* end of file */