/*
 * (c) by DESY 2016 - Group MCS 1, Hamburg, Germany
 * 
 * For more information about the license you should have a look at the
 * following web page:
 * 
 *         http://www.desy.de/legal/license/index_eng.html
 * 
 * If no concrete license can be found at this web page or the connection is
 * broken, then the following license is relevant:
 * 
 * Copyright (C) 2016 DESY.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301  USA
 */
package de.desy.mst.libs.util.helper.ui.version;

import de.desy.mst.libs.util.helper.Version;
import java.awt.Frame;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JDialog;
import javax.swing.WindowConstants;

/**
 * UI for presenting the used versions and their developers.  
 * 
 * @author <a href="mailto:marcus.walla(at)desy.de">Marcus Walla</a>
 * @version $Id: UIVersion.java 73822 2016-11-15 20:05:55Z walla $
 */
public final class UIVersion 
{
	public static final String MENU_ITEM_INFO = "Information about the application";

	/**
	 * Creates the version dialog box responsible for displaying the versions
	 * (also including the TINE version) for the users in a dialog box. One
	 * can quit the box via pressing the OK button or by closing the box.
     * <p>
     * <b>Note:</b> be aware that calling this method including the parent frame
     * you must have already initialized it, especially the frame icon must
     * already be set otherwise this method cannot confirm on this image and
     * it will be empty.
	 * 
	 * @param parent Parent frame. Can also be {@code null}.
     * @param tineVersion TINE version number. 
	 * @param version Version of the current application.
	 * @param versions Versions of the provided libraries.
	 * @return new create version dialog box.
	 */
	public static JDialog createVersionDialog(final Frame parent, final String tineVersion, final Version version, final Version... versions) 
	{
		final JDialog dialog = new JDialog(parent) {
			private static final long serialVersionUID = 1L;

			@Override public void setVisible(final boolean isVisiable) {
				if (isVisiable && parent != null)
					super.setLocationRelativeTo(parent);
				super.setVisible(isVisiable);
			}
		};
		
		final Icon icon;
		if (parent != null && parent.getIconImage() != null) {
			icon = new ImageIcon(parent.getIconImage());
			dialog.setIconImage(parent.getIconImage());
		}
		else
			icon = null;

        dialog.setContentPane(new VersionPanel(dialog, icon, tineVersion, version, versions));
		dialog.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		dialog.addWindowListener(new WindowAdapter() {
			@Override public void windowClosing(final WindowEvent event) {
				dialog.setVisible(false);
			}
		});
		dialog.setResizable(true);
		dialog.setTitle(MENU_ITEM_INFO + " " + version.application());
		dialog.pack();
		
		return dialog;
	}
}
/* end of file */