/*
 * (c) by DESY 2016 - Group MCS 1, Hamburg, Germany
 * 
 * For more information about the license you should have a look at the
 * following web page:
 * 
 *         http://www.desy.de/legal/license/index_eng.html
 * 
 * If no concrete license can be found at this web page or the connection is
 * broken, then the following license is relevant:
 * 
 * Copyright (C) 2016 DESY.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301  USA
 */
package de.desy.mst.libs.util.helper.ui.version;

import de.desy.mst.libs.util.helper.Version;
import java.lang.management.ManagementFactory;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * Version system information panel.
 * 
 * @author <a href="marcus.walla(at)desy.de">Marcus Walla</a>
 * @version $Id: VersionSystemPanel.java 73822 2016-11-15 20:05:55Z walla $
 */
public class VersionSystemPanel extends javax.swing.JPanel
{
    // --- CLASS INSTANCES -----------------------------------------------------
    private static final String SYSTEM_PROPERTY_CLASS_LIBS = "java.library.path";
    
    // --- CONSTRUCTOR ---------------------------------------------------------
    /**
     * Creates new form VersionSystemPanel
     * @param applicationVersion version specification
     */
    public VersionSystemPanel(final Version applicationVersion)
    {
        initComponents();
        createFilling(applicationVersion);
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents()
    {

        versionDesyIconPnl = new de.desy.mst.libs.util.helper.ui.version.VersionDesyIconPanel();
        lblOperationSystemHeader = new javax.swing.JLabel();
        lblOperationSystem = new javax.swing.JLabel();
        lblJavaVersionHeader = new javax.swing.JLabel();
        lblJavaVersion1 = new javax.swing.JLabel();
        lblJavaDirectoryHeader = new javax.swing.JLabel();
        lblJavaDirectory = new javax.swing.JLabel();
        lblJavaClasspathHeader = new javax.swing.JLabel();
        jScrollPane1 = new javax.swing.JScrollPane();
        lblJavaClasspath = new javax.swing.JTextPane();
        lblJavaVersion2 = new javax.swing.JLabel();

        lblOperationSystemHeader.setText("Operation System");

        lblOperationSystem.setText("OS");

        lblJavaVersionHeader.setText("Java Version");

        lblJavaVersion1.setText("VERSION");

        lblJavaDirectoryHeader.setText("Java Directory");

        lblJavaDirectory.setBackground(new java.awt.Color(255, 255, 255));
        lblJavaDirectory.setText("DIRECTORY");
        lblJavaDirectory.setBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)), javax.swing.BorderFactory.createLineBorder(new java.awt.Color(255, 255, 255), 2)));
        lblJavaDirectory.setOpaque(true);

        lblJavaClasspathHeader.setText("Java Classpath");

        lblJavaClasspath.setEditable(false);
        lblJavaClasspath.setText("CLASSPATH");
        jScrollPane1.setViewportView(lblJavaClasspath);

        lblJavaVersion2.setText("VERSION");

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(versionDesyIconPnl, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                    .addComponent(lblJavaDirectoryHeader, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(lblJavaVersionHeader, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(lblOperationSystemHeader, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 100, Short.MAX_VALUE)
                    .addComponent(lblJavaClasspathHeader, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(lblJavaVersion1, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(lblOperationSystem, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 191, Short.MAX_VALUE)
                    .addComponent(lblJavaDirectory, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(lblJavaVersion2, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(lblOperationSystemHeader)
                            .addComponent(lblOperationSystem))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(lblJavaVersionHeader)
                            .addComponent(lblJavaVersion1))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(lblJavaVersion2)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(lblJavaDirectoryHeader)
                            .addComponent(lblJavaDirectory, javax.swing.GroupLayout.Alignment.TRAILING))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(lblJavaClasspathHeader)
                            .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 114, javax.swing.GroupLayout.PREFERRED_SIZE)))
                    .addComponent(versionDesyIconPnl, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents

    private void createFilling(final Version applicationVersion)
    {
        lblOperationSystem.setText(System.getProperty("os.name") + " " + System.getProperty("os.version") + " (" + System.getProperty("os.arch") + ")");
        lblJavaVersion1.setText(ManagementFactory.getRuntimeMXBean().getSpecVersion());
        lblJavaVersion2.setText(Runtime.class.getPackage().getImplementationVersion() + " (VM: " + ManagementFactory.getRuntimeMXBean().getVmVersion() + ") by " + ManagementFactory.getRuntimeMXBean().getSpecVendor());
        lblJavaDirectory.setText(System.getProperty("java.home"));
        final StringBuilder builder = new StringBuilder();
        final List<String> classpath = Arrays.asList(System.getProperty(SYSTEM_PROPERTY_CLASS_LIBS).split(System.getProperty("path.separator")));
        Collections.sort(classpath);
        classpath.stream().forEach((path) -> {
            builder.append(path).append('\n');
        });
        lblJavaClasspath.setText(builder.toString());
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JTextPane lblJavaClasspath;
    private javax.swing.JLabel lblJavaClasspathHeader;
    private javax.swing.JLabel lblJavaDirectory;
    private javax.swing.JLabel lblJavaDirectoryHeader;
    private javax.swing.JLabel lblJavaVersion1;
    private javax.swing.JLabel lblJavaVersion2;
    private javax.swing.JLabel lblJavaVersionHeader;
    private javax.swing.JLabel lblOperationSystem;
    private javax.swing.JLabel lblOperationSystemHeader;
    private de.desy.mst.libs.util.helper.ui.version.VersionDesyIconPanel versionDesyIconPnl;
    // End of variables declaration//GEN-END:variables
}
/* end of file */