/*
 * jdoocs
 * java interface for DOOCS, TINE, TANGO and EPICS control system
 * Copyright (C) 2011, MCS4 DESY
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

typedef  unsigned char u_char;   

const	DATA_NULL	= 0;
const	DATA_INT	= 1;
const	DATA_FLOAT	= 2;
const	DATA_STRING	= 3;
const	DATA_BOOL	= 4;
const	DATA_STRING16	= 5;
const	DATA_DOUBLE     = 6;
const	DATA_TEXT       = 7;
const	DATA_SHORT      = 8;

const	DATA_YT		= 10;
const	DATA_XYT	= 11;
const	DATA_TDS	= 12;
const	DATA_XY		= 13;
const	DATA_IIII	= 14;
const	DATA_IFFF	= 15;
const	DATA_USTR	= 16;
const	DATA_FIS	= 17;
const	DATA_TTII	= 18;
const	DATA_SPECTRUM	= 19;
const	DATA_XML	= 20;
const	DATA_XYZS	= 21;
const	DATA_IMAGE      = 22; /*not yet implemented */
const   DATA_THUMBNAIL  = 23;
const	DATA_GSPECTRUM  = 24;

const	DATA_A_FLOAT	= 100;
const	DATA_A_TDS	= 101;
const	DATA_A_XY	= 102;
const	DATA_A_USTR	= 103;
const	DATA_A_FIS	= 104; //not in C-API
const	DATA_A_INT	= 105;
const	DATA_A_BYTE	= 106;
const	DATA_A_TDSi	= 107; //no mor in C-API
const	DATA_A_XYZS	= 108;
const	DATA_MDA_FLOAT  = 109;
const	DATA_A_DOUBLE   = 110;
const	DATA_A_BOOL     = 111;
const	DATA_A_STRING   = 112;
const	DATA_A_SHORT    = 113;
const	DATA_A_LONG     = 114;

const   DATA_A_THUMBNAIL = 120;

const	DATA_A_NAME16FI = 201;
const	DATA_A_NAME16II = 202;
const	DATA_A_NAME16   = 203;  //not complete integrated!!
const   DATA_A_FI       = 204;
const   DATA_A_II       = 206;
const	DATA_A_NAME64   = 207;
const	DATA_A_NAME32   = 208;

const	DATA_A_TS_BOOL      = 1000;
const	DATA_A_TS_INT       = 1001;
const	DATA_A_TS_FLOAT     = 1002;
const	DATA_A_TS_DOUBLE    = 1003;
const	DATA_A_TS_STRING    = 1005;
const	DATA_A_TS_USTR	    = 1006;
const	DATA_A_TS_XML       = 1007;
const	DATA_A_TS_XY        = 1008;
const	DATA_A_TS_IIII      = 1009;
const	DATA_A_TS_IFFF      = 1010;
const	DATA_A_TS_FIS       = 1011;
const	DATA_A_TS_TTII      = 1012;
const	DATA_A_TS_SPECTRUM  = 1013;
const	DATA_A_TS_XYZS      = 1014;
const   DATA_A_TS_GSPECTRUM = 1015;

const   DATA_KEYVAL         = 1016;

const	FACILITY_STRING_LENGTH	= 64;
const	DEVICE_STRING_LENGTH	= 64;
const	LOCATION_STRING_LENGTH	= 64;
const	PROPERTY_STRING_LENGTH	= 64;
/*C++ definitions:
//#define	FACILITY_STRING_LENGTH  64
//#define	DEVICE_STRING_LENGTH    64
//#define	LOCATION_STRING_LENGTH  64
//#define	PROPERTY_STRING_LENGTH  64
//

#define HOST_STRING_LENGTH     256
#define AUTH_STRING_LENGTH     128
#define SERVER_STRING_LENGTH   128
#define FILE_STRING_LENGTH     256
*/

const	SHORT_STRING_LENGTH	= 16;
const	STRING_LENGTH		= 80;
const   TEXT_LENGTH             = 1048576;      /* 1Mb length */
const	RB_LENGTH		= 2000;
const	TDS_LENGTH		= 2000;
const	BOOL_LENGTH		= 8192;
const	FLOAT_LENGTH		= 262144;
const	DOUBLE_LENGTH		= 131072;
const	INT_LENGTH		= 262144;
const	LONG_LENGTH		= 262144;
const	IMAGE_LENGTH		= 33554432;	/* 32Mb length */
const	BYTE_ARRAY_LENGTH	= 33554432;	/* 8Mb length */
const	XML_ARRAY_LENGTH	= 4194304;	/* 4Mb length, 64k is max TCP length */
const	XY_LENGTH		= 8192;
const	SPECTRUM_LENGTH		= 8192;
const	SPECTRUM_LENGTH_MAX	= 1048576;      /* 4Mb length */
const	FIS_LENGTH		= 128;
const	FI_LENGTH		= 1000;
const	II_LENGTH		= 1000;
const	USTR_LENGTH		= 4000;
const	STRING_ARRAY_LENGTH	= 128;
const	XYZS_LENGTH		= 1000;
const	THUMBNAIL_LENGTH	= 1000;
const	THUMBNAIL_LENGTH_MAX	= 10000;
const	MDA_LENGTH              = 16384;
const	GSPECTRUM_LENGTH	= 4194304;
const	KEYVAL_LENGTH           = 1000;
const	KEYVAL_STRING_LENGTH	= 1000;
const	NAME16_LENGTH		= 1000; // for NAME16FI and NAME16II

const   TS_BOOL_LENGTH          = 8192;
const   TS_INT_LENGTH           = 8192;
const   TS_FLOAT_LENGTH         = 8192;
const   TS_DOUBLE_LENGTH        = 8192;
const   TS_STRING_LENGTH        = 8192;
const	TS_USTR_LENGTH	        = 1000;
const   TS_XML_LENGTH           = 128;
const   TS_XY_LENGTH            = 8192;
const   TS_IIII_LENGTH          = 8192;
const   TS_IFFF_LENGTH          = 8192;
const   TS_FIS_LENGTH           = 8192;
const   TS_TTII_LENGTH          = 8192;
const   TS_SPECTRUM_LENGTH      = 128;
const   TS_XYZS_LENGTH          = 8192;
const   TS_GSPECTRUM_LENGTH     = 64;


typedef long time_t;

struct TDS {
	time_t	tm;
	float	data;
	u_char	status;
};

struct TDSi {
	time_t	tm;
	float	data;
	u_short	status;
	u_short	t_ms;
};

struct XY {
	float	x_data;
	float	y_data;
};

struct SPECTRUM {
	char	comment<STRING_LENGTH>;	
	time_t	tm;
	float	s_start;
	float	s_inc;
	int	status;
	float	d_spect_array<SPECTRUM_LENGTH>;
};


struct A_BYTE {
	int	x_dim;
	int	y_dim;
	int	x_offset;
	int	y_offset;
	int	option;
	u_char	d_byte_array<BYTE_ARRAY_LENGTH>;
};

struct XML {
	char	d_xml_array<XML_ARRAY_LENGTH>;
};

struct IIII {
	int	i1_data;
	int	i2_data;
	int	i3_data;
	int	i4_data;
};

struct IFFF {	
	int	i1_data;
	float	f1_data;
	float	f2_data;
	float	f3_data;
};

struct SYT  { float d_y;            time_t tm; };
struct SXYT { float d_x; float d_y; time_t tm; };

union UstrUnion switch (int data_sel) {
		case DATA_YT	: struct SYT  d_syt;
		case DATA_XYT	: struct SXYT d_xyt;
		default	: void;
};

struct	USTR {
	int	i1_data;
	float	f1_data;
	float	f2_data;
	time_t	tm;
	char	str_data<STRING_LENGTH>;
};

struct	NAME16FI {
	char	str_data<16>;
	float	f_data;
	int	i_data;
};

struct	NAME16II {
	char	str_data<16>;
	int	i1_data;
	int	i2_data;
};

struct FIS {
	float	f_data;
	int	i_data;
	u_char	status;
};

struct FI {
	float	f_data;
	int	i_data;
};

struct II {
	int	i1_data;
	int	i2_data;
};

struct TTII {
	time_t	tm1;
	time_t	tm2;
	int	i1_data;
	int	i2_data;
};

struct XYZS {
	int	status;
	float	x;
	float	y;
	float	z;
	char	loc<LOCATION_STRING_LENGTH>;
};

struct MDAF {
	char	comment<STRING_LENGTH>;	
        int     dimn;
	int	dims [DIM_MAX];
	float	d_mdfa<MDA_LENGTH>;
};

struct IMH {
	int     width;
	int     height;
	int     aoi_width;
	int     aoi_height;
	int     x_start;
	int     y_start;
	int     bpp;
	int     ebitpp;
	int     hbin;
	int     vbin;
	int     source_format;
	int     image_format;
	u_int   frame;
	u_int   event;
        float   scale_x;
        float   scale_y;
        float   image_rotation;
        float   fspare2;
        float   fspare3;
        float   fspare4;
        int     image_flags;
        int     ispare2;
        int     ispare3;
        int     ispare4;
        int     length;
};

struct IMAGE {
        IMH             hdr;
	time_t          sec;
	time_t          usec;
	int             status;
	char	        comment<STRING_LENGTH>;
	u_char	        val<IMAGE_LENGTH>;
	ByteBuffer 	bbuf;
};

struct THUMBNAIL {
        double average;
        double deviation;
        u_int count;
        u_int time_sec_0;
        u_int time_sec_min;
        u_int time_sec_max;
        double data_min;
        double data_max;
};

struct GSPECTRUM {
	char	comment<STRING_LENGTH>;
        int     id;
	time_t	tm;
	float	s_start;
	float	s_inc;
	u_int	status;
        int     groups;
        int     group_inc;
        int     stat_mask;
        float   statistics[24];
        int     timing_params[8];
	float	d_gspect_array<GSPECTRUM_LENGTH>;
};

struct TSH {
	u_int           tsec;
	u_short         tmsec;
	u_short         status;
};



struct TS_BOOL {
        struct TSH       hdr;
	bool             val;
};



struct TS_INT {
        struct TSH       hdr;
	int              val;
};



struct TS_FLOAT {
        struct TSH       hdr;
	float            val;
};



struct TS_DOUBLE {
        struct TSH       hdr;
	double           val;
};



struct TS_STRING {
        struct TSH       hdr;
	char             val <STRING_LENGTH>;
};



struct TS_USTR {
        struct TSH       hdr;
	struct USTR      val;
};



struct TS_XML {
        struct TSH       hdr;
	struct XML       val;
};



struct TS_XY {
        struct TSH       hdr;
	struct XY        val;
};



struct TS_IIII {
        struct TSH       hdr;
	struct IIII      val;
};



struct TS_IFFF {
        struct TSH       hdr;
	struct IFFF      val;
};



struct TS_FIS {
        struct TSH       hdr;
	struct FIS       val;
};



struct TS_TTII {
        struct TSH       hdr;
	struct TTII      val;
};



struct TS_SPECTRUM {
        struct TSH       hdr;
	struct SPECTRUM  val;
};



struct TS_XYZS {
        struct TSH       hdr;
	struct XYZS      val;
};


struct TS_GSPECTRUM {
        struct TSH       hdr;
	struct GSPECTRUM val;
};

struct KEYVAL {
        struct {
            u_int       val_len;
            char*   val_val; 
        }
};

union DataUnion switch (int data_sel) {
	case DATA_INT		: int		d_int;
	case DATA_SHORT		: short		d_short;
	case DATA_BOOL		: int		d_bool;
	case DATA_FLOAT		: float		d_float;
	case DATA_DOUBLE	: double	d_double;
	case DATA_STRING	: char		d_char<STRING_LENGTH>;
	case DATA_STRING16	: char		d_char16[SHORT_STRING_LENGTH];
	case DATA_TDS		: TDS		d_tds;
	case DATA_XY		: XY		d_xy;
	case DATA_IIII		: IIII		d_iiii;
	case DATA_IFFF		: IFFF		d_ifff;
	case DATA_FIS		: FIS		d_fis;
	case DATA_TTII		: TTII		d_ttii;
        case DATA_A_NAME16FI    : NAME16FI      d_name16fi<NAME16_LENGTH>;
        case DATA_A_NAME16II    : NAME16II      d_name16ii<NAME16_LENGTH>;
        case DATA_A_NAME16      : NAME16        d_name16<NAME16_LENGTH>;
        case DATA_A_NAME32      : NAME32        d_name32<NAME16_LENGTH>;
        case DATA_A_NAME64      : NAME64        d_name64<NAME16_LENGTH>;
	case DATA_SPECTRUM	: SPECTRUM	d_spectrum;
	case DATA_XML		: XML		d_xml;
	case DATA_TEXT		: TEXT		d_text;
	case DATA_XYZS		: XYZS		d_xyzs;
        case DATA_IMAGE		: IMAGE		d_image;
	case DATA_THUMBNAIL	: THUMBNAIL	d_thumbnail;
        case DATA_GPECTRUM       : GSPECTRUM    d_gspectrum;

	case DATA_MDA_FLOAT	: MDAF   	d_mdaf;
	case DATA_A_INT		: int		d_int_array<INT_LENGTH>;
	case DATA_A_SHORT	: short		d_short_array<INT_LENGTH>;
	case DATA_A_LONG	: long		d_long_array<INT_LENGTH>;
	case DATA_A_FLOAT	: float		d_float_array<FLOAT_LENGTH>;
	case DATA_A_DOUBLE	: double	d_double_array<FLOAT_LENGTH>;
	case DATA_A_BYTE	: A_BYTE	d_byte_struct;
	case DATA_A_TDS		: TDS		d_tds_array<TDS_LENGTH>;
	case DATA_A_TDSi	: TDSi		d_tdsi_array<TDS_LENGTH>;
	case DATA_A_XY		: XY		d_xy_array<XY_LENGTH>;
	case DATA_A_USTR	: USTR		d_ustr_array<USTR_LENGTH>;
	case DATA_A_FIS		: FIS		d_fis_array<FIS_LENGTH>;
	case DATA_A_FI		: FI		d_fi_array<FI_LENGTH>;
	case DATA_A_II		: II		d_ii_array<II_LENGTH>;
	case DATA_A_XYZS	: XYZS	        d_xyzs_array  <XYZS_LENGTH>;
	case DATA_A_THUMBNAIL	: THUMBNAIL	d_thumbnail_array   <THUMBNAIL_LENGTH_MAX>;

	case DATA_A_TS_BOOL	 : TS_BOOL      d_ts_bool_array      <TS_BOOL_LENGTH>;
	case DATA_A_TS_INT	 : TS_INT	d_ts_int_array       <TS_INT_LENGTH>;
	case DATA_A_TS_FLOAT	 : TS_FLOAT     d_ts_float_array     <TS_FLOAT_LENGTH>;
	case DATA_A_TS_DOUBLE	 : TS_DOUBLE    d_ts_double_array    <TS_DOUBLE_LENGTH>;
	case DATA_A_TS_STRING	 : TS_STRING    d_ts_char_array      <TS_STRING_LENGTH>;
	case DATA_A_TS_USTR	 : TS_USTR      d_ts_ustr_array      <TS_USTR_LENGTH>;
	case DATA_A_TS_XML	 : TS_XML	d_ts_xml_array       <TS_XML_LENGTH>;
	case DATA_A_TS_XY	 : TS_XY	d_ts_xy_array        <TS_XY_LENGTH>;
	case DATA_A_TS_IIII	 : TS_IIII	d_ts_iiii_array      <TS_IIII_LENGTH>;
	case DATA_A_TS_IFFF	 : TS_IFFF      d_ts_ifff_array      <TS_IFFF_LENGTH>;
	case DATA_A_TS_FIS	 : TS_FIS       d_ts_fis_array       <TS_FIS_LENGTH>;
	case DATA_A_TS_TTII	 : TS_TTII      d_ts_ttii_array      <TS_TTII_LENGTH>;
	case DATA_A_TS_SPECTRUM	 : TS_SPECTRUM	d_ts_spectrum_array  <TS_SPECTRUM_LENGTH>;
	case DATA_A_TS_XYZS	 : TS_XYZS      d_ts_xyzs_array      <TS_XYZS_LENGTH>;
        case DATA_A_TS_GSPECTRUM : TS_GSPECTRUM d_ts_gspectrum_array <TS_GSPECTURM_LENGTH>;
	default			: void;
};

struct EqDataBlock {
	time_t		tm;
	int		error;
	DataUnion	data_u;
};

struct EqNameString {
	char	n_facility<FACILITY_STRING_LENGTH>;
	char	n_location<LOCATION_STRING_LENGTH>;
	char	n_object<DEVICE_STRING_LENGTH>;
	char	n_property<PROPERTY_STRING_LENGTH>;
};

struct get_param {
	EqNameString*	eq_name;
	EqDataBlock*	eq_data;
};

struct set_param {
	EqNameString*	eq_name;
	EqDataBlock*	eq_data;
};

struct name_param {
	EqNameString*	eq_name;
	EqDataBlock*	eq_data;
};

struct create_param {
	EqNameString*	eq_name;
	EqDataBlock*	eq_data;
};




struct get_results {
	EqDataBlock	eq_data;
};

struct set_results {
	EqDataBlock	eq_data;
};

struct name_results {
	EqDataBlock	eq_data;
};

struct create_results {
	EqDataBlock	eq_data;
};




program EQLIBPROG {
    version EQLIBVERS {
        get_results	RPC_EQ_GET (get_param)		= 1;
        set_results	RPC_EQ_SET (set_param)		= 2;
        name_results	RPC_EQ_NAMES (name_param)	= 3;
        create_results	RPC_EQ_CREATE (create_param)	= 4;
    } = 1;
} = 0x24635321;		/* 0x20000000 + F52 [hex] + 1 */

